#include "bodymodel.h"

BodyModel::BodyModel(WorldModel *worldModel){

  this->worldModel = worldModel;
  
  initialiseEffectors();
  initialiseComponents();
  
  gyroRates = VecPosition(0, 0,0);
  
  for(int i = 0; i < HJ_NUM; ++i){
    joint.push_back(Joint());
  }

  initialiseEffectors();
  initialiseComponents();

  for(int i = 0; i < LEG_NUM; ++i){
    FRPCentre.push_back(VecPosition(0, 0, 0));
    FRPForce.push_back(VecPosition(0, 0, 0));
  }

  bodyWorldInterface = HCTMatrix(HCT_ROTATE_Z, 90.0);
  refreshTorso();

  ll1 = HCTMatrix(component[COMP_LHIP1].backTranslateMatrix);
  ll1.multiply(component[COMP_LHIP2].translateMatrix);

  ll2 = HCTMatrix(component[COMP_LHIP2].backTranslateMatrix);
  ll2.multiply(component[COMP_LTHIGH].translateMatrix);

  ll3 = HCTMatrix(component[COMP_LTHIGH].backTranslateMatrix);
  ll3.multiply(component[COMP_LSHANK].translateMatrix);

  ll4 = HCTMatrix(component[COMP_LSHANK].backTranslateMatrix);
  ll4.multiply(component[COMP_LANKLE].translateMatrix);

  ll5 = HCTMatrix(component[COMP_LANKLE].backTranslateMatrix);
  ll5.multiply(component[COMP_LFOOT].translateMatrix);


  rl1 = HCTMatrix(component[COMP_RHIP1].backTranslateMatrix);
  rl1.multiply(component[COMP_RHIP2].translateMatrix);

  rl2 = HCTMatrix(component[COMP_RHIP2].backTranslateMatrix);
  rl2.multiply(component[COMP_RTHIGH].translateMatrix);

  rl3 = HCTMatrix(component[COMP_RTHIGH].backTranslateMatrix);
  rl3.multiply(component[COMP_RSHANK].translateMatrix);

  rl4 = HCTMatrix(component[COMP_RSHANK].backTranslateMatrix);
  rl4.multiply(component[COMP_RANKLE].translateMatrix);

  rl5 = HCTMatrix(component[COMP_RANKLE].backTranslateMatrix);
  rl5.multiply(component[COMP_RFOOT].translateMatrix);

}

BodyModel::~BodyModel(){
}

void BodyModel::initialiseEffectors(){

  for(int i = 0; i < EFF_NUM; ++i){
    effector.push_back(Effector(0, 0, 0, 0, 0, 0));
  }


  effector[EFF_H1] = Effector(-120.0, 120.0, 0.15, 0, 0.01, 2.0);//NECK
  effector[EFF_H2] = Effector(-45.0, 45.0, 0.15, 0, 0.01, 2.0);//HEAD

  effector[EFF_LA1] = Effector(-120.0, 120.0, 0.15, 0, 0.01, 2.0);//SHOULDER
  effector[EFF_LA2] = Effector(-1.0, 95.0, 0.15, 0, 0.01, 2.0);//UPPERARM
  effector[EFF_LA3] = Effector(-120.0, 120.0, 0.15, 0, 0.01, 2.0);//ELBOW
  effector[EFF_LA4] = Effector(-90.0, 1.0, 0.15, 0, 0.01, 2.0);//LOWERARM

  effector[EFF_RA1] = Effector(-120.0, 120.0, 0.15, 0, 0.01, 2.0);//SHOULDER
  effector[EFF_RA2] = Effector(-95.0, 1.0, 0.15, 0, 0.01, 2.0);//UPPERARM
  effector[EFF_RA3] = Effector(-120.0, 120.0, 0.15, 0, 0.01, 2.0);//ELBOW
  effector[EFF_RA4] = Effector(-1.0, 90.0, 0.15, 0, 0.01, 2.0);//LOWERARM

  effector[EFF_LL1] = Effector(-90.0, 1.0, 0.15, 0, 0.01, 2.0);//HIP1
  effector[EFF_LL2] = Effector(-25.0, 45.0, 0.15, 0, 0.01, 2.0);//HIP2
  effector[EFF_LL3] = Effector(-25.0, 100.0, 0.15, 0, 0.01, 2.0);//THIGH
  effector[EFF_LL4] = Effector(-130.0, 1.0, 0.15, 0, 0.01, 2.0);//SHANK
  effector[EFF_LL5] = Effector(-45.0, 75.0, 0.15, 0, 0.01, 2.0);//ANKLE
  effector[EFF_LL6] = Effector(-45.0, 25.0, 0.15, 0, 0.01, 2.0);//FOOT

  effector[EFF_RL1] = Effector(-90.0, 1.0, 0.15, 0, 0.01, 2.0);//HIP1
  effector[EFF_RL2] = Effector(-45.0, 25.0, 0.15, 0, 0.01, 2.0);//HIP2
  effector[EFF_RL3] = Effector(-25.0, 100.0, 0.15, 0, 0.01, 2.0);//THIGH
  effector[EFF_RL4] = Effector(-130.0, 1.0, 0.15, 0, 0.01, 2.0);//SHANK
  effector[EFF_RL5] = Effector(-25.0, 75.0, 0.15, 0, 0.01, 2.0);//ANKLE
  effector[EFF_RL6] = Effector(-45.0, 25.0, 0.15, 0, 0.01, 2.0);//FOOT
}

void BodyModel::initialiseComponents(){

  for(int i = 0; i < COMP_NUM; ++i){
    component.push_back(Component(0, 0, VecPosition(0, 0, 0), VecPosition(0, 0, 0), VecPosition(0, 0, 0)));
  }

  int index;
  int parent;
  double mass;
  VecPosition translation;
  VecPosition anchor;
  VecPosition axis;

  //TORSO
  index = COMP_TORSO;
  parent = COMP_TORSO;
  mass = 1.2171;
  translation = VecPosition(0, 0, 0);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //NECK
  index = COMP_NECK;
  parent = COMP_TORSO;
  mass = 0.05;
  translation = VecPosition(0, 0, 0.09);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(0, 0, 1.0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //HEAD
  index = COMP_HEAD;
  parent = COMP_NECK;
  mass = 0.35;
  translation = VecPosition(0, 0, 0.065);
  anchor = VecPosition(0, 0, -0.005);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT SHOULDER
  index = COMP_LSHOULDER;
  parent = COMP_TORSO;
  mass = 0.07;
  translation = VecPosition(-0.098, 0, 0.075);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT UPPERARM
  index = COMP_LUPPERARM;
  parent = COMP_LSHOULDER;
  mass = 0.15;
  translation = VecPosition(-0.01, 0.02, 0);
  anchor = -translation;
  axis = VecPosition(0, 0, 1.0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT LOWERARM
  index = COMP_LLOWERARM;
  parent = COMP_LELBOW;
  mass = 0.2;
  translation = VecPosition(0, 0.05, 0);
  anchor = -translation;
  axis = VecPosition(0, 0, 1.0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT SHOULDER
  index = COMP_RSHOULDER;
  parent = COMP_TORSO;
  mass = 0.07;
  translation = VecPosition(0.098, 0, 0.075);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT UPPERARM
  index = COMP_RUPPERARM;
  parent = COMP_RSHOULDER;
  mass = 0.15;
  translation = VecPosition(0.01, 0.02, 0);
  anchor = -translation;
  axis = VecPosition(0, 0, 1.0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT ELBOW
  index = COMP_RELBOW;
  parent = COMP_RUPPERARM;
  mass = 0.035;
  translation = VecPosition(-0.01, 0.07, 0.009);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(0, 1.0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT LOWERARM
  index = COMP_RLOWERARM;
  parent = COMP_RELBOW;
  mass = 0.2;
  translation = VecPosition(0, 0.05, 0);
  anchor = -translation;
  axis = VecPosition(0, 0, 1.0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT HIP1
  index = COMP_LHIP1;
  parent = COMP_TORSO;
  mass = 0.09;
  translation = VecPosition(-0.055, -0.01, -0.115);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(-1.0, 0, -1.0).normalize();
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT HIP2
  index = COMP_LHIP2;
  parent = COMP_LHIP1;
  mass = 0.125;
  translation = VecPosition(0, 0, 0);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(0, 1.0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT THIGH
  index = COMP_LTHIGH;
  parent = COMP_LHIP2;
  mass = 0.275;
  translation = VecPosition(0, 0.01, -0.04);
  anchor = -translation;
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT SHANK
  index = COMP_LSHANK;
  parent = COMP_LTHIGH;
  mass = 0.225;
  translation = VecPosition(0, 0.005, -0.125);
  anchor = VecPosition(0, -0.01, 0.045);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT ANKLE
  index = COMP_LANKLE;
  parent = COMP_LSHANK;
  mass = 0.125;
  translation = VecPosition(0, -0.01, -0.055);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //LEFT FOOT
  index = COMP_LFOOT;
  parent = COMP_LANKLE;
  mass = 0.2;
  translation = VecPosition(0, 0.03, -0.035);
  anchor = VecPosition(0, -0.03, 0.035);
  axis = VecPosition(0, 1.0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT HIP1
  index = COMP_RHIP1;
  parent = COMP_TORSO;
  mass = 0.09;
  translation = VecPosition(0.055, -0.01, -0.115);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(-1.0, 0, 1.0).normalize();
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT HIP2
  index = COMP_RHIP2;
  parent = COMP_RHIP1;
  mass = 0.125;
  translation = VecPosition(0, 0, 0);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(0, 1.0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT THIGH
  index = COMP_RTHIGH;
  parent = COMP_RHIP2;
  mass = 0.275;
  translation = VecPosition(0, 0.01, -0.04);
  anchor = -translation;
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT SHANK
  index = COMP_RSHANK;
  parent = COMP_RTHIGH;
  mass = 0.225;
  translation = VecPosition(0, 0.005, -0.125);
  anchor = VecPosition(0, -0.01, 0.045);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT ANKLE
  index = COMP_RANKLE;
  parent = COMP_RSHANK;
  mass = 0.125;
  translation = VecPosition(0, -0.01, -0.055);
  anchor = VecPosition(0, 0, 0);
  axis = VecPosition(1.0, 0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);

  //RIGHT FOOT
  index = COMP_RFOOT;
  parent = COMP_RANKLE;
  mass = 0.2;
  translation = VecPosition(0, 0.03, -0.035);
  anchor = VecPosition(0, -0.03, 0.035);
  axis = VecPosition(0, 1.0, 0);
  component[index] = Component(parent, mass, translation, anchor, axis);
}

void BodyModel::refresh(){


  for(int i = 0; i < HJ_NUM; ++i){
    setCurrentAngle(i, joint[i].angle);
  }

  computeFallAngle();

  //Only needed for cout information. 
  //  refreshTorso();
  //  refreshHead();
  //  refreshLeftArm();
  //  refreshRightArm();
  //  refreshLeftLeg();
  //  refreshRightLeg();
}

void BodyModel::refreshTorso(){

  component[COMP_TORSO].transformFromParent = HCTMatrix(bodyWorldInterface);
  component[COMP_TORSO].transformFromRoot = HCTMatrix(bodyWorldInterface);
}

void BodyModel::refreshComponent(const int &index){

  component[index].transformFromParent.createIdentity();
  
  HCTMatrix translate = HCTMatrix(HCT_TRANSLATE, component[index].translation + component[index].anchor);
  //Hack: Assumes joint[index] = component[index + 1]
  HCTMatrix rotate = HCTMatrix(HCT_GENERALIZED_ROTATE, component[index].axis, joint[index - 1].angle);
  HCTMatrix backTranslate = HCTMatrix(HCT_TRANSLATE, -component[index].anchor);
  
  component[index].transformFromParent.multiply(translate);
  component[index].transformFromParent.multiply(rotate);
  component[index].transformFromParent.multiply(backTranslate);

  component[index].transformFromRoot = HCTMatrix(component[component[index].parent].transformFromRoot);
  component[index].transformFromRoot.multiply(component[index].transformFromParent);
}

void BodyModel::refreshHead(){

  refreshComponent(COMP_NECK);
  refreshComponent(COMP_HEAD);
}

void BodyModel::refreshLeftArm(){

  refreshComponent(COMP_LSHOULDER);
  refreshComponent(COMP_LUPPERARM);
  refreshComponent(COMP_LELBOW);
  refreshComponent(COMP_LLOWERARM);

}

void BodyModel::refreshRightArm(){

  refreshComponent(COMP_RSHOULDER);
  refreshComponent(COMP_RUPPERARM);
  refreshComponent(COMP_RELBOW);
  refreshComponent(COMP_RLOWERARM);
}

void BodyModel::refreshLeftLeg(){

  refreshComponent(COMP_LHIP1);
  refreshComponent(COMP_LHIP2);      
  refreshComponent(COMP_LTHIGH);    
  refreshComponent(COMP_LSHANK);     
  refreshComponent(COMP_LANKLE);    
  refreshComponent(COMP_LFOOT);     
}

void BodyModel::refreshRightLeg(){

  refreshComponent(COMP_RHIP1);
  refreshComponent(COMP_RHIP2);      
  refreshComponent(COMP_RTHIGH);    
  refreshComponent(COMP_RSHANK);     
  refreshComponent(COMP_RANKLE);    
  refreshComponent(COMP_RFOOT);     
}

double BodyModel::computeTorque(const int &effectorID){
  
  effector[effectorID].updateErrors();
  
  double torque = effector[effectorID].k1 * effector[effectorID].currentError;
  torque += effector[effectorID].k2 * effector[effectorID].cumulativeError;
  torque += effector[effectorID].k3 * (effector[effectorID].currentError - effector[effectorID].previousError);
  
  return effector[effectorID].scale * torque;
}

void BodyModel::display(){
  
  cout << "*****************Body Model******************************\n";
  
  cout << "HJ_H1: " << getJointAngle(HJ_H1) << " ";
  cout << "HJ_H2: " << getJointAngle(HJ_H2) << "\n";
  
  cout << "HJ_LA1: " << getJointAngle(HJ_LA1) << " ";
  cout << "HJ_LA2: " << getJointAngle(HJ_LA2) << " ";
  cout << "HJ_LA3: " << getJointAngle(HJ_LA3) << " ";
  cout << "HJ_LA4: " << getJointAngle(HJ_LA4) << "\n";
  
  cout << "HJ_RA1: " << getJointAngle(HJ_RA1) << " ";
  cout << "HJ_RA2: " << getJointAngle(HJ_RA2) << " ";
  cout << "HJ_RA3: " << getJointAngle(HJ_RA3) << " ";
  cout << "HJ_RA4: " << getJointAngle(HJ_RA4) << "\n";
  
  cout << "HJ_LL1: " << getJointAngle(HJ_LL1) << " ";
  cout << "HJ_LL2: " << getJointAngle(HJ_LL2) << " ";
  cout << "HJ_LL3: " << getJointAngle(HJ_LL3) << " ";
  cout << "HJ_LL4: " << getJointAngle(HJ_LL4) << " ";
  cout << "HJ_LL5: " << getJointAngle(HJ_LL5) << " ";
  cout << "HJ_LL6: " << getJointAngle(HJ_LL6) << "\n";
  
  cout << "HJ_RL1: " << getJointAngle(HJ_RL1) << " ";
  cout << "HJ_RL2: " << getJointAngle(HJ_RL2) << " ";
  cout << "HJ_RL3: " << getJointAngle(HJ_RL3) << " ";
  cout << "HJ_RL4: " << getJointAngle(HJ_RL4) << " ";
  cout << "HJ_RL5: " << getJointAngle(HJ_RL5) << " ";
  cout << "HJ_RL6: " << getJointAngle(HJ_RL6) << "\n";

  cout << "GyroRates: " << gyroRates << ").\n";
  
  cout << "FRP Left: " << FRPCentre[LEG_LEFT] << ", " << FRPForce[LEG_LEFT] << "\n";
  cout << "FRP Right: " << FRPCentre[LEG_RIGHT] << ", " << FRPForce[LEG_RIGHT] << "\n";
  
}

void BodyModel::displayDerived(){

  VecPosition CG = worldModel->l2g(VecPosition(0, 0, 0));
  cout << "CG: " << CG << "\n";
  
  /*
    VecPosition leftFootCG = worldModel->l2g(HCTMatrix((component[COMP_LFOOT].transformFromRoot)).transform(VecPosition(0, 0, 0)));
    VecPosition rightFootCG = worldModel->l2g(HCTMatrix((component[COMP_RFOOT].transformFromRoot)).transform(VecPosition(0, 0, 0)));
    
    VecPosition leftArmCG = worldModel->l2g(HCTMatrix((component[COMP_LLOWERARM].transformFromRoot)).transform(VecPosition(0, 0, 0)));
    VecPosition rightArmCG = worldModel->l2g(HCTMatrix((component[COMP_RLOWERARM].transformFromRoot)).transform(VecPosition(0, 0, 0)));
  */

  VecPosition leftFootCG = HCTMatrix((component[COMP_LFOOT].transformFromRoot)).transform(VecPosition(0, 0, 0));
  VecPosition leftAnkleCG = HCTMatrix((component[COMP_LANKLE].transformFromRoot)).transform(VecPosition(0, 0, 0));
  VecPosition leftShankCG = HCTMatrix((component[COMP_LSHANK].transformFromRoot)).transform(VecPosition(0, 0, 0));
  VecPosition leftThighCG = HCTMatrix((component[COMP_LTHIGH].transformFromRoot)).transform(VecPosition(0, 0, 0));
  VecPosition leftHip2CG = HCTMatrix((component[COMP_LHIP2].transformFromRoot)).transform(VecPosition(0, 0, 0));
  VecPosition leftHip1CG = HCTMatrix((component[COMP_LHIP1].transformFromRoot)).transform(VecPosition(0, 0, 0));

  VecPosition rightFootCG = HCTMatrix((component[COMP_RFOOT].transformFromRoot)).transform(VecPosition(0, 0, 0));

  cout << "LHIP1 CG: " << leftHip1CG << "\n";
  cout << "LHIP2 CG: " << leftHip2CG << "\n";
  cout << "LTHIGH CG: " << leftThighCG << "\n";
  cout << "LSHANK CG: " << leftShankCG << "\n";
  cout << "LANKLE CG: " << leftAnkleCG << "\n";
  cout << "LFOOT CG: " << leftFootCG << "\n";

  cout << "Right foot CG: " << rightFootCG << "\n";
  //  cout << "Right Ankle CG: " << rightAnkleCG << "\n";

  //  cout << "Left arm CG: " << leftArmCG << "\n";
  //  cout << "Right arm CG: " << rightArmCG << "\n";
}

bool BodyModel::reachOutLeg(const int &legIndex, const VecPosition &localTarget, const double &angle){

  double a1, a2, a3, a4, a5, a6;

  bool possible = legInverseKinematics(legIndex, localTarget, angle, a1, a2, a3, a4, a5, a6);
  
  if(possible){
    if(legIndex == LEG_LEFT){
      setTargetAngle(EFF_LL1, a1);
      setTargetAngle(EFF_LL2, a2);
      setTargetAngle(EFF_LL3, a3);
      setTargetAngle(EFF_LL4, a4);
      setTargetAngle(EFF_LL5, a5);
      setTargetAngle(EFF_LL6, a6);
    }
    else{
      setTargetAngle(EFF_RL1, a1);
      setTargetAngle(EFF_RL2, a2);
      setTargetAngle(EFF_RL3, a3);
      setTargetAngle(EFF_RL4, a4);
      setTargetAngle(EFF_RL5, a5);
      setTargetAngle(EFF_RL6, a6);
    }
  }
  else{
    cout << localTarget << "not reachable.\n";
  }
  
  return possible;
}

bool BodyModel::targetsReached(){

  bool valid = true;

  for(int i = 0; i < EFF_NUM; ++i){
    
    if(!(effector[i].targetReached())){
      //      cout << i << " not reached\n";
      //      cout << effector[i].targetAngle << ", " << effector[i].currentAngle << ", " << effector[i].currentError << "\n";
      valid = false;
      //      return false;
    }
  }

  return valid;
}

VecPosition BodyModel::getFootCG(const int &legIndex, const double &ang1, const double &ang2, const double &ang3, const double &ang4, const double &ang5, const double &ang6){
  
  HCTMatrix m = HCTMatrix();
  getFootTransform(legIndex, m, ang1, ang2, ang3, ang4, ang5, ang6);

  return m.transform(VecPosition(0, 0, 0));
}


void BodyModel::getFootTransform(const int &legIndex, HCTMatrix &m, const double &ang1, const double &ang2, const double &ang3, const double &ang4, const double &ang5, const double &ang6){

  m = HCTMatrix(component[COMP_TORSO].transformFromRoot);

  if(legIndex == LEG_LEFT){
   
    m.multiply(component[COMP_LHIP1].translateMatrix);
    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_LHIP1].axis, ang1));
    m.multiply(ll1);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_LHIP2].axis, ang2));
    m.multiply(ll2);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_LTHIGH].axis, ang3));
    m.multiply(ll3);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_LSHANK].axis, ang4));
    m.multiply(ll4);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_LANKLE].axis, ang5));
    m.multiply(ll5);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_LFOOT].axis, ang6));
    m.multiply(component[COMP_LFOOT].backTranslateMatrix);
  }
  else if(legIndex == LEG_RIGHT){
   
    m.multiply(component[COMP_RHIP1].translateMatrix);
    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_RHIP1].axis, ang1));
    m.multiply(rl1);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_RHIP2].axis, ang2));
    m.multiply(rl2);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_RTHIGH].axis, ang3));
    m.multiply(rl3);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_RSHANK].axis, ang4));
    m.multiply(rl4);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_RANKLE].axis, ang5));
    m.multiply(rl5);

    m.multiply(HCTMatrix(HCT_GENERALIZED_ROTATE, component[COMP_RFOOT].axis, ang6));
    m.multiply(component[COMP_RFOOT].backTranslateMatrix);
  }
  
}

VecPosition BodyModel::getFootCG(const int &legIndex){

  double a1,a2, a3, a4, a5, a6;

  if(legIndex == LEG_LEFT){
    a1 = getCurrentAngle(EFF_LL1);
    a2 = getCurrentAngle(EFF_LL2);
    a3 = getCurrentAngle(EFF_LL3);
    a4 = getCurrentAngle(EFF_LL4);
    a5 = getCurrentAngle(EFF_LL5);
    a6 = getCurrentAngle(EFF_LL6);
  }
  else{
    a1 = getCurrentAngle(EFF_RL1);
    a2 = getCurrentAngle(EFF_RL2);
    a3 = getCurrentAngle(EFF_RL3);
    a4 = getCurrentAngle(EFF_RL4);
    a5 = getCurrentAngle(EFF_RL5);
    a6 = getCurrentAngle(EFF_RL6);
  }

  HCTMatrix m = HCTMatrix();
  getFootTransform(legIndex, m, a1, a2, a3, a4, a5, a6);

  return m.transform(VecPosition(0, 0, 0));
}


void BodyModel::getFootGlobalCGAndAxes(const int &legIndex, VecPosition &CG, VecPosition &xAxis, VecPosition &yAxis, VecPosition &zAxis){
  
  double a1,a2, a3, a4, a5, a6;

  if(legIndex == LEG_LEFT){
    a1 = getCurrentAngle(EFF_LL1);
    a2 = getCurrentAngle(EFF_LL2);
    a3 = getCurrentAngle(EFF_LL3);
    a4 = getCurrentAngle(EFF_LL4);
    a5 = getCurrentAngle(EFF_LL5);
    a6 = getCurrentAngle(EFF_LL6);
  }
  else{
    a1 = getCurrentAngle(EFF_RL1);
    a2 = getCurrentAngle(EFF_RL2);
    a3 = getCurrentAngle(EFF_RL3);
    a4 = getCurrentAngle(EFF_RL4);
    a5 = getCurrentAngle(EFF_RL5);
    a6 = getCurrentAngle(EFF_RL6);
  }

  HCTMatrix m = HCTMatrix();
  getFootTransform(legIndex, m, a1, a2, a3, a4, a5, a6);

  CG = worldModel->l2g(m.transform(VecPosition(0, 0, 0)));

  //Adjustment used!

  xAxis = (worldModel->l2g(m.transform(VecPosition(0, 1.0, 0))) - CG).normalize();
  yAxis = (worldModel->l2g(m.transform(VecPosition(-1.0, 0, 0))) - CG).normalize();
  zAxis = (worldModel->l2g(m.transform(VecPosition(0, 0, 1.0))) - CG).normalize();
}


//Correct for parallelness later, if necessary.
bool BodyModel::legInverseKinematics(const int &legIndex, const VecPosition &localTarget, const double &footAngle, double &a1, double &a2, double &a3, double &a4, double &a5, double &a6){

  double xScale = 500.0;
  double yScale = 500.0;
  double zScale = 500.0;

  double tolerance = 0.001;

  int maxIterations = 100;

  a1 = (legIndex == LEG_LEFT)? -footAngle : footAngle;
  a2 = 0;
  a3 = 0;
  a4 = 0;
  a5 = 0;
  a6 = 0;
  
  VecPosition currentPos = getFootCG(legIndex, a1, a2, a3, a4, a5, a6);
  VecPosition currentDiff = VecPosition(localTarget) - currentPos;

  int numIterations;
  for(numIterations = 0; (numIterations < maxIterations) && (currentDiff.getMagnitude() >= tolerance); numIterations++){

    //X
    a3 += xScale * currentDiff.getX();
    a5 -= xScale * currentDiff.getX();
    
    //Y
    a2 += yScale * currentDiff.getY();
    a6 -= yScale * currentDiff.getY();
    
    //Z
    a3 += zScale * currentDiff.getZ() * 0.25;
    a4 -= zScale * currentDiff.getZ() * 0.5;
    a5 += zScale * currentDiff.getZ() * 0.25;

    VecPosition newPos = getFootCG(legIndex, a1, a2, a3, a4, a5, a6);
    VecPosition newDiff = VecPosition(localTarget) - newPos;

    if(newDiff.getX() * currentDiff.getX() < 0){
      xScale *= 0.5;
    }

    if(newDiff.getY() * currentDiff.getY() < 0){
      yScale *= 0.5;
    }

    if(newDiff.getZ() * currentDiff.getZ() < 0){
      zScale *= 0.5;
    }

    currentPos = newPos;
    currentDiff = newDiff;
  }

  //  cout << "IK: " << numIterations << ", " << currentDiff.getMagnitude() << "\n";
  //  cout << a1 << ", " << a2 << ", " << a3 << ", " << a4 << ", " << a5 << ", " << a6 << "\n";

  return (currentDiff.getMagnitude() < tolerance);
}

