#include <errno.h>
#include <signal.h>
#include <iostream>
#include <rcssnet/tcpsocket.hpp>
//#include <rcssnet/udpsocket.hpp>
#include <rcssnet/exception.hpp>
#include <netinet/in.h>
#include "behavior.h"
//#include <boost/scoped_ptr.hpp>
//#include "soccerbehavior.h"
//#include "soccerbotbehavior.h"
//#include "carbehavior.h"
//#include "leggedspherebehavior.h"
//#include "hoap2behavior.h"
#include "naobehavior.h"
#include "worldmodel.h"
#include <fstream>

using namespace rcss::net;
using namespace std;
//using namespace boost;

string outputFile;
map<string, float> namedParams;


TCPSocket gSocket;
//UDPSocket gSocket;
string gHost = "127.0.0.1";
int gPort = 3100;

// bool to indicate whether to continue the agent mainloop
static bool gLoop = true;

// SIGINT handler prototype
extern "C" void handler(int sig) {
    if (sig == SIGINT)
        gLoop = false;
}

void PrintGreeting() {
    cout << "agentspark, a spark demo agent\n"
         << "Copyright (C) 2004 Koblenz University.\n"
         << "2004 RoboCup Soccer Server 3D Maintenance Group.\n\n";
}

void PrintHelp() {
    cout << "\nusage: agentspark [options]" << endl;
    cout << "\noptions:" << endl;
    cout << " --help      prints this message." << endl;
    cout << " --host=IP   IP of the server." << endl;
    cout << " --team TeamName Name of Team." << endl;
    cout << " --unum UNum Uniform Number of Player." << endl;
    cout << " --experimentout filename The file to output the results of an experimental run" << endl;
    cout << " --experimentin filename The file to read named parameters from" << endl;
    cout << "\n";
}

string teamName;
string inputsFile;

/*
 * Read in parameters from inputsFile, which should be formatted 
 * with a set of parameters as key value pairs from strings to 
 * floats.  The parameter name should be separated from its value
 * with a tab and parameters should be separated from each other
 * with a single newline.  Parameters will be loaded into the
 * namedParams map.
 */
void LoadParams() {
  ifstream infile;
  infile.open(inputsFile.c_str(), ifstream::in);
  
  if(!infile) {
    cerr << "Could not open parameter file " << inputsFile << endl;
    exit(1);
  }
  
  string name;
  while(!infile.eof())
  {

    std::string key;
    std::string value;
    std::getline(infile, key, '\t');      //read thru tab
    std::getline(infile, value);          //read thru newline
    if(value.empty()) {
      continue;
    }
    namedParams[key] = strtof(value.c_str(), NULL);
  }
  infile.close();
}

int uNum;

void ReadOptions(int argc, char* argv[]) {

    teamName = "UTAustinVilla";
    uNum = 1;

    for(int i = 0; i < argc; i++) {
      if ( strcmp( argv[i], "--help" ) == 0) {
              PrintHelp();
              exit(0);
          }
      else if (strncmp( argv[i], "--host", 6 ) == 0 ) {
              string tmp=argv[i];
          
              if ( tmp.length() <= 7 ) { // minimal sanity check
                PrintHelp();
                exit(0);
              }
              gHost = tmp.substr(7);
          }
      else if (strcmp(argv[i], "--team") == 0) {
          if(i == argc - 1){
              PrintHelp();
              exit(0);
          }
          
          teamName = argv[i + 1];
      }
      else if (strcmp(argv[i], "--unum") == 0) {
          if(i == argc - 1){
              PrintHelp();
              exit(0);
          }
          uNum = atoi(argv[i + 1]);
      }
      else if (strcmp(argv[i], "--experimentout") == 0) {
          if(i == argc - 1) {
            PrintHelp();
            exit(0);
          }
          outputFile = argv[i+1];
      }
      else if(strcmp(argv[i], "--experimentin") == 0) {
          if(i == argc - 1) {
            PrintHelp();
            exit(0);
          }
          inputsFile = argv[i+1];
          LoadParams();
      }
  }
}

bool Init() {
    cout << "connecting to TCP " << gHost << ":" << gPort << "\n";
    //cout << "connecting to UDP " << gHost << ":" << gPort << "\n";

    try {
            Addr local(INADDR_ANY,INADDR_ANY);
            gSocket.bind(local);
    }

    catch (BindErr error) {
            cerr << "failed to bind socket with '"
                 << error.what() << "'" << endl;

            gSocket.close();
            return false;
        }

    try {
            Addr server(gPort,gHost);
            gSocket.connect(server);
        }

    catch (ConnectErr error) {
            cerr << "connection failed with: '"
                 << error.what() << "'" << endl;
            gSocket.close();
            return false;
        }

    return true;
}

void Done() {
    gSocket.close();
    cout << "closed connection to " << gHost << ":" << gPort << "\n";
}

bool SelectInput() {
    fd_set readfds;
    struct timeval tv = {60,0};
    FD_ZERO(&readfds);
    FD_SET(gSocket.getFD(),&readfds);
    
    while(1) {
        switch(select(gSocket.getFD()+1,&readfds, 0, 0, &tv)) {
        case 1:
            return 1;
        case 0:
            cerr << "(SelectInput) select failed " << strerror(errno) << endl;
            abort();
            return 0;
        default:
            if(errno == EINTR)
                continue;
            cerr << "(SelectInput) select failed " << strerror(errno) << endl;
            abort();
            return 0;
        }
    }
}

void PutMessage(const string& msg) {
    if (msg.empty()){
      return;
    }

    // prefix the message with it's payload length
    unsigned int len = htonl(msg.size());
    string prefix((const char*)&len,sizeof(unsigned int));
    string str = prefix + msg;
    write(gSocket.getFD(), str.data(), str.size());
}

bool GetMessage(string& msg) {
    static char buffer[16 * 1024];

    unsigned int bytesRead = 0;
    while(bytesRead < sizeof(unsigned int)) {
      SelectInput();
      int readResult = read(gSocket.getFD(), buffer + bytesRead, sizeof(unsigned int) - bytesRead);
      if(readResult < 0)
        continue;
      bytesRead += readResult;
    }

    //cerr << "buffer = |" << string(buffer+1) << "|\n";
    //cerr << "bytesRead = |" << bytesRead << "|\n";
    //cerr << "Size of buffer = |" << sizeof(buffer) << "|\n";
    //cerr << "buffer = |" << buffer << "|\n";
    //cerr << "buffer[5] = |" << buffer[5] << "|\n";
    //printf ("xxx-%s\n", buffer+5);

    // msg is prefixed with it's total length
    unsigned int msgLen = ntohl(*(unsigned int*)buffer);
    // cerr << "GM 6 / " << msgLen << " (bytesRead " << bytesRead << ")\n";
    if(sizeof(unsigned int) + msgLen > sizeof(buffer)) {
        cerr << "too long message; aborting" << endl;
        abort();
    }

    // read remaining message segments
    unsigned int msgRead = bytesRead - sizeof(unsigned int);

    //cerr << "msgRead = |" << msgRead << "|\n";

    char *offset = buffer + bytesRead;

    while (msgRead < msgLen) {
      if (!SelectInput()) {
              return false;
      }
     
      int readLen = sizeof(buffer) - msgRead;
      if(readLen > msgLen - msgRead)
          readLen = msgLen - msgRead;
     
      int readResult = read(gSocket.getFD(), offset, readLen);
      if(readResult < 0)
          continue;
      msgRead += readResult;
      offset += readResult;
      //cerr << "msgRead = |" << msgRead << "|\n";
    }

    // zero terminate received data
    (*offset) = 0;

    msg = string(buffer+sizeof(unsigned int));

    // DEBUG
    //cout << msg << endl;

    return true;
}

void Run() {    
  //scoped_ptr<Behavior> behavior(new SoccerbotBehavior(teamName, uNum));
  //scoped_ptr<Behavior> behavior(new SoccerBehavior());
  //scoped_ptr<Behavior> behavior(new CarBehavior());
  //scoped_ptr<Behavior> behavior(new LeggedSphereBehavior());
  //scoped_ptr<Behavior> behavior(new Hoap2Behavior());
  
  //scoped_ptr<Behavior> behavior(new NaoBehavior(teamName, uNum));
  Behavior *behavior = new NaoBehavior(teamName, uNum);
  
  PutMessage(behavior->Init());
  
  string msg;
  while (gLoop) {  
    GetMessage(msg);
    PutMessage(behavior->Think(msg));
  }
}

int main(int argc, char* argv[]) {
  // registering the handler, catching SIGINT signals 
  signal(SIGINT, handler);
    PrintGreeting();
  ReadOptions(argc,argv);
  
  if (!Init()) {
    return 1;
  }
  
  Run();
  Done();
}
