#include "naobehavior.h"
#include <iostream>
#include <sstream>
#include <cmath>
#include <map>
extern map<string, float> namedParams;

NaoBehavior::NaoBehavior(const std::string teamName, int uNum){


  mInit = false;
  initBeamed = false;

  agentTeamName = teamName;
  agentUNum = uNum;
  
  worldModel = new WorldModel();
  bodyModel = new BodyModel(worldModel);
  parser = new Parser(worldModel, bodyModel, teamName);

  initBeamed = false;
  initialized = false;

  fallState = 0;
  fallenLeft = false;
  fallenRight = false;
  fallenUp = false;
  fallenDown = false;

  resetSkills();
}

NaoBehavior::~NaoBehavior(){

  delete parser;
  delete worldModel;
  delete bodyModel;
}

string NaoBehavior::Init(){

  return "(scene rsg/agent/nao/nao.rsg)";
}

string NaoBehavior::Think(const std::string& message){

  //cout << "(SoccerbotBehavior) received message " << message << endl;
    
  bool parseSuccess = parser->parse(message);
  if(!parseSuccess){
    cout << "****************************************\n";
    cout << "Could not parse message: " << message << "\n";
    cout << "****************************************\n";
  }

  //  cout << "\nparseSuccess: " << parseSuccess << "\n";
  // worldModel->display();
  worldModel->quickDisplay();
  bodyModel->refresh();
  //  bodyModel->display();
  //  bodyModel->displayDerived();

  string action;

  if (!mInit){
    
    mInit = true;
    stringstream ss;
    ss << "(init (unum " << agentUNum << ")(teamname " << agentTeamName << "))";
    action = ss.str();
    return action;
  }

  if(!initialized){
    
    if(!worldModel->getUNumSet() || !worldModel->getSideSet()){
      cout << "UNum and side not received yet.\n";
      action = "";
      return action;
    }

    if(!initBeamed && worldModel->getTime() > 0.6){

      initBeamed = true;

      double beamX, beamY, beamAngle;

      //1: Goalie.
      if(worldModel->getUNum() == 1){
        beamX = -5.5;
        beamY = 0;
        beamAngle = 0;
      }
      else if(worldModel->getUNum() == 2){
	beamX = namedParams["agent2beamX"];
	beamY = namedParams["agent2beamY"];
	beamAngle = namedParams["agent2beamAngle"];
        cerr << "Beamed agent 2 to: (" << beamX << ", " << beamY << ") theta=" 
	  << beamAngle << endl;
        //beamX = -0.3;
        //beamY = 0.05;
        //beamAngle = 0;
      }
      else if(worldModel->getUNum() == 3){
        beamX = -0.5;
        beamY = 1.5;
        beamAngle = 0;
      }
      else{
        beamX = -2.0 - ((double)(worldModel->getUNum()) / 100.0);
        beamY = (drand48() - 0.5) * ((double)(worldModel->getUNum()) / 100.0);
        beamAngle = 0;
      }

      stringstream ss;
      ss << "(beam " << beamX << " " << beamY << " " << beamAngle << ")";
      action = ss.str();
      return action;
    }
    else{

      cout << "Not Initialized.\n";
      initialized = setInitialHead();
      initialized = setInitialArm(ARM_LEFT) && initialized;
      initialized = setInitialArm(ARM_RIGHT) && initialized;
      initialized = setInitialLeg(LEG_LEFT) && initialized;
      initialized = setInitialLeg(LEG_RIGHT) && initialized;
    }
  }
  else{
    
    act();
  }

  //action = "(say Hello)(he2 1)(he1 1)";
  //checkAngles();

  action = composeAction();
  
  //std::cout << "Sending action: " << action << "\n";
  return action;
}

void NaoBehavior::act(){

  refresh();

  //if(checkingFall()){
  // resetSkills();
  //  return;
  //}

  //  if(worldModel->getCycle() < 500){
  //    return;
  //  }

  //  skill = SKILL_KICK_FRONT;
  //  skill = SKILL_TURN_RIGHT;
  //  skill = SKILL_STAND;
  //  skill = SKILL_WALK_BACK;
  //  skill = SKILL_WALK_FAST;

  //  cout << "LF: " << bodyModel->getFootCG(LEG_LEFT) << "\n";
  //  cout << "RF: " << bodyModel->getFootCG(LEG_RIGHT) << "\n";
  //cout << "Ball: " << ball << "\n";

  if(skillState == 0){
      skill = selectSkill();
    }
 

  if(skill == SKILL_WALK_FRONT){
    walk_front();
  }
  else if(skill == SKILL_WALK_BACK){
    walk_back();
  }
  else if(skill == SKILL_WALK_LEFT){
    walk_left();
  }
  else if(skill == SKILL_WALK_RIGHT){
    walk_right();
  }
  else if(skill == SKILL_TURN_LEFT){
    turn_left();
  }
  else if(skill == SKILL_TURN_RIGHT){
    turn_right();
  }
  else if(skill == SKILL_KICK_FRONT){
    kick_front();
  }
  else if(skill == SKILL_STAND){
    stand();
  }
}

