#include "parser.h"

///////////////////////////
//Tak care of side both halves - constructor wrong currently.
///////////////////////////

Parser::Parser(WorldModel *worldModel, BodyModel *bodyModel, const string &teamName){

  this->worldModel = worldModel;
  this->bodyModel = bodyModel;

  //Both side and UNum not necessarily correct, but stop gap initially.
  this->side = SIDE_LEFT;
  this->uNum = 1;

  this->teamName = teamName;

  visionSense = new VisionSense(); 
}

Parser::~Parser(){

  delete visionSense;
}

vector<string> Parser::tokenise(const string &s){
  
  int length = s.length();
  string currentString = "";
  bool currentValid = false;

  vector<string> v;

  for(int i = 0; i < length; ++i){

    char c = s.at(i);
    if(c != '(' && c != ')' && c != ' '){
      currentString.append(1, c);
    }
    else{

      if(currentString.length() > 0){
	v.push_back(currentString);
	currentString = "";
      }
    }
  }

  /*
    cout << "Tokens: ";
    for(int i = 0; i < v.size(); ++i){
    cout << v[i] << " ";
    }
    cout << "\n";
  */

  return v;
}

bool Parser::parseTime(const string &str){

  bool valid = false;
  double time = 0;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size() - 1; ++i){
    if(!(tokens[i].compare("now"))){
      time = atof(tokens[i + 1].c_str());
      valid = true;
    }
  }

  if(valid){
    //    cout << "Time: " << time << "\n";
    worldModel->setTime(time);
    worldModel->incrementCycle();
  }

  return valid;
}

bool Parser::parseGameState(const string &str){

  bool gameTimeValid = false;
  bool playModeValid = false;
  bool uNumValid = false;
  bool sideValid = false;

  double gameTime = 0;
  string playModeStr = "";

  int playMode = -1;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size() - 1; ++i){

    if(!(tokens[i].compare("t"))){
      gameTime = atof(tokens[i + 1].c_str());
      gameTimeValid = true;
      ++i;
    }
    else if(!(tokens[i].compare("pm"))){
      
      playModeStr = tokens[i + 1];
      
      
      if(!(playModeStr.compare("BeforeKickOff"))){
	playMode = PM_BEFORE_KICK_OFF;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickOff_Left"))){
	playMode = PM_KICK_OFF_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickOff_Right"))){
	playMode = PM_KICK_OFF_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PlayOn"))){
	playMode = PM_PLAY_ON;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickIn_Left"))){
	playMode = PM_KICK_IN_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickIn_Right"))){
	playMode = PM_KICK_IN_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("Goal_Left"))){
	playMode = PM_GOAL_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("Goal_Right"))){
	playMode = PM_GOAL_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("GameOver"))){
	playMode = PM_GAME_OVER;
	playModeValid = true;
      }
      else{
	playModeValid = false;
      }
      ++i;
    }
    else if(!(tokens[i].compare("unum"))){

      uNum = atoi(tokens[i + 1].c_str());
      if(1 <= uNum && uNum <= 11){
	uNumValid = true;
      }
      ++i;
    }
    else if(!(tokens[i].compare("team"))){

      if(!(tokens[i + 1].compare("left"))){
	side = SIDE_LEFT;
	sideValid = true;
      }
      else if(!(tokens[i + 1].compare("right"))){
	side = SIDE_RIGHT;
	sideValid = true;
      }
      ++i;
    }

  }
  
  if(gameTimeValid){
    //    cout << "Game Time: " << gameTime << "\n";
    worldModel->setGameTime(gameTime);
  }

  if(playModeValid){
    //    cout << "Play Mode: " << playMode << "\n";
    worldModel->setPlayMode(playMode);
  }

  if(uNumValid){
    cout << "UNum: " << uNum << "\n";
    worldModel->setUNum(uNum);
    worldModel->setUNumSet(true);
  }

  if(sideValid){
    cout << "Side: " << side << "\n";
    worldModel->setSide(side);
    worldModel->setSideSet(true);
  }

  bool valid = gameTimeValid || playModeValid;

  return valid;
}

bool Parser::parseGyro(const string &str){

  bool valid = false;

  double rateX, rateY, rateZ;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){
    if(!tokens[i].compare("rt")){
      if(i + 3 < tokens.size()){
	
	rateX = atof(tokens[i + 1].c_str());
	rateY = atof(tokens[i + 2].c_str());
	rateZ = atof(tokens[i + 3].c_str());
	valid = true;
      }
    }

  }

  if(valid){
    //cout << "Gyro: " << rateX << " " << rateY << " " << rateZ << "\n";
    bodyModel->setGyroRates(rateX, rateY, rateZ);
  }

  return valid;
}

//to handle -- do when needed.
bool Parser::parseHear(const string &str){

  bool valid = false;

  double hearTime;
  bool self;
  double angle;
  string message;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 4);
  
  hearTime = atof(tokens[1].c_str());
  if(!(tokens[2].compare("self"))){
    self = true;
    angle = 0;
  }
  else{
    self = false;
    angle = atof(tokens[2].c_str());
  }
  
  message = tokens[3];

  if(valid){
    //    cout << "Hear Time: " << hearTime << "\n";
    //    cout << "Self: " << self << "\n";
    //    cout << "Angle: " << angle << "\n";
    //    cout << "Message: " << message << "\n";
  }

  return valid;
}

bool Parser::parseHingeJoint(const string &str){

  bool valid;

  string name;
  double angle;

  bool validName = false;
  bool validAngle = false;

  int hingeJointIndex = -1;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){

    if(!tokens[i].compare("n")){
      if(i + 1 < tokens.size()){
	
	name = tokens[i + 1];
	
	if(!(name.compare("hj1"))){
	  hingeJointIndex = HJ_H1;
	  validName = true;
	}
	else if(!(name.compare("hj2"))){
	  hingeJointIndex = HJ_H2;
	  validName = true;
	}
	else if(!(name.compare("laj1"))){
	  hingeJointIndex = HJ_LA1;
	  validName = true;
	}
	else if(!(name.compare("laj2"))){
	  hingeJointIndex = HJ_LA2;
	  validName = true;
	}
	else if(!(name.compare("laj3"))){
	  hingeJointIndex = HJ_LA3;
	  validName = true;
	}
	else if(!(name.compare("laj4"))){
	  hingeJointIndex = HJ_LA4;
	  validName = true;
	}
	else if(!(name.compare("raj1"))){
	  hingeJointIndex = HJ_RA1;
	  validName = true;
	}
	else if(!(name.compare("raj2"))){
	  hingeJointIndex = HJ_RA2;
	  validName = true;
	}
	else if(!(name.compare("raj3"))){
	  hingeJointIndex = HJ_RA3;
	  validName = true;
	}
	else if(!(name.compare("raj4"))){
	  hingeJointIndex = HJ_RA4;
	  validName = true;
	}
	else if(!(name.compare("llj1"))){
	  hingeJointIndex = HJ_LL1;
	  validName = true;
	}
	else if(!(name.compare("llj2"))){
	  hingeJointIndex = HJ_LL2;
	  validName = true;
	}
	else if(!(name.compare("llj3"))){
	  hingeJointIndex = HJ_LL3;
	  validName = true;
	}
	else if(!(name.compare("llj4"))){
	  hingeJointIndex = HJ_LL4;
	  validName = true;
	}
	else if(!(name.compare("llj5"))){
	  hingeJointIndex = HJ_LL5;
	  validName = true;
	}
	else if(!(name.compare("llj6"))){
	  hingeJointIndex = HJ_LL6;
	  validName = true;
	}
	else if(!(name.compare("rlj1"))){
	  hingeJointIndex = HJ_RL1;
	  validName = true;
	}
	else if(!(name.compare("rlj2"))){
	  hingeJointIndex = HJ_RL2;
	  validName = true;
	}
	else if(!(name.compare("rlj3"))){
	  hingeJointIndex = HJ_RL3;
	  validName = true;
	}
	else if(!(name.compare("rlj4"))){
	  hingeJointIndex = HJ_RL4;
	  validName = true;
	}
	else if(!(name.compare("rlj5"))){
	  hingeJointIndex = HJ_RL5;
	  validName = true;
	}
	else if(!(name.compare("rlj6"))){
	  hingeJointIndex = HJ_RL6;
	  validName = true;
	}
      }
    }

    if(!tokens[i].compare("ax")){
      if(i + 1 < tokens.size()){
	
	angle = atof(tokens[i + 1].c_str());
	validAngle = true;
      }
    }

  }

  valid = validName && validAngle;

  if(valid){
    //    cout << "HJ: " << name << " " << angle << "\n";
    bodyModel->setJointAngle(hingeJointIndex, angle);
  }

  return valid;
}

bool Parser::parseSee(const string &str){

  visionSense->teammatePolar.clear();
  visionSense->opponentPolar.clear();

  bool valid = true;

  vector<string> strSegments = segment(str, true);
  
  for(int i = 0; i < strSegments.size(); ++i){
    
    //Goalpost
    if(strSegments[i].at(1) == 'G'){
      valid = parseGoalPost(strSegments[i]) && valid;
    }
    //Flags
    else if(strSegments[i].at(1) == 'F'){
      valid = parseFlag(strSegments[i]) && valid;
    }
    //Ball
    else if(strSegments[i].at(1) == 'B'){
      valid = parseBall(strSegments[i]) && valid;
    }
    //Player
    else if(strSegments[i].at(1)== 'P'){
      valid = parsePlayer(strSegments[i]) && valid;
    }
    else{
      valid = false;
    }
    
  }
  
  return valid;
}

bool Parser::parseGoalPost(const string &str){

  bool valid = false;

  string name;
  double r, theta, phi;
  
  int goalPostIndex = -1;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 5);
  
  if(valid){
    name = tokens[0];
    valid = !(tokens[1].compare("pol")) && valid;

    r = atof(tokens[2].c_str());
    theta = atof(tokens[3].c_str());
    phi = atof(tokens[4].c_str());

    if(!(name.compare("G1L"))){
      goalPostIndex = GOALPOST_1_L;
    }
    else if(!(name.compare("G1R"))){
      goalPostIndex = GOALPOST_1_R;
    }
    else if(!(name.compare("G2L"))){
      goalPostIndex = GOALPOST_2_L;
    }
    else if(!(name.compare("G2R"))){
      goalPostIndex = GOALPOST_2_R;
    }
    else{
      valid = false;
    }
  }

  if(valid){
    //    cout << name << ": " << r << ", " << theta << ", " << phi << "\n";
    visionSense->goalPostPolar[goalPostIndex] = VecPosition(r, theta, phi);
  }

  return valid;
}

bool Parser::parseFlag(const string &str){
  bool valid = false;

  string name;
  double r, theta, phi;

  int flagIndex = -1;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 5);
  
  if(valid){
    name = tokens[0];
    valid = !(tokens[1].compare("pol")) && valid;

    r = atof(tokens[2].c_str());
    theta = atof(tokens[3].c_str());
    phi = atof(tokens[4].c_str());

    if(!(name.compare("F1L"))){
      flagIndex = FLAG_1_L;
    }
    else if(!(name.compare("F1R"))){
      flagIndex = FLAG_1_R;
    }
    else if(!(name.compare("F2L"))){
      flagIndex = FLAG_2_L;
    }
    else if(!(name.compare("F2R"))){
      flagIndex = FLAG_2_R;
    }
    else{
      valid = false;
    }
  }

  if(valid){
    //    cout << name << ": " << r << ", " << theta << ", " << phi << "\n";
    visionSense->flagPolar[flagIndex] = VecPosition(r, theta, phi);
  }

  return valid;
}

bool Parser::parseBall(const string &str){

  bool valid = false;

  string name;
  double r, theta, phi;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 5);
  
  if(valid){
    name = tokens[0];
    valid = !(tokens[1].compare("pol")) && valid;

    r = atof(tokens[2].c_str());
    theta = atof(tokens[3].c_str());
    phi = atof(tokens[4].c_str());
  }

  if(valid){
    //    cout << name << ": " << r << ", " << theta << ", " << phi << "\n";
    visionSense->ballPolar = VecPosition(r, theta, phi);
  }

  return valid;
}

bool Parser::parsePlayer(const string &str){
  //cerr << "Debug: " << str << endl;
  bool valid = false;

  string playerTeamName;
  bool playerTeamNameValid = false;

  int uNum;
  bool uNumValid = false;

  double r, theta, phi;
  bool coordinatesValid = false;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){

    if(!(tokens[i].compare("P"))){
      //Nothing
    }
    else if(!(tokens[i].compare("team")) && i < tokens.size() - 1){

      playerTeamName = tokens[i + 1];
      ++i;
      playerTeamNameValid = true;
    }
    else if(!(tokens[i].compare("id")) && i < tokens.size() - 1){
      uNum = atoi(tokens[i + 1].c_str());
      ++i;
      uNumValid = true;
    }
    else if(!(tokens[i].compare("pol")) &&  i < tokens.size() - 3){
      r = atof(tokens[i + 1].c_str());
      theta = atof(tokens[i + 2].c_str());
      phi = atof(tokens[i + 3].c_str());
      i += 3;
      coordinatesValid = true;
    }
    else{
      cout << "Unknown token " << tokens[i] << "\n";
    }

  }

  valid = playerTeamNameValid && uNumValid && coordinatesValid;

  if(valid){

    //    cout << "P, " << playerTeamName << ", " << uNum << ": " << r << ", " << theta << ", " << phi << "\n";
    if(!playerTeamName.compare(teamName)){
      visionSense->teammatePolar.push_back(VecPosition(r, theta, phi));
    }
    else{
      visionSense->opponentPolar.push_back(VecPosition(r, theta, phi));
    }
  }
  else{
    cout << "Player String" << str << "not valid.\n";
  }
  return valid;
}

bool Parser::parseFRP(const string &str){

  bool valid;

  string name;
  double centreX, centreY, centreZ;
  double forceX, forceY, forceZ;

  bool validName = false;
  bool validCentre = false;
  bool validForce = false;

  int FRPIndex = -1;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){

    if(!tokens[i].compare("n")){
      if(i + 1 < tokens.size()){
	
	name = tokens[i + 1];

	if(!(name.compare("lf"))){
	  FRPIndex = LEG_LEFT;
	  validName = true;
	}
	if(!(name.compare("rf"))){
	  FRPIndex = LEG_RIGHT;
	  validName = true;
	}
      }
    }

    if(!tokens[i].compare("c")){
      if(i + 3 < tokens.size()){
	
	centreX = atof(tokens[i + 1].c_str());
	centreY = atof(tokens[i + 2].c_str());
	centreZ = atof(tokens[i + 3].c_str());
	validCentre = true;
      }
    }

    if(!tokens[i].compare("f")){
      if(i + 3 < tokens.size()){
	
	forceX = atof(tokens[i + 1].c_str());
	forceY = atof(tokens[i + 2].c_str());
	forceZ = atof(tokens[i + 3].c_str());
	validForce = true;
      }
    }

  }

  valid = validName && validCentre && validForce;

  if(valid){

    //    cout << "FRP: " << name << " (c: " << centreX << " " << centreY << " " << centreZ << ") (f: " << forceX << " " << forceY << " " << forceZ << ")\n";

    bodyModel->setFRP(FRPIndex, VecPosition(centreX, centreY, centreZ), VecPosition(forceX, forceY, forceZ));
  }

  return valid;
}

vector<string> Parser::segment(const string &str, const bool &omitEnds){

  //  cout << "Segmenting " << str << "\n";

  vector<string> v;

  int ptr = 0;
  int length = str.length();

  if(omitEnds){
    ptr = 1;
    length = str.length() - 1;
  }

  int bracCount = 0;

  string currentString = "";

  do{
    
    while(str.at(ptr) != '('){
      ptr++;
    }

    if(ptr < length){

      currentString = "";
      
      do{

	char c = str.at(ptr);
	
	if(c == '('){
	  bracCount++;
	}
	else if(c == ')'){
	  bracCount--;
	}
      
	currentString.append(1, c);
	ptr++;
	
      }while(bracCount != 0 && ptr < length);

      if(bracCount == 0){
	v.push_back(currentString);
	currentString = "";
      }
    }
    
  }while(ptr < length);

  return v;
}

bool Parser::parse(const string &input){
  
  bool valid = true;

  vector<string> inputSegments = segment(input, false);

  for(int i = 0; i < inputSegments.size(); ++i){
    
    //Time
    if(inputSegments[i].at(1)== 't'){
      valid = parseTime(inputSegments[i]) && valid;
    }
    //GameState
    else if(inputSegments[i].at(1) == 'G'){
      
      if(inputSegments[i].at(2) == 'S'){
	valid = parseGameState(inputSegments[i]) && valid;
      }
      else{
	valid = parseGyro(inputSegments[i]) && valid;
      }
    }
    //Hear
    else if(inputSegments[i].at(1) == 'h'){
      valid = parseHear(inputSegments[i]) && valid;
    }
    //Hinge Joint
    else if(inputSegments[i].at(1) == 'H'){
      valid = parseHingeJoint(inputSegments[i]) && valid;
    }
    //See
    else if(inputSegments[i].at(1) == 'S'){
      valid = parseSee(inputSegments[i]) && valid;

      if(valid){
	processVision();
      }
    }
    //FRP
    else if(inputSegments[i].at(1) == 'F'){
      valid = parseFRP(inputSegments[i]) && valid;
    }
    else{
      valid = false;
    }
    
  }
  
  return valid;
}

void Parser::processVision(){

  vector<VecPosition> flagLocal;
  for(int i = 0; i < FLAG_NUM; ++i){
    flagLocal.push_back(visionSense->flagPolar[i].getCartesianFromPolar());
  }

  double fieldWidth = flagLocal[FLAG_1_L].getDistanceTo(flagLocal[FLAG_2_L]);
  double fieldLength = flagLocal[FLAG_1_L].getDistanceTo(flagLocal[FLAG_1_R]);
  
  worldModel->setFieldLength(fieldLength);
  worldModel->setFieldWidth(fieldWidth);

  VecPosition fieldXPlusYPlus, fieldXPlusYMinus, fieldXMinusYPlus, fieldXMinusYMinus;
  if(side == SIDE_LEFT){
    fieldXPlusYPlus = flagLocal[FLAG_1_R];
    fieldXPlusYMinus = flagLocal[FLAG_2_R];
    fieldXMinusYPlus = flagLocal[FLAG_1_L];
    fieldXMinusYMinus = flagLocal[FLAG_2_L];
  }
  else{//side == SIDE_RIGHT
    fieldXPlusYPlus = flagLocal[FLAG_2_L];
    fieldXPlusYMinus = flagLocal[FLAG_1_L];
    fieldXMinusYPlus = flagLocal[FLAG_2_R];
    fieldXMinusYMinus = flagLocal[FLAG_1_R];
  }

  VecPosition localFieldCentre = (fieldXPlusYPlus + fieldXMinusYMinus) * 0.5;
  VecPosition localFieldXDirection = (fieldXPlusYPlus - fieldXMinusYPlus).normalize();
  VecPosition localFieldYDirection = (fieldXPlusYPlus - fieldXPlusYMinus).normalize();
  VecPosition localFieldZDirection = (localFieldXDirection.crossProduct(localFieldYDirection)).normalize();
  
  worldModel->setLocalToGlobal(0, 0, localFieldXDirection.getX());
  worldModel->setLocalToGlobal(0, 1, localFieldXDirection.getY());
  worldModel->setLocalToGlobal(0, 2, localFieldXDirection.getZ());
  worldModel->setLocalToGlobal(0, 3, -(localFieldCentre.dotProduct(localFieldXDirection)));
  
  worldModel->setLocalToGlobal(1, 0, localFieldYDirection.getX());
  worldModel->setLocalToGlobal(1, 1, localFieldYDirection.getY());
  worldModel->setLocalToGlobal(1, 2, localFieldYDirection.getZ());
  worldModel->setLocalToGlobal(1, 3, -(localFieldCentre.dotProduct(localFieldYDirection)));
  
  worldModel->setLocalToGlobal(2, 0, localFieldZDirection.getX());
  worldModel->setLocalToGlobal(2, 1, localFieldZDirection.getY());
  worldModel->setLocalToGlobal(2, 2, localFieldZDirection.getZ());
  worldModel->setLocalToGlobal(2, 3, -(localFieldCentre.dotProduct(localFieldZDirection)));
  
  worldModel->setLocalToGlobal(3, 0, 0);
  worldModel->setLocalToGlobal(3, 1, 0);
  worldModel->setLocalToGlobal(3, 2, 0);
  worldModel->setLocalToGlobal(3, 3, 1.0);
  
  
  VecPosition localBodyCentre = VecPosition(0, 0, 0);
  VecPosition localBodyXDirection = VecPosition(1.0, 0, 0);
  VecPosition localBodyYDirection = VecPosition(0, 1.0, 0);
  VecPosition localBodyZDirection = VecPosition(0, 0, 1.0);
  
  VecPosition bodyCentre = worldModel->l2g(localBodyCentre);
  VecPosition bodyXDirection = (worldModel->l2g(localBodyXDirection) - bodyCentre).normalize();
  VecPosition bodyYDirection = (worldModel->l2g(localBodyYDirection) - bodyCentre).normalize();
  VecPosition bodyZDirection = (worldModel->l2g(localBodyZDirection) - bodyCentre).normalize();
  
  worldModel->setGlobalToLocal(0, 0, bodyXDirection.getX());
  worldModel->setGlobalToLocal(0, 1, bodyXDirection.getY());
  worldModel->setGlobalToLocal(0, 2, bodyXDirection.getZ());
  worldModel->setGlobalToLocal(0, 3, -(bodyCentre.dotProduct(bodyXDirection)));
  
  worldModel->setGlobalToLocal(1, 0, bodyYDirection.getX());
  worldModel->setGlobalToLocal(1, 1, bodyYDirection.getY());
  worldModel->setGlobalToLocal(1, 2, bodyYDirection.getZ());
  worldModel->setGlobalToLocal(1, 3, -(bodyCentre.dotProduct(bodyYDirection)));
  
  worldModel->setGlobalToLocal(2, 0, bodyZDirection.getX());
  worldModel->setGlobalToLocal(2, 1, bodyZDirection.getY());
  worldModel->setGlobalToLocal(2, 2, bodyZDirection.getZ());
  worldModel->setGlobalToLocal(2, 3, -(bodyCentre.dotProduct(bodyZDirection)));
  
  worldModel->setGlobalToLocal(3, 0, 0);
  worldModel->setGlobalToLocal(3, 1, 0);
  worldModel->setGlobalToLocal(3, 2, 0);
  worldModel->setGlobalToLocal(3, 3, 1.0);
  
  
  for(int i = 0; i < GOALPOST_NUM; ++i){
    VecPosition goalPostLocal = visionSense->goalPostPolar[i].getCartesianFromPolar();
    VecPosition goalPost = worldModel->l2g(goalPostLocal);
    worldModel->setGoalPost(i, goalPost);
  }
  
  {
    VecPosition ballLocal = visionSense->ballPolar.getCartesianFromPolar();
    VecPosition ball = worldModel->l2g(ballLocal);
    worldModel->setBall(ball);
  }
  
  int numTeammates = visionSense->teammatePolar.size();
  worldModel->setNumTeammates(numTeammates);
  
  for(int i = 0; i < numTeammates; ++i){
    VecPosition teammateLocal = visionSense->teammatePolar[i].getCartesianFromPolar();
    VecPosition teammate = worldModel->l2g(teammateLocal);
    worldModel->setTeammate(i, teammate);
  }
  
  int numOpponents = visionSense->opponentPolar.size();
  worldModel->setNumOpponents(numOpponents);
  
  for(int i = 0; i < numOpponents; ++i){
    VecPosition opponentLocal = visionSense->opponentPolar[i].getCartesianFromPolar();
    VecPosition opponent = worldModel->l2g(opponentLocal);
    worldModel->setOpponent(i, opponent);
  }
}

