#include "naobehavior.h"
bool kickedonce = 0;

int NaoBehavior::selectSkill(){
  if (!kickedonce) {
    kickedonce = 1;
    return SKILL_KICK_FRONT;
  } else {
    return SKILL_STAND;
  }
  
  if(turningLeft){

    double distance, angle;
    getTargetDistanceAndAngle(turnTarget, distance, angle);

    if(fabs(angle) > turnTargetAngleThreshold){
      return SKILL_TURN_LEFT;
    }

    turningLeft = false;
  }

  if(turningRight){

    double distance, angle;
    getTargetDistanceAndAngle(turnTarget, distance, angle);

    if(fabs(angle) > turnTargetAngleThreshold){
      return SKILL_TURN_RIGHT;
    }

    turningRight = false;
  }

  return goToBall();
  // return goToTarget(ball);
}

//Remember -- if the ball moves, things must be reset.
int NaoBehavior::goToBall(){

  VecPosition goalPostMid = (worldModel->getOppLeftGoalPost() + worldModel->getOppRightGoalPost()) * 0.5;
  goalPostMid.setZ(0);

  VecPosition direction = (goalPostMid - ball).normalize();

  double angleThreshold;
  double lateralThreshold;

  VecPosition target = ball - (VecPosition(direction) * 0.08);
  target.setZ(0);

  if(me.getDistanceTo(ball) > 0.6){
    positioning = false;
  }

  if(me.getDistanceTo(ball) > 0.4 && !positioning){
    return goToTarget(target);
  }
  else{

    int tempSkill;

    positioning = true;
    //dribble:    0.055, 15
    //kick:    0.035, 10

    /*
    //Dribble
    angleThreshold = 15.0;
    lateralThreshold = 0.055;
    bool canKick = positioningToKick(direction, angleThreshold, lateralThreshold, target, tempSkill);
    if(canKick){
      return SKILL_WALK_FRONT;
    }
    */
    
    //Kick
    angleThreshold = 10.0;
    lateralThreshold = 0.035;
    bool canKick = positioningToKick(direction, angleThreshold, lateralThreshold, target, tempSkill);
    if(canKick){
    kickDirection = direction;
    //    kickType = KICK_LONG;
    kickType = KICK_SHORT;
    return SKILL_KICK_FRONT;
    }

    return tempSkill;

  }

}

//Assumes target = z-0. Get to within 0.5m of target. Needs further tuning
int NaoBehavior::goToTarget(const VecPosition &target){

  double distance, angle;
  getTargetDistanceAndAngle(target, distance, angle);

  cout << "D, A: " << distance << ", " << angle << "\n";
  if(distance > 5.5){

    if(angle <= -14.0){
      setTurn(DIR_RIGHT, target, 10.0); return SKILL_TURN_RIGHT;
    }
    else if(angle >= 14.0){
      setTurn(DIR_LEFT, target, 10.0); return SKILL_TURN_LEFT;
    }
  }
  if(distance > 3.0){

    if(angle <= -16.0){
      setTurn(DIR_RIGHT, target, 11.0); return SKILL_TURN_RIGHT;
    }
    else if(angle >= 16.0){
      setTurn(DIR_LEFT, target, 11.0); return SKILL_TURN_LEFT;
    }
  }
  else if(distance > 0.25){

    if(angle <= -18.0){
      setTurn(DIR_RIGHT, target, 13.0); return SKILL_TURN_RIGHT;
    }
    else if(angle >= 18.0){
      setTurn(DIR_LEFT, target, 13.0); return SKILL_TURN_LEFT;
    }
  }

  return SKILL_WALK_FRONT;
}

void NaoBehavior::refresh(){

  myXDirection = worldModel->l2g(VecPosition(1.0, 0, 0)) - worldModel->l2g(VecPosition(0, 0, 0));
    myXDirection.setZ(0);
  myXDirection.normalize();
  
  myYDirection = worldModel->l2g(VecPosition(0, 1.0, 0)) - worldModel->l2g(VecPosition(0, 0, 0));
    myYDirection.setZ(0);
  myYDirection.normalize();

  //Anomalous
   myZDirection = worldModel->l2g(VecPosition(0, 0, 1.0)) - worldModel->l2g(VecPosition(0, 0, 0));
  //  myZDirection.setZ(0);
  myZDirection.normalize();

  me = worldModel->l2g(VecPosition(0, 0, 0));
  me.setZ(0);

  ball = worldModel->getBall();
  ball.setZ(0);

}

//Assumes target it z-0.
void NaoBehavior::getTargetDistanceAndAngle(const VecPosition &target, double &distance, double &angle){

  VecPosition targetDirection = VecPosition(target) - me;
  targetDirection.setZ(0);
  targetDirection.normalize();

  distance = (VecPosition(target) - me).getMagnitude();
  angle = VecPosition(0, 0, 0).getAngleBetweenPoints(myXDirection, targetDirection);
  if(myYDirection.dotProduct(targetDirection) < 0){
    angle = -angle;
  }

  cout << "Distance: " << distance << ", Angle: " << angle << "\n";
}

void NaoBehavior::resetSkills(){

  skill = SKILL_STAND;
  skillState = 0;

  turningLeft = false;
  turningRight = false;
  turnTarget = VecPosition(0, 0, 0);
  turnTargetAngleThreshold = 10.0;

  kickLeg = LEG_LEFT;
  kickDirection = VecPosition(1.0, 0, 0);

  positioning = false;

  bodyModel->setScale(EFF_LL3, 1.0);
  bodyModel->setScale(EFF_RL3, 1.0);

  kickType = KICK_LONG;
}

void NaoBehavior::setTurn(const int &direction, const VecPosition &target, const double &angleThreshold){

  turnTarget = target;
  turnTargetAngleThreshold = angleThreshold;

  if(direction == DIR_LEFT){
    turningLeft = true;
  }
  else{//direction == DIR_RIGHT
    turningRight = true;
  }
}


//Assumes within Xm of ball.
bool NaoBehavior::positioningToKick(const VecPosition &direction, const double &angleThreshold, const double &lateralThreshold, const VecPosition &targetPos, int &skillToTake){
  
  VecPosition target = VecPosition(targetPos);
  target.setZ(0);
  
  VecPosition leftCG, leftXAxis, leftYAxis, leftZAxis;
  bodyModel->getFootGlobalCGAndAxes(LEG_LEFT, leftCG, leftXAxis, leftYAxis, leftZAxis);
  
  double leftDistance = leftCG.getDistanceTo(target);
  double leftAngle = VecPosition(0, 0, 0).getAngleBetweenPoints(leftXAxis, direction);

  VecPosition rightCG, rightXAxis, rightYAxis, rightZAxis;
  bodyModel->getFootGlobalCGAndAxes(LEG_RIGHT, rightCG, rightXAxis, rightYAxis, rightZAxis);
  
  double rightDistance = rightCG.getDistanceTo(target);
  double rightAngle = VecPosition(0, 0, 0).getAngleBetweenPoints(rightXAxis, direction);

  VecPosition lateralDirection = VecPosition(0, 0, 1.0).crossProduct(direction);

  bool behindBall = (ball - me).dotProduct(direction) > 0.13;
  cout << "Behind Ball: " << behindBall << "\n";

  if(behindBall){

    double directionAngle = VecPosition(0, 0, 0).getAngleBetweenPoints(myXDirection, direction);
    if(myYDirection.dotProduct(direction) < 0){
      directionAngle = -directionAngle;
    }

    if(directionAngle > angleThreshold){
      skillToTake = SKILL_TURN_LEFT;
      return false;
    }
    else if(directionAngle < -angleThreshold){
      skillToTake = SKILL_TURN_RIGHT;
      return false;
    }

    if(leftDistance < rightDistance){

      if((target - leftCG).dotProduct(lateralDirection) > lateralThreshold){
	skillToTake = SKILL_WALK_LEFT;
	return false;
      }
      else if((target - leftCG).dotProduct(lateralDirection) < -lateralThreshold){
	skillToTake = SKILL_WALK_RIGHT;
	return false;
      }
      else if((target - leftCG).dotProduct(direction) > 0.13){
	skillToTake = SKILL_WALK_FRONT;
	return false;
      }
      else{
	kickLeg = LEG_LEFT;
	return true;
      }

    }
    else{

      if((target - rightCG).dotProduct(lateralDirection) > lateralThreshold){
	skillToTake = SKILL_WALK_LEFT;
	return false;
      }
      else if((target - rightCG).dotProduct(lateralDirection) < -lateralThreshold){
	skillToTake = SKILL_WALK_RIGHT;
	return false;
      }
      else if((target - rightCG).dotProduct(direction) > 0.13){
	skillToTake = SKILL_WALK_FRONT;
	return false;
      }
      else{
	kickLeg = LEG_RIGHT;
	return true;
      }
    }
  }
  else{ //In front of ball
    
    double lateralDirectionAngle = VecPosition(0, 0, 0).getAngleBetweenPoints(myXDirection, VecPosition(lateralDirection));
    if(myYDirection.dotProduct(VecPosition(lateralDirection)) < 0){
      lateralDirectionAngle = -lateralDirectionAngle;
    }

    cout << "LDA: " << lateralDirectionAngle << "\n";

    if((me - ball).dotProduct(lateralDirection) > 0){
      
      if(lateralDirectionAngle >= 0 && lateralDirectionAngle <= 160.0){
	skillToTake = SKILL_TURN_RIGHT;
	return false;
      }
      else if(lateralDirectionAngle < 0 && lateralDirectionAngle > -160.0){
	skillToTake = SKILL_TURN_LEFT;
	return false;
      }

      if((me - ball).dotProduct(lateralDirection) < 0.13){
	skillToTake = SKILL_WALK_BACK;
	return false;
      }
      else{
	skillToTake = SKILL_WALK_RIGHT;
	return false;
      }
    }
    else{

      if(lateralDirectionAngle >= 20.0 && lateralDirectionAngle <= 180.0){
	skillToTake = SKILL_TURN_LEFT;
	return false;
      }
      else if(lateralDirectionAngle < -20.0 && lateralDirectionAngle > -180.0){
	skillToTake = SKILL_TURN_RIGHT;
	return false;
      }

      if((me - ball).dotProduct(lateralDirection) > -0.13){
	skillToTake = SKILL_WALK_BACK;
	return false;
      }
      else{
	skillToTake = SKILL_WALK_LEFT;
	return false;
      }
    }

  }

  skillToTake = SKILL_WALK_FRONT;
  return false;
}

