#include "worldmodel.h"
#include "fstream.h"

extern map<string,float> namedParams;
extern string outputFile;

WorldModel::WorldModel(){
    written = 0;
        
    cycle = 0;
    time = -1.0;
    gameTime = -1.0;
    playMode = PM_BEFORE_KICK_OFF;
    
    uNum = 1;//dummy
    uNumSet = false;

    side = SIDE_LEFT;//dummy
    sideSet = false;

    for(int i = 0; i < GOALPOST_NUM; ++i){
      goalPost.push_back(VecPosition(0, 0, 0));
    }

    ball = VecPosition(0, 0, 0);
    
    for(int i = 0; i < TEAMMATE_NUM; ++i){
      teammate.push_back(VecPosition(0, 0, 0));
    }

    for(int i = 0; i < OPPONENT_NUM; ++i){
      opponent.push_back(VecPosition(0, 0, 0));
    }
    
    numTeammates = 0;
    numOpponents = 0;

    //In general, will be inferred from flags.
    fieldLength = 12.0;
    fieldWidth = 8.0;

    localToGlobal = HCTMatrix();
    globalToLocal = HCTMatrix();
}

WorldModel::~WorldModel(){
}

void WorldModel::display(){

    cout << "****************World Model******************************\n";
    std::cout << "Cycle: " << cycle << "\n";
    std::cout << "Time: " << time << "\n";
    std::cout << "Game Time: " << gameTime << "\n";
    std::cout << "Play Mode: " << playMode << "\n";

    std::cout << "uNumSet: " << uNumSet << ", uNum: " << uNum << "\n";
    std::cout << "sideSet: " << sideSet << ", side: " << side << "\n";

    cout << "GoalPost_1_l: " << goalPost[GOALPOST_1_L] << "\n";
    cout << "GoalPost_1_r: " << goalPost[GOALPOST_1_R] << "\n";
    cout << "GoalPost_2_l: " << goalPost[GOALPOST_2_L] << "\n";
    cout << "GoalPost_2_r: " << goalPost[GOALPOST_2_R] << "\n";

    cout << "Ball: " << ball << "\n";

    cout << "Teammates:\n";
    for(int i = 0; i < numTeammates; ++i){
        cout << i << ": " << teammate[i] << "\n";
    }

    cout << "Opponents:\n";
    for(int i = 0; i < numOpponents; ++i){
        cout << i << ": " << opponent[i] << "\n";
    }

    cout << "Field Length: " << fieldLength << "\n";
    cout << "Field Width: " << fieldWidth << "\n";

    cout << "*********************************************************\n";
}


/* 
 * Writes the value of the objective function to the file specified on
 * the command line with the --experimentout option.
 */
void WorldModel::quickDisplay() {
  VecPosition beamed = VecPosition(namedParams["agent2beamX"], namedParams["agent2beamY"], 0.3);
  double error = l2g(VecPosition(0,0,0)).getDistanceTo(beamed);
  
  if (gameTime > 0.5 && gameTime < 0.7) {
    cerr << "I am at: " << l2g(VecPosition(0,0,0)) << endl;
  }

  if (gameTime > 9.0 && !written) { 
    if (error > 3.0) {
      cout << "This one doesn't count.  We're " << error << "m from where we should be" << endl;
      exit(1); // Let's just let it re-run
    }
    double balldist = ball.getDistanceTo(VecPosition(6.0,0.0,0.0));
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  balldist << endl;
    file.close();
    cerr << "Ball at: " << ball << endl; 
    cerr << "Distance from goal: " << balldist << endl;
    written = 1;
  }
}


