#ifndef WORLDMODEL_H
#define WORLDMODEL_H

#include "headers.h"
#include "vecposition.h"
#include "hctmatrix.h"

#include <vector>
#include <map>
#include <string>

using namespace std;


class WorldModel{
 /*
 public:
   static std::string outputFile;
   static std::map<std::string, float> namedParams;
 */
   
 private:

  unsigned long cycle;
  double time;
  double gameTime;
  int playMode;

  int uNum;
  int side;
  
  bool uNumSet;
  bool sideSet;

  int written;
  
  std::vector<VecPosition> goalPost;
  VecPosition ball;
  std::vector<VecPosition> teammate;
  std::vector<VecPosition> opponent;

  int numTeammates, numOpponents;
  
  double fieldLength, fieldWidth;
  
  HCTMatrix localToGlobal, globalToLocal;

 public:
  
  WorldModel();
  ~WorldModel();
  
  inline unsigned long getCycle(){ return cycle; }
  inline void incrementCycle(){ cycle++;}
  inline void setCycle(const unsigned long &cycle){ this->cycle = cycle; }
  
  inline double getTime(){ return time; }
  inline void setTime(const double &time){ this->time = time; }

  inline double getGameTime(){ return gameTime; }
  inline void setGameTime(const double &gameTime){ this->gameTime = gameTime; }
  
  inline int getPlayMode(){ return playMode; }
  inline void setPlayMode(const int &playMode){ this->playMode = playMode; }
  
  inline int getUNum(){ return uNum; }
  inline void setUNum(const int &uNum){ this->uNum = uNum; }
  
  inline bool getUNumSet(){ return uNumSet; }
  inline void setUNumSet(const bool &uNumSet){ this->uNumSet = uNumSet; }
  
  inline int getSide(){ return side; }
  inline void setSide(const int &side){ this->side = side; }
  
  inline bool getSideSet(){ return sideSet; }
  inline void setSideSet(const bool &sideSet){ this->sideSet = sideSet; }
  
  inline VecPosition getGoalPost(const int &i){ return goalPost[i]; };
  inline void setGoalPost(const int &i, const VecPosition &goalPost){ this->goalPost[i] = goalPost; }
  
  inline VecPosition getMyLeftGoalPost(){ return ((side == SIDE_LEFT)? goalPost[GOALPOST_1_L] : goalPost[GOALPOST_2_R]); }
  inline VecPosition getMyRightGoalPost(){ return ((side == SIDE_LEFT)? goalPost[GOALPOST_2_L] : goalPost[GOALPOST_1_R]); }
  inline VecPosition getOppLeftGoalPost(){ return ((side == SIDE_LEFT)? goalPost[GOALPOST_1_R] : goalPost[GOALPOST_2_L]); }
  inline VecPosition getOppRightGoalPost(){ return ((side == SIDE_LEFT)? goalPost[GOALPOST_2_R] : goalPost[GOALPOST_1_L]); }

  inline VecPosition getBall(){ return ball; };
  inline void setBall(const VecPosition &ball){ this->ball = ball; }
  
  inline VecPosition getTeammate(const int &i){ return teammate[i]; };
  inline void setTeammate(const int &i, const VecPosition &teammate){ this->teammate[i] = teammate; }
  
  inline void setNumTeammates(const int &num){ this->numTeammates = num; }
  
  inline VecPosition getOpponent(const int &i){ return opponent[i]; };
  inline void setOpponent(const int &i, const VecPosition &opponent){ this->opponent[i] = opponent; }
  
  inline void setNumOpponents(const int &num){ this->numOpponents = num; }

  inline double getFieldLength(){ return fieldLength; }
  inline void setFieldLength(const double &fieldLength){ this->fieldLength = fieldLength; }
  
  inline double getFieldWidth(){ return fieldWidth; }
  inline void setFieldWidth(const double &fieldWidth){ this->fieldWidth = fieldWidth; }
  
  inline void setGlobalToLocal(const int &i, const int &j, const double &v){ this->globalToLocal.setCell(i, j, v); }

  inline void setLocalToGlobal(const int &i, const int &j, const double &v){ this->localToGlobal.setCell(i, j, v); }


  inline VecPosition g2l(const VecPosition &global){ return globalToLocal.transform(global); }
  inline VecPosition l2g(const VecPosition &local){ return localToGlobal.transform(local); }

  void display();
  void quickDisplay();
  void writeValue();
  double value();
};

#endif // WORLDMODEL_H

