#ifndef _WORLD_OBJECT_H
#define _WORLD_OBJECT_H

#include <boost/assign/list_of.hpp> // for 'map_list_of()'
using boost::assign::map_list_of;      // bring 'map_list_of()' into scope

#include <map>
using std::map;

#include "vecposition.h"
#include "Geometry.h"
#include "VisionObject.h"

enum WorldObjType {   // Types of objects
  WO_BALL, 

// Self and Teammates
  WO_TEAMMATE1,
  WO_TEAMMATE2,
  WO_TEAMMATE3,
  WO_TEAMMATE4,
  WO_TEAMMATE5,
  WO_TEAMMATE6,
  WO_TEAMMATE7,
  WO_TEAMMATE8,
  WO_TEAMMATE9,
  WO_TEAMMATE10,
  WO_TEAMMATE11,

  // Opponents?
  WO_OPPONENT1,
  WO_OPPONENT2,
  WO_OPPONENT3,
  WO_OPPONENT4,
  WO_OPPONENT5,
  WO_OPPONENT6,
  WO_OPPONENT7,
  WO_OPPONENT8,
  WO_OPPONENT9,
  WO_OPPONENT10,
  WO_OPPONENT11,

  FLAG_1_L,
  FLAG_1_R,
  FLAG_2_L,
  FLAG_2_R,

  //Goal Posts
  GOALPOST_1_L,
  GOALPOST_1_R,
  GOALPOST_2_L,
  GOALPOST_2_R,

  NUM_WORLD_OBJS,

};

static map<int,const char*> WorldObjType2Str = map_list_of
( WO_BALL,       "WO_BALL")
( WO_TEAMMATE1,  "WO_TEAMMATE1" )
( WO_TEAMMATE2,  "WO_TEAMMATE2" )
( WO_TEAMMATE3,  "WO_TEAMMATE3" )
( WO_TEAMMATE4,  "WO_TEAMMATE4" )
( WO_TEAMMATE5,  "WO_TEAMMATE5" )
( WO_TEAMMATE6,  "WO_TEAMMATE6" )
( WO_TEAMMATE7,  "WO_TEAMMATE7" )
( WO_TEAMMATE8,  "WO_TEAMMATE8" )
( WO_TEAMMATE9,  "WO_TEAMMATE9" )
( WO_TEAMMATE10, "WO_TEAMMATE10" )
( WO_TEAMMATE11, "WO_TEAMMATE11" )
( WO_OPPONENT1,  "WO_OPPONENT1" )
( WO_OPPONENT2,  "WO_OPPONENT2" )
( WO_OPPONENT3,  "WO_OPPONENT3" )
( WO_OPPONENT4,  "WO_OPPONENT4" )
( WO_OPPONENT5,  "WO_OPPONENT5" )
( WO_OPPONENT6,  "WO_OPPONENT6" )
( WO_OPPONENT7,  "WO_OPPONENT7" )
( WO_OPPONENT8,  "WO_OPPONENT8" )
( WO_OPPONENT9,  "WO_OPPONENT9" )
( WO_OPPONENT10, "WO_OPPONENT10" )
( WO_OPPONENT11, "WO_OPPONENT11" )
( FLAG_1_L,      "FLAG_1_L" )
( FLAG_1_R,      "FLAG_1_R" )
( FLAG_2_L,      "FLAG_2_L" )
( FLAG_2_R,      "FLAG_2_R" )
( GOALPOST_1_L,  "GOALPOST_1_L" )
( GOALPOST_1_R,  "GOALPOST_1_R" )
( GOALPOST_2_L,  "GOALPOST_2_L" )
( GOALPOST_2_R,  "GOALPOST_2_R" )
;

struct WorldObject {

  WorldObject() {
    pos.setVecPosition( 0, 0, 0 );
    id = -1;
    currentlySeen = false;
    cycleLastSeen = -1;
    timeLastSeen = -1;
    validPosition = false;
  }
  
  VecPosition pos;
  int id; // unique id for all objects that can ever be seen
  bool currentlySeen;
  int cycleLastSeen;
  double timeLastSeen;
  bool validPosition;
  VisionObject vision;
};

#endif
