#include "naobehavior.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include <cmath>
#include <cctype>
#include <exception>

#include "skillparser.h"
#include "skill.h"
/*
 * namedParams_ are a mapping between parameters and their values
 */
NaoBehavior::
NaoBehavior(const std::string teamName, int uNum, const map<string, string>& namedParams_) :
  namedParams( namedParams_ )
  {

  cout << "Constructing of Nao Behavior" << endl;

//  cerr << "DANIEL map: " << endl;
//  map<string, string>::const_iterator b( namedParams.begin() ), e( namedParams.end() );
//  for( ; b != e; ++b )
//    cerr << "key=" << b->first << " value=" << b->second << endl;

  classname = "NaoBehavior"; //TODO: eliminate it...

  mInit = false;
  initBeamed = false;

  agentTeamName = teamName;
  agentUNum = uNum;
  
  worldModel = new WorldModel();
  bodyModel = new BodyModel(worldModel);
  particleFilter = new PFLocalization( worldModel );
  parser = new Parser(worldModel, bodyModel, teamName, particleFilter);

  initBeamed = false;
  initialized = false;

  fallState = 0;
  fallenLeft = false;
  fallenRight = false;
  fallenDown = false;
  fallenUp = false;
  fallTimeStamp = -1;
  fallTimeWait = -1;
  kickTimeWait = -1;

  resetSkills();

  // TODO: read later from different files. Also treat paths more correctly? (system independent way)
  try {
    readSkillsFromFile( "./skills/walk_front.skl" );
    readSkillsFromFile( "./skills/walk_back.skl" );
    readSkillsFromFile( "./skills/walk_sideways.skl" );
    readSkillsFromFile( "./skills/stand.skl" );
    readSkillsFromFile( "./skills/turn.skl" );
    readSkillsFromFile( "./skills/walk_front0.skl" );
    readSkillsFromFile( "./skills/walk_front1.skl" );
  }
  catch( std::string& what ) {
    cerr << "Exception caught: " << what << endl;
    exit(1);
  }
  catch (std::exception& e)
  {
    cerr << e.what() << endl;
    exit(1);
  }

}

NaoBehavior::~NaoBehavior(){

  delete parser;
  delete worldModel;
  delete bodyModel;
}

string NaoBehavior::Init(){

  return "(scene rsg/agent/nao/nao.rsg)";
}

string NaoBehavior::Think(const std::string& message){

  //  cout << "(NaoBehavior) received message " << message << endl;


  bool parseSuccess = parser->parse(message);
  if(!parseSuccess && (worldModel->getPlayMode() != PM_BEFORE_KICK_OFF)){
    cout << "****************************************\n";
    cout << "Could not parse message: " << message << "\n";
    cout << "****************************************\n";
  }

  //  cout << "\nparseSuccess: " << parseSuccess << "\n";
  //  worldModel->display();
  this->updateFitness();
  bodyModel->refresh();
  //  bodyModel->display();
  //  bodyModel->displayDerived();



  string action;

  if (!mInit){

    mInit = true;
    stringstream ss;
    ss << "(init (unum " << agentUNum << ")(teamname " << agentTeamName << "))";
    action = ss.str();
    return action;
  }

  if(!initialized){

    if(!worldModel->getUNumSet() || !worldModel->getSideSet()){
      //      cout << "UNum and side not received yet.\n";
      action = "";
      return action;
    }

    if(!initBeamed){

      initBeamed = true;

      double beamX, beamY, beamAngle;

      // Call a virtual function
      // It could either be implemented here (real game) 
      // or in the inherited classes - for optimization agent.
      // Parameters are being filled in the beam function.
      this->beam( beamX, beamY, beamAngle );

      stringstream ss;
      ss << "(beam " << beamX << " " << beamY << " " << beamAngle << ")";
      action = ss.str();
      return action;
    }
    else{

      //      cout << "Not Initialized.\n";
      initialized = bodyModel->setInitialHead();
      initialized = bodyModel->setInitialArm(ARM_LEFT) && initialized;
      initialized = bodyModel->setInitialArm(ARM_RIGHT) && initialized;
      initialized = bodyModel->setInitialLeg(LEG_LEFT) && initialized;
      initialized = bodyModel->setInitialLeg(LEG_RIGHT) && initialized;
    }
  }
  else{

    act();
  }

  //action = "(say Hello)(he2 1)(he1 1)";
  //checkAngles();

  action = composeAction();

  //std::cout << "Sending action: " << action << "\n";
  return action;
}

void NaoBehavior::act(){

  refresh();

  if(checkingFall()){
    resetSkills();
    return;
  }

  if(improperPlayMode()){
    resetSkills();
    skill = SKILL_STAND;
  }
  else{
    
    
    //  if(worldModel->getCycle() < 500){
    //    return;
    //  }
    
    //  skill = SKILL_KICK_FRONT;
    //  skill = SKILL_TURN_RIGHT;
    //    skill = SKILL_STAND;
    //  skill = SKILL_WALK_BACK;
    //  skill = SKILL_WALK_FAST;
    
    //  cout << "LF: " << bodyModel->getFootCG(LEG_LEFT) << "\n";
    //  cout << "RF: " << bodyModel->getFootCG(LEG_RIGHT) << "\n";
    //cout << "Ball: " << ball << "\n";
    
    if( skills[skill]->done( bodyModel, worldModel->getTime() ) ) {
      skills[skill]->reset();
      skill = this->selectSkill();
//      skill = SKILL_WALK_FRONT;
//      static int i = 0;
//      if( i % 6 < 3 )
//        skill = SKILL_TURN_RIGHT;
//      else 
//        skill = SKILL_TURN_LEFT;
//      if ( i < 8 )
//        skill = SKILL_WALK_FRONT;
//      else if ( i < 9 )
//        skill = SKILL_TURN_LEFT;
//      else if ( i < 10 )
//        skill = SKILL_WALK_FRONT;
//      else 
//        skill = SKILL_TURN_LEFT;
//      ++i;
//       EnumParser<SkillType> enumParser; 
//       cerr << "Selected skill: " << enumParser.getStringFromEnum(skill) << " time: " << worldModel->getTime() << endl;  
    }
    
  }

//  cerr << "Selected skill: " << SkillType2Str[skill] << " time: " << worldModel->getTime() << endl;  

  
  
  // TODO: hack until Patrick removes these skills
  if (skill == SKILL_FIND_BALL_LEFT){
    skill = SKILL_TURN_LEFT;
  }
  if (skill == SKILL_FIND_BALL_RIGHT){
    skill = SKILL_TURN_RIGHT;
  }

  //skill is one of SKILL_WALK_FRONT, SKILL_WALK_BACK, SKILL_WALK_LEFT, SKILL_WALK_RIGHT, SKILL_TURN_LEFT, SKILL_TURN_RIGHT, SKILL_STAND, SKILL_KICK_FRONT (<- special treatment for now)
  skills[skill]->execute( bodyModel, worldModel->getTime() );
  worldModel->setLastSkill(skill);


  // TODO: hack, set the head turn behavior
  //if( skill == SKILL_KICK_FRONT ) {
  VecPosition me = worldModel->getMyPosition();
  me.setZ(0);
  ball = worldModel->getBall();
  ball.setZ(0);
  if (me.getDistanceTo(ball) < .4) {
      bodyModel->setScale(EFF_H1, 1);
      bodyModel->setTargetAngle(EFF_H1, 0.0);
  } else {
    // Currently, every 2 seconds
    int panState = ( static_cast<int>( worldModel->getTime() ) ) % 4;
    if( panState == 0 || panState == 2 ) {
      bodyModel->setScale(EFF_H1, 0.2);
      bodyModel->setTargetAngle(EFF_H1, .0);
    } else {
      int direction = (panState == 1) ? 1 : -1;
      bodyModel->setScale(EFF_H1, 0.2);
      bodyModel->setTargetAngle(EFF_H1, direction*32);
    }
  }
}

/*
 * Throws string
 */
void NaoBehavior::readSkillsFromFile( const std::string& filename ) {
  cerr << "Loading skills from file " << filename << endl;


  // Load a skill file to memory. Assuming a file is < 4K
  
  int buffsize = 4096;
  char buff[buffsize];
  fstream skillFile( filename.c_str() );
  skillFile.read( buff, buffsize );
  if( !skillFile.eof() ) {
    throw "failed to read the whole skill file " + filename;
  }
  // padding with \0 at the end
  int numRead = skillFile.gcount();
  buff[numRead] = '\0';



  // Preprocessing: replace parameters by values.

  string skillDescription("");
  skillDescription.reserve( buffsize );
  for( int i = 0; i < numRead; ++i ) {
    char c = buff[i];
    if( c == '$' ) {
      // parameter - replace it

      string param("");
      i += 1;
      while( i < numRead && ( isalnum( buff[i] ) || buff[i] == '_' ) ) {
        param += buff[i];
        ++i;
      }

      map<string, string>::const_iterator it = namedParams.find( param );
      if( it == namedParams.end() )
        throw "Missing parameter in parameter file " + param;
      skillDescription += it->second;

      if( i < numRead ) 
        skillDescription += buff[i]; 

    } else {
      // not a param, just concatenate c
      skillDescription += c;

    }
  }
  //cerr << "skillDescription is: \n" << skillDescription << endl;






  // TODO: hack, should be removed once we have a proper skill
  shared_ptr<Skill> kickSkill( new KickSkill( worldModel, kickLeg, ball, kickDirection, kickType ) );
  skills[SKILL_KICK_FRONT] = kickSkill;




  // Parse

  SkillParser parser( skills, bodyModel );
  parse_info<iterator_t> info = parse( skillDescription.c_str(), 
                                       parser, 
                                       ( space_p | comment_p("#") ) 
                                      );


//  // Daniel: parse with file iterators
//  // prepare file iterators
//  iterator_t first(filename.c_str());
//  if (!first)
//  {
//    throw "Unable to open file!\n";
//    
//  } else {
//    std::cout << "Opened file" << filename << endl;
//  }
//  iterator_t last = first.make_end();
//
//  // parse
//  parse_info<iterator_t> info = parse( first, last, 
//                                       parser, 
//                                       ( space_p | comment_p("#") ) 
//                                      );

/*  cerr << "DEBUG: skills number: " << skills.size() << endl;
  map< SkillType, shared_ptr<Skill> >::iterator begin( skills.begin() );
  map< SkillType, shared_ptr<Skill> >::iterator end( skills.end() );
  EnumParser<SkillType> enumParser; 
  for( ; begin != end ; ++begin ) {
    cerr << "skill=" << enumParser.getStringFromEnum(begin->first) << endl; 
    begin->second->display();
  }
  cerr << "DONE DEBUG\n";

*/

  // check results
  // TODO: cannot get full here - check WHY?
  if (info.hit)
  {
    cout << "-------------------------\n";
    cout << "Parsing succeeded\n";
    cout << "-------------------------\n";
    cout << "stop "  << info.stop << endl;
    cout << "full " << info.full << endl;
    cout << "length " << info.length << endl;
  }
  else
  {
    cout << "-------------------------\n";
    cout << "Parsing failed\n";
    //            cout << "stopped at: \": " << info.stop << "\"\n";
    cout << "-------------------------\n";
    throw "Parsing failed";
  }

}

/*
 * Real game beaming.
 * Filling params x y angle 
 */
void NaoBehavior::
beam( double& beamX, double& beamY, double& beamAngle ) {

  // Real game beaming
  //1: Goalie.
  if(worldModel->getUNum() == 1){
    beamX = -HALF_FIELD_X + .5;
    beamY = 0;
    beamAngle = 0;
  }
  else if(worldModel->getUNum() == 2){
    beamX = -0.2; // -5; // -4; // 3; // -0.2;
    beamY = 0.05; // 0; // -4; // -4; // 2.5; // 0.05;
    beamAngle = 0; // 65; // 180; // 315; //0;
  }
  else if(worldModel->getUNum() == 3){
    //	beamX = -0.5;
    //	beamY = 1.5;
    //	beamAngle = 0;

    beamX = -.5;
    beamY = 2;
    beamAngle = -45;
  }
  else if(worldModel->getUNum() == 4){
    beamX = -.5;
    beamY = -2;
    beamAngle = 45;
  }
  else if(worldModel->getUNum() == 5){
    beamX = -3;
    beamY = 1;
    beamAngle = -20;
  }
  else if(worldModel->getUNum() == 6){
    beamX = -3;
    beamY = -1;
    beamAngle = 20;
  }
  else{
    beamX = -2.0 - ((double)(worldModel->getUNum()) / 100.0);
    beamY = (drand48() - 0.5) * ((double)(worldModel->getUNum()) / 100.0);
    beamAngle = 0;
  }
}

