#ifndef NAOBEHAVIOR_H
#define NAOBEHAVIOR_H

#include "behavior.h"
#include "headers.h"
#include "parser.h"
#include "worldmodel.h"
#include "bodymodel.h"
#include "PFLocalization.h"
#include "skill.h"

#include <map>

using namespace std;

enum TurnState {NONE, TURNING_LEFT, TURNING_RIGHT, TURNING_LEFT_FOR_BALL, TURNING_RIGHT_FOR_BALL};

class NaoBehavior : public Behavior{

 protected:
  
  // TODO: eliminate it and use a better solution
  string classname;

  map< SkillType, shared_ptr<Skill> > skills;
  const map<string, string>& namedParams;
  
  std::string agentTeamName;
  int agentUNum;
  
  Parser *parser;
  WorldModel *worldModel;
  BodyModel *bodyModel;
  PFLocalization* particleFilter;
  
  bool mInit;
  bool initBeamed;
  
  bool initialized;

  TurnState currentTurnState;

  SkillType skill;
  int skillState;

  int fallState;
  bool fallenLeft, fallenRight, fallenDown, fallenUp;
  double fallTimeStamp;
  double fallTimeWait;
  double kickTimeWait;

  VecPosition turnTarget;
  double turnTargetAngleThreshold;
  
  int kickLeg;
  VecPosition kickDirection;
  int kickType;

  bool positioning;


  VecPosition me;
  VecPosition myXDirection, myYDirection, myZDirection;

  VecPosition ball;

  bool closestTeammateToBall;
  bool closestTeammateToGoal;

  string composeAction();
  void checkAngles();
  
  void resetSkills();
  void refresh();
  void act();

  // ----------------------------------------------------
  // ---------  THESE FUNCTIONS ARE 
  // ---------  TO BE OVERRIDEN BY OPTIMIZATION AGENTS...
  // TODO: Virtual only to be overriden by optimization behaviors. Is there a better way?
  virtual SkillType selectSkill();
  virtual void beam( double& beamX, double& beamY, double& beamAngle );
  virtual void updateFitness() {}
  // ----------------------------------------------------

  void walk_front();
  void walk_back();
  void walk_left();
  void walk_right();
  void turn_left();
  void turn_right();
  void kick_front();
  void find_ball_left();
  void find_ball_right();

  bool checkingFall();
  void riseLeft();
  void riseRight();
  void riseUpLeft();
  void riseUpRight();
  void riseDown();
  void stand();
  void walk_fast();

  SkillType goToTarget(const VecPosition &target);
  //  int goToBall();
  SkillType driveBallToGoal();
  SkillType findBall();
  SkillType goToSpace();
  SkillType watchBall();

  void getTargetDistanceAndAngle(const VecPosition &target, double &distance, double &angle);
  void setTurn(const int &direction, const VecPosition &target, const double &angleThreshold);

  bool positioningToKick(const VecPosition &direction, const double &angleThreshold, const double &lateralThreshold, const VecPosition &targetPos, SkillType &skillToTake);

  bool improperPlayMode();
  SkillType goalie();

  void readSkillsFromFile( const std::string& filename );

 public:

  NaoBehavior(const std::string teamName, int uNum, const map<string, string>& namedParams_);
  virtual ~NaoBehavior();
  
  virtual std::string Init();
  virtual std::string Think(const std::string& message);

};

#endif // NAOBEHAVIOR_H

