#include "optimizationbehaviors.h"
#include <fstream>


/*
 *
 *
 * WALK FRONT OPTIMIZATION AGENT
 *
 *
 *
 */
OptimizationBehaviorWalkFront::
OptimizationBehaviorWalkFront( const std::string teamName, 
                               int uNum, 
                               const map<string, string>& namedParams_, 
                               const string& outputFile_) 
  : NaoBehavior( teamName, 
                 uNum, 
                 namedParams_ ), 
    outputFile( outputFile_ ) {

}

void OptimizationBehaviorWalkFront::
beam( double& beamX, double& beamY, double& beamAngle ) {
  beamX = -8.5;
  beamY = -3.00;
  beamAngle = 0;
}

SkillType OptimizationBehaviorWalkFront::
selectSkill() {
  static int i = -1;
  ++i;
  if( i < 3 ) {
//    cerr << "SKILL_WALK_FRONT0" << endl;
    return SKILL_WALK_FRONT0;
  }  
  else {
    //cerr << "SKILL_WALK_FRONT" << endl;
    return SKILL_WALK_FRONT;
  } 
}

void OptimizationBehaviorWalkFront::
updateFitness() {

  double gameTime = worldModel->getGameTime();
  static VecPosition at1s;
  static bool written = false;

  if (gameTime > 0.8 && gameTime < 1.0) {
    at1s = worldModel->getMyPositionGroundTruth();
  }

  if (gameTime > 0.5 && gameTime < 16.0) {
//    cerr << "I am at: " << worldModel->getMyPositionGroundTruth() << endl;
  }

  if (gameTime > 16.0 && !written) { 
    VecPosition me = worldModel->getMyPositionGroundTruth(); 
    VecPosition start = at1s; 
    //double walkdist = start.getDistanceTo(me);
    //double walkdist = abs( (me-start).getX()/(me-start).getY() ) >= 5 ? (me-start).getX() : -1; // start.getDistanceTo(me);
    double walkdist = (me-start).getX();
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  walkdist << endl;
    file.close();
    cerr << "At 1s, I was at: " << at1s << endl;
    cerr << "I am at: " << me << endl; 
    cerr << "Distance walked: " << walkdist << endl;
    written = true;
  }

}


/*
 *
 *
 * WALK BACK OPTIMIZATION AGENT
 *
 *
 *
 */
OptimizationBehaviorWalkBack::
OptimizationBehaviorWalkBack( const std::string teamName, 
                               int uNum, 
                               const map<string, string>& namedParams_, 
                               const string& outputFile_) 
  : NaoBehavior( teamName, 
                 uNum, 
                 namedParams_ ), 
    outputFile( outputFile_ ) {

}

void OptimizationBehaviorWalkBack::
beam( double& beamX, double& beamY, double& beamAngle ) {
  beamX = -9.0;
  beamY = -6.0;
  beamAngle = 180.0;
}


SkillType OptimizationBehaviorWalkBack::
selectSkill() {
  return SKILL_WALK_BACK;
}

void OptimizationBehaviorWalkBack::
updateFitness() {

  double gameTime = worldModel->getGameTime();
  static VecPosition at1s;
  static bool written = false;

  if (gameTime > 0.8 && gameTime < 1.0) {
    at1s = worldModel->getMyPositionGroundTruth();
  }

  if (gameTime > 0.5 && gameTime < 16.0) {
    cerr << "I am at: " << worldModel->getMyPositionGroundTruth() << endl;
  }

  if (gameTime > 16.0 && !written) { 
    VecPosition me = worldModel->getMyPositionGroundTruth(); 
    VecPosition start = at1s; 
    //double walkdist = start.getDistanceTo(me);
    //double walkdist = abs( (me-start).getX()/(me-start).getY() ) >= 5 ? (me-start).getX() : -1; // start.getDistanceTo(me);
    double walkdist = (me-start).getX();
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  walkdist << endl;
    file.close();
    cerr << "At 1s, I was at: " << at1s << endl;
    cerr << "I am at: " << me << endl; 
    cerr << "Distance walked: " << walkdist << endl;
    written = true;
  }

}


/*
 *
 *
 * TURN LEFT OPTIMIZATION AGENT
 *
 *
 *
 */
OptimizationBehaviorTurnLeft::
OptimizationBehaviorTurnLeft( const std::string teamName, 
                               int uNum, 
                               const map<string, string>& namedParams_, 
                               const string& outputFile_) 
  : NaoBehavior( teamName, 
                 uNum, 
                 namedParams_ ), 
    outputFile( outputFile_ ) {

}

void OptimizationBehaviorTurnLeft::
beam( double& beamX, double& beamY, double& beamAngle ) {
  beamX = -3.5;
  beamY = 0;
  beamAngle = 0;
}

SkillType OptimizationBehaviorTurnLeft::
selectSkill() {
  return SKILL_TURN_LEFT;
}

void OptimizationBehaviorTurnLeft::
updateFitness() {

  double gameTime = worldModel->getGameTime();
  static double at1s;
  static double lastAngle;
  static double totalAngle = 0;
  static bool fallen = false;
  static bool written = false;

  if (gameTime > 0.8 && gameTime < 2) {
    at1s = worldModel->getMyAngDegGroundTruth();
    lastAngle = 0; // TODO: assuming we beam at 0 // at1s; 
  }

  if (gameTime > 2 && gameTime < 31.0) {

    if( worldModel->getMyPositionGroundTruth().getZ() < 0.3 )
      fallen = true;

    if( !fallen ) {
    // TODO: uncomment this line and comment the next line
      double currentAngle = worldModel->getMyAngDegGroundTruth();
//      double currentAngle = worldModel->getMyAngDeg();

      totalAngle += diff(currentAngle, lastAngle);
      lastAngle = currentAngle;

      cerr << "I am at: " << currentAngle << " degrees and the total angle is " << totalAngle <<  endl;
    } else {
      // do nothing stop accumulate angle
    }
    
  }

  if (gameTime > 31.0 && !written) { 
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  totalAngle << endl;
    file.close();
    cerr << "At 1s, I was at: " << at1s << endl;
    cerr << "TotalAngleTurned: " << totalAngle << endl;
    written = true;
  }

}

double OptimizationBehaviorTurnLeft::
diff( double currentAngle, double lastAngle ) {
  
  // usually it's easy, only make sure it's still correct when crossing the 360 deg line
  if( abs( currentAngle - lastAngle ) > 180 ) { // crossed the 360 line, I assume.
    if( currentAngle < lastAngle ) {
      currentAngle += 360;
    } else {
      lastAngle += 360;
    }

  }
  
  return currentAngle - lastAngle;

}


/*
 *
 *
 * WALK SIDE OPTIMIZATION AGENT
 *
 *
 *
 */
OptimizationBehaviorWalkSideway::
OptimizationBehaviorWalkSideway( const std::string teamName, 
                               int uNum, 
                               const map<string, string>& namedParams_, 
                               const string& outputFile_) 
  : NaoBehavior( teamName, 
                 uNum, 
                 namedParams_ ), 
    outputFile( outputFile_ ) {

}

void OptimizationBehaviorWalkSideway::
beam( double& beamX, double& beamY, double& beamAngle ) {
  beamX = -8.5;
  beamY = -3.00;
  beamAngle = 90;
}

SkillType OptimizationBehaviorWalkSideway::
selectSkill() {
  return SKILL_WALK_RIGHT;
}

void OptimizationBehaviorWalkSideway::
updateFitness() {

  double gameTime = worldModel->getGameTime();
  static VecPosition at1s;
  static bool written = false;

  if (gameTime > 0.8 && gameTime < 1.0) {
    at1s = worldModel->getMyPositionGroundTruth();
  }

  if (gameTime > 0.5 && gameTime < 31.0) {
    cerr << "I am at: " << worldModel->getMyPositionGroundTruth() << endl;
  }

  if (gameTime > 31.0 && !written) { 
    VecPosition me = worldModel->getMyPositionGroundTruth(); 
    VecPosition start = at1s; 
    //double walkdist = start.getDistanceTo(me);
    VecPosition walked = me-start;
    double walkdist = ( walked.getX() > 0 && 
                        abs( walked.getY() ) / walked.getX()  < 0.33 )
                      ? walked.getX() : -1; // start.getDistanceTo(me);
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  walkdist << endl;
    file.close();
    cerr << "At 1s, I was at: " << at1s << endl;
    cerr << "I am at: " << me << endl; 
    cerr << "Distance walked: " << walkdist << endl;
    written = true;
  }
}


/*
 *
 *
 * WALK TARGET OPTIMIZATION AGENT
 *
 *
 *
 */
OptimizationBehaviorWalkTarget::
OptimizationBehaviorWalkTarget( const std::string teamName, 
                               int uNum, 
                               const map<string, string>& namedParams_, 
                               const string& outputFile_) 
  : NaoBehavior( teamName, 
                 uNum, 
                 namedParams_ ), 
    outputFile( outputFile_ ) {

}

void OptimizationBehaviorWalkTarget::
beam( double& beamX, double& beamY, double& beamAngle ) {
  beamX = -9.0;
  beamY = 0.0;
  beamAngle = 0;
}

SkillType OptimizationBehaviorWalkTarget::
selectSkill() {
  return goToTarget(VecPosition(9.0,0.0,0));
}

void OptimizationBehaviorWalkTarget::
updateFitness() {

  double gameTime = worldModel->getGameTime();
  static bool written = false;

  if (gameTime > 20.0 && !written) { 
    VecPosition me = worldModel->getMyPositionGroundTruth();  
    double walkdist = -me.getDistanceTo(VecPosition(9.0,0.0,0));
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  walkdist << endl;
    file.close();
    cerr << "Distance from target: " << walkdist << endl;
    written = true;
  }

}
