#ifndef PARSER_H
#define PARSER_H

#include <iostream>
#include <string>
#include <sstream>
#include <vector>

#include "headers.h"
#include "worldmodel.h"
#include "bodymodel.h"
#include "VisionObject.h"
#include "PFLocalization.h"
#include <boost/shared_ptr.hpp>

using namespace std;

class Parser{

 private:

  WorldModel *worldModel;
  BodyModel *bodyModel;
  PFLocalization* particleFilter;

  int uNum;
  int side;

  string teamName;

  bool fProcessedVision;

 protected:
  
  vector<string> tokenise(const string &s);
  vector<string> tokeniseCommaDelim(const string &s);
  bool parseTime(const string &str);
  bool parseGameState(const string &str);
  bool parseGyro(const string &str);
  bool parseAccelerometer(const string &str);
  bool parseHear(const string &str);
  bool parseHingeJoint(const string &str);
  bool parseSee(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs);
  bool parseGoalPost(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs);
  bool parseFlag(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs);
  bool parseBall(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs);
  bool parsePlayer(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs);
  bool parseFRP(const string &str);
  bool parseMyPos(const string &str);
  vector<string> segment(const string &str, const bool &omitEnds);
  void processVision(vector< boost::shared_ptr<VisionObject> >& visionObjs);
  void computeLocalCornerPositions( VecPosition& fieldXPlusYPlus,
                                    VecPosition& fieldXPlusYMinus,
                                    VecPosition& fieldXMinusYPlus, 
                                    VecPosition& fieldXMinusYMinus );
 public:
  
  Parser(WorldModel *worldModel, BodyModel *bodyModel, const string &teamName, PFLocalization* particleFilter);
  ~Parser();

  bool parse(const string &input);
};

#endif // PARSER_H

