#include "simplelocalization.h"

bool SimpleLocalization::getLocalization(
    WorldModel *worldModel,
    BodyModel *bodyModel) 
{
  /* Here is where we should add localization code that, based upon seeing 
     enough set landmarks, is able to calculate the approximate local 
     coordinates for all the flags.  High level psuedo code would probably be
     something like the following:

     - If we have at least 2 landmarks use trigonometry to determine
     approximate global cooirdinate position of agent
     - With global coordinates of agent determine all locations of flags
     relative to agent (local coordinate system)
     - Populate localFlags vector with these local coordinates for the flags
     - Also compute and populate visonSense->goalPostPolar vector with local
     polar coordinates for the goal posts that weren't seen

     Ground truth values of landmarks:
     if(side == SIDE_LEFT){
     FLAG_1L = VecPosition(-6, 4, 0)
     FLAG_1R = VecPosition(6, 4, 0)
     FLAG_2L = VecPosition(-6, -4, 0)
     FLAG_2R = VecPosition(6, -4, 0)
     GP_1L = VecPosition(-6, .7, .8)
     GP_1R = VecPosition(6, .7, .8)
     GP_2L = VecPosition(-6, -.7, .8)
     GP_2R = VecPosition(6, -.7, .8)
     } else {//side == SIDE_RIGHT (we just flip the X, Y signs)
     FLAG_1L = VecPosition(6,-4, 0)
     FLAG_1R = VecPosition(-6, -4, 0)
     FLAG_2L = VecPosition(6, 4, 0)
     FLAG_2R = VecPosition(-6, 4, 0)
     GP_1L = VecPosition(6, -.7, .8)
     GP_1R = VecPosition(-6, -.7, .8)
     GP_2L = VecPosition(6, .7, .8)
     GP_2R = VecPosition(-6, .7, .8)
     }
     */

//  visionSense->landmarkPolar.clear();
//  visionSense->landmarkGlobal.clear();

//  double headAnglePan = bodyModel->getJointAngle(EFF_H1);
//  double headAngleTilt = bodyModel->getJointAngle(EFF_H2);
//  VecPosition headAnglePolarCorrection = VecPosition(0, headAnglePan, headAngleTilt);

  static VecPosition cGlobal = VecPosition(0,0,1);
  static double orientation = 0;
//
//  for (int i = FLAG_1_L; i <= FLAG_2_R; i++) {
//    WorldObject* pObj = worldModel->getWorldObject( i );
//    if ( pObj->currentlySeen ) {
//      double theta = pObj->polar->getY();
//      if (theta < 90 && theta > -90) {
//        //cout << "Using flag " << i << " theta = " << theta << endl;
//        visionSense->landmarkPolar.push_back(visionSense->flagPolar[i-FLAG_1_L]+headAnglePolarCorrection);
//        visionSense->landmarkGlobal.push_back(worldModel->getWorldObject(i)->pos);
//      }
//    }
//  }
//  for (int i = GOALPOST_1_L; i <= GOALPOST_2_R; i++) {
//    if (visionSense->goalPostPolar[i-GOALPOST_1_L] != NULL) {
//      double theta = visionSense->goalPostPolar[i-GOALPOST_1_L].getCartesianFromPolar().getTheta();
//      if (theta < 90 && theta > -90) {
//        //cout << "Using gp " << i << " theta = " << theta << endl;
//        visionSense->landmarkPolar.push_back(visionSense->goalPostPolar[i-GOALPOST_1_L]+headAnglePolarCorrection);
//        visionSense->landmarkGlobal.push_back(worldModel->getWorldObject(i)->pos);
//      }
//    }
//  }
//
//  if (worldModel->getSide() == SIDE_RIGHT) {
//    // Flip the signs of X and Y if we're on the right side
//    for (int i = 0; i < visionSense->landmarkGlobal.size(); i++) {
//      visionSense->landmarkGlobal[i] *= VecPosition(-1, -1, 1); 
//    }
//  }

  //double fieldWidth = flagLocal[FLAG_1_L].getDistanceTo(flagLocal[FLAG_2_L]);
  //double fieldLength = flagLocal[FLAG_1_L].getDistanceTo(flagLocal[FLAG_1_R]);

//  worldModel->setFieldLength(FIELD_LENGTH);
//  worldModel->setFieldWidth(FIELD_WIDTH);

  vector<int> seenObjects;
  for (int i = FLAG_1_L; i <= GOALPOST_2_R; i++) {
    WorldObject* pObj = worldModel->getWorldObject( i );
    if( pObj->currentlySeen) {
      double theta = pObj->vision.polar.getCartesianFromPolar().getTheta();
      if (theta < 90 && theta > -90) {
	seenObjects.push_back( i );
      }
    }
  }

  if (seenObjects.size() < 2) {
    const double TURN_DEGREE_DELTA = .45;
    const double FRONT_WALK_DELTA = .009;
    const double BACK_WALK_DELTA = -.009;
    const double SIDE_WALK_DELTA = .003;
    worldModel->setLocalized(false);
    //cout << "Not localized!" << endl;
    switch(worldModel->getLastSkill()) {
      case SKILL_TURN_RIGHT:
        orientation -= TURN_DEGREE_DELTA;
        break;
      case SKILL_TURN_LEFT:
        orientation += TURN_DEGREE_DELTA;
        break;
      case SKILL_WALK_FRONT:
        cGlobal += VecPosition(FRONT_WALK_DELTA*cosDeg(orientation), FRONT_WALK_DELTA*sinDeg(orientation), 0);
        break;
      case SKILL_WALK_BACK:
        cGlobal += VecPosition(BACK_WALK_DELTA*cosDeg(orientation), BACK_WALK_DELTA*sinDeg(orientation), 0);
        break;
      case SKILL_WALK_RIGHT:
        cGlobal += VecPosition(SIDE_WALK_DELTA*sinDeg(orientation), SIDE_WALK_DELTA*cosDeg(orientation), 0);
        break;
      case SKILL_WALK_LEFT:
        cGlobal += VecPosition(-SIDE_WALK_DELTA*sinDeg(orientation), -SIDE_WALK_DELTA*cosDeg(orientation), 0);
        break;
      case SKILL_FIND_BALL_LEFT:
        orientation += TURN_DEGREE_DELTA;
        break;
      case SKILL_FIND_BALL_RIGHT:
        orientation -= TURN_DEGREE_DELTA;
        break;
    }
  } else {  
    WorldObject* obj0 = worldModel->getWorldObject( seenObjects[0] );
    VecPosition landmarkLocalA = obj0->vision.polar.getCartesianFromPolar();
    VecPosition landmarkGlobalA = obj0->pos;

    WorldObject* obj1 = worldModel->getWorldObject( seenObjects[1] );
    VecPosition landmarkLocalB =  obj1->vision.polar.getCartesianFromPolar();
    VecPosition landmarkGlobalB = obj1->pos;

    // Throw out Z values as we only care about 2D
    landmarkLocalA.setZ(0);
    landmarkGlobalA.setZ(0);
    landmarkLocalB.setZ(0);
    landmarkGlobalB.setZ(0);

    // Make sure landmark A is the furthest left in our vision of the two landmarks
    if (landmarkLocalA.getTheta() < landmarkLocalB.getTheta()) {
      VecPosition temp = landmarkLocalA;
      landmarkLocalA = landmarkLocalB;
      landmarkLocalB = temp;
      temp = landmarkGlobalA;
      landmarkGlobalA = landmarkGlobalB;
      landmarkGlobalB = temp;
    }
    //cout << "aTheat = " << landmarkLocalA.getTheta() << ", bTheata = " << landmarkLocalB.getTheta() << endl;

    /* Law of cosines 
       c
       A-----B
       \   /
       b \ / a
       C
       (agent)

       AngleA = arccos((b^2+c^2-a^2)/2bc)
       */
    double a = landmarkLocalB.getMagnitude();
    double b = landmarkLocalA.getMagnitude();
    double c = (landmarkGlobalA-landmarkGlobalB).getMagnitude();

    double angleA = acos((pow(b,2)+pow(c,2)-pow(a,2))/(2*b*c));

    //cout << "a = " << a << ", b = " << b << ", c = " << c << ", angleA = " << angleA << endl;
    double offsetAngle;
    /*if (landmarkGlobalB.getX()-landmarkGlobalA.getX() == 0) {
      if (landmarkGlobalA.getY() < landmarkGlobalB.getY()) {
        offsetAngle = 3*M_PI/2;
      } else {
        offsetAngle = M_PI/2;
      }
    } else if (landmarkGlobalB.getY()-landmarkGlobalA.getY() == 0) {
      if (landmarkGlobalA.getX() < landmarkGlobalB.getX()) {
        offsetAngle = M_PI;
      } else {
        offsetAngle = 0;
      }
    } else {
      double slope = (landmarkGlobalB.getY()-landmarkGlobalA.getY())/(landmarkGlobalB.getX()-landmarkGlobalA.getX());
      offsetAngle = atan(slope);
      }*/
    offsetAngle = atan2(landmarkGlobalA.getY()-landmarkGlobalB.getY(), landmarkGlobalA.getX()-landmarkGlobalB.getX());
    offsetAngle -= angleA;

    //cout << "offsetAngle = " << Rad2Deg(offsetAngle) << endl;
    double cX = landmarkGlobalA.getX() - b*cos(offsetAngle);
    double cY = landmarkGlobalA.getY() - b*sin(offsetAngle);
    double orien = Rad2Deg(offsetAngle)-landmarkLocalA.getTheta();
    if (orien <= -180) {
      orien += 360;
    }
    if (isnan(cX) || isnan(cY) || isnan(orien)) {
      // probably fell over and have all kinds of erroneous input
      worldModel->setLocalized(false);
    } else {
      cGlobal = VecPosition(cX, cY, 1);
      orientation = orien;
      worldModel->setLocalized(true);
    }
  }
  worldModel->setMyPosition( cGlobal );
  worldModel->setMyAngDeg( orientation );
  return worldModel->isLocalized();
}

