#ifndef SKILL_H
#define SKILL_H

#include <boost/shared_ptr.hpp>
#include "bodymodel.h"

using boost::shared_ptr;

/////////////////////////////////////////////////////
// Macros
/////////////////////////////////////////////////////
class Macro{
  private:
  protected:
  public:
    Macro() {}
    virtual ~Macro() {}

    virtual void execute(BodyModel *bodyModel) = 0;
    virtual shared_ptr<Macro> getReflection(BodyModel *bodyModel) = 0;
    virtual void display() = 0;
};

class IncTar : public Macro{

  private:
  protected:

  // These two vectors define a mapping between effector->increment
  vector<int> effectorIDs;
  vector<double> increments;
//    int effID;
//    double increment;

  public:

    IncTar(const vector<int> &effectorIDs_, const vector<double> &increments_);
    ~IncTar() {}
    
    virtual void execute(BodyModel *bodyModel);
    
    virtual shared_ptr<Macro> getReflection(BodyModel *bodyModel);

    virtual void display();
};

class SetTar : public Macro{

  private:
  protected:

  // These two vectors define a mapping between effector->increment
  vector<int> effectorIDs;
  vector<double> targetAngles;
//    int effID;
//    double increment;

  public:

    SetTar(const vector<int> &effectorIDs_, const vector<double> &targetAngles_);
    ~SetTar() {}
    
    virtual void execute(BodyModel *bodyModel);
    
    virtual shared_ptr<Macro> getReflection(BodyModel *bodyModel);

    virtual void display();
};

class Reset : public Macro{

  private:
  protected:
    vector<int> components;
  public:

    Reset(const std::vector<int>& components_);
    ~Reset() {}
    
    virtual void execute(BodyModel *bodyModel);
    
    virtual shared_ptr<Macro> getReflection(BodyModel *bodyModel);

    virtual void display();
};

class SetScale : public Macro{

  private:
  protected:

  // These two vectors define a mapping between effector->increment
  vector<int> effectorIDs;
  vector<double> targetScales;
//    int effID;
//    double increment;

  public:

    SetScale(const vector<int> &effectorIDs_, const vector<double> &targetScales_);
    ~SetScale() {}
    
    virtual void execute(BodyModel *bodyModel);
    
    virtual shared_ptr<Macro> getReflection(BodyModel *bodyModel);

    virtual void display();
};



/////////////////////////////////////////////////////
// KeyFrames
/////////////////////////////////////////////////////

class KeyFrame{

  private:
  protected:
    vector< shared_ptr<Macro> > macros;

    bool toWaitTime; // Default: false.
    double waitTime; // Default: 0.

    bool toWaitTargets; // Default: false.
    double maxWaitTime; // Default: INF.

  public:

    KeyFrame();
    ~KeyFrame() {};

    void appendMacro(shared_ptr<Macro> macro);
    void execute(BodyModel *bodyModel); // Mainly a set.

    // Setting values to specific data members.
    // We want to be able to manually set any subset of them.
    void setToWaitTime( bool value );
    void setWaitTime( double value );
    void setToWaitTargets( bool value );
    void setMaxWaitTime( double value );
    
    bool done(BodyModel *bodyModel, const double &setTime, const double &currentTime);
    shared_ptr<KeyFrame> getReflection(BodyModel *bodyModel);

    void display();

};



///////////////////////////////////////////////////////
// Skill 
// TODO: remove virtual from here because shouldn't inherit from it (e.g. remove kickSkill)
///////////////////////////////////////////////////////
class Skill{

  private:
  protected:
    vector< shared_ptr<KeyFrame> > keyFrames;
    int currentKeyFrame;
    bool currentKeyFrameSet;
    double currentKeyFrameSetTime;

  public:

    Skill();
    virtual ~Skill() {};

    virtual void reset();
    
    virtual bool done(BodyModel *bodyModel, const double &currentTime);
    
    virtual void execute(BodyModel *bodyModel, const double &currentTime);

    virtual void appendKeyFrame( shared_ptr<KeyFrame> keyFrame );

    virtual shared_ptr<Skill> getReflection(BodyModel *bodyModel);

    virtual void display();
};



//////////////////////////////////////////////////////////////
// TODO: Temporary class, until we properly define the kick skill
//////////////////////////////////////////////////////////////
class KickSkill : public Skill {
  
    int skillState;
    double kickTimeWait;
    WorldModel* worldModel;
    int& kickLeg;
    VecPosition& ball;
    VecPosition& kickDirection;
    int& kickType;

  public:
    KickSkill( WorldModel* worldModel_, int& kickLeg_, VecPosition& ball_, VecPosition& kickDirection_, int& kickType_ );
    virtual ~KickSkill() {}

    virtual void reset();
    
    virtual bool done(BodyModel *bodyModel, const double &currentTime);
    
    virtual void execute(BodyModel *bodyModel, const double &currentTime);

    virtual void display();
    
};


#endif

