#include "PFLocalization.h"
#include <iostream>
#include "MyRandom.h"
PFLocalization::PFLocalization(WorldModel* worldModel_) : 
  m_boundary( GRASS ) {
  printf("Creating Particle Filter Localization system\n");

//  memory = NULL;
//  current = NULL;
  worldModel = worldModel_;


  partParams.NUM_PARTICLES=1000;

  // TODO: sample every cycle - does it work?
  partParams.RESAMPLE_FREQ = 1; // 20; //20; // --5 --15;
  partParams.RESEED_FREQ = 15; //15; //  
  partParams.DELTA_DIST = 150; // --75;
  partParams.DELTA_ANG = DEG_T_RAD * 30; // --15;
  partParams.DEGRADE_FACTOR = 0.99; // --0.99

  partParams.RESET_PROB_EVERY = true; //

  partParams.RANDOM_PROB = 0.05; //
  partParams.NUM_RANDOM = 10; //
  partParams.RANDOM_FREQ = 15; //

  // history
  partParams.OBSERVATION_LENGTH = 30; //15; //60; //
  partParams.MAX_HIST_DIST = 150; //
  partParams.MAX_HIST_BEAR = DEG_T_RAD * 10.0; //

  // reseed
  partParams.OLDEST_RESEED_FRAME = 30; //
  partParams.MIN_RESEED_PROB = 0.8; //
  partParams.MAX_RESEED_PROB = 0.9;
  partParams.MAX_RESEED_PARTICLES = 5; //
  partParams.MIN_RESEED_BEARING_DIFF = DEG_T_RAD * 0.75;// --2; //
  partParams.RESEED_PROB_BOOST = 0.0;
  partParams.DO_ONE_LM_RESEED = true;
  partParams.NUM_ONE_LM_PARTS = 3;
  partParams.USE_INT_FOR_RESEED = false;

  // flags
  partParams.LOC_DEBUG = false; // --true;
  
  partParams.USE_LANDMARKS = true; //
  partParams.USE_LINES = true; // --true --false --true;
  partParams.USE_INTERSECTIONS = true; // --false; --true;
  partParams.USE_GOALPOSTS = true; //
  partParams.USE_DISTANCES = true; //
  partParams.USE_BEARINGS = true; //
  partParams.USE_NEGATIVE_INFO = false; //

  partParams.USE_RANDOM = false; //
  partParams.USE_CIRCLE = true;

  // sigma's
//  partParams.BEACON_DIST_SIGMA = 12500; //
//  partParams.BEACON_BEAR_SIGMA = 10; //50; // // 10; //500;
  partParams.GOAL_DIST_SIGMA = 0.2; // 0.1; // 0.2; // 0.2; // 0.1; //0.2; // 0.3; //
  partParams.GOAL_BEAR_SIGMA = 0.1; // 0.5; // 0.25; // 0.5; // 0.1; // 0.2; // 0.5; // // 10; //500;
//  partParams.CLOSEGOAL_DIST_SIGMA = 51250; //
//  partParams.LINE_DIST_SIGMA = 25000; //51250; //51250; //
//  partParams.LINE_BEAR_SIGMA = 50; // //1; //50;
//  partParams.INT_DIST_SIGMA = 25000; //51250; //51250; //
//  partParams.INT_BEAR_SIGMA = 50; //500; // //10; //500;
//  partParams.CROPPED_SIGMA_FACTOR = 5.0; //
//  partParams.USE_GOALIE_HACKS = false; 

  partParams.USE_BALL = true;
//  partParams.BALL_DIST_SIGMA = 12500;
//  partParams.BALL_BEAR_SIGMA = 50;
//
//  partParams.MAX_LM_DIST = 8000;
//  partParams.MAX_INT_DIST = 1500;
//  partParams.MAX_LINE_DIST = 1000;
//
//  // max dist/angle to even consider a line/int match
//  partParams.MAX_SIM_DISTANCE = 250; //
//  partParams.MAX_SIM_BEARING = RAD_T_DEG * 25; //
//  partParams.MAX_NEGINFO_DISTANCE = 4000; //
//  partParams.MAX_NEGINFO_BEARING = RAD_T_DEG * 120; //
  partParams.NO_LINE_MATCH_PROB = 0.1; // -- dont penalize for bad int/lines obs 
  partParams.NO_INT_MATCH_PROB = 0.1; //
  partParams.MIN_LINE_PROB = 0.1; //
  partParams.MIN_INT_PROB = 0.1; //

  partParams.IGNORE_CENTER_LINES = false; //true;
  partParams.IGNORE_CENTER_DIST = 1000;
  partParams.IGNORE_CENTER_BEAR = DEG_T_RAD * 60.0;

  // negative info parameters
  partParams.NUM_MISSED_FRAMES = 5; //
  partParams.MISS_PROB = 0.2; //
  partParams.IMAGE_BUFFER = 15; //

  // ball kf
//  partParams.RESET_DISTANCE = 300; //

  partParams.WRITE_RESULT_FILE = false;

//  // num particles cannot be greater than maxparticles
//  if (partParams.NUM_PARTICLES > MAX_PARTICLES) {
//    cout << "ERROR: NUM_PARTICLES(" << partParams.NUM_PARTICLES
//	 << ") greater than MAX_PARTICLES(" << MAX_PARTICLES << ")" << endl;
//    partParams.NUM_PARTICLES = MAX_PARTICLES;
//  }
  

  particles = new Particle[partParams.NUM_PARTICLES];
  // init temp particles
  tmpParticles = new Particle[partParams.NUM_PARTICLES];
  
  // init particles
  resetParticles();
}

PFLocalization::~PFLocalization() {
  delete [] particles;
  delete [] tmpParticles;
  if (partParams.WRITE_RESULT_FILE) {
    fclose(resultFile);
  }
}

void PFLocalization::processFrame() {
//  timePassed = current->commonMem->seconds - timeLast;
  timePassed = worldModel->getTime() - timeLast;
//  timeLast = current->commonMem->seconds;
  timeLast = worldModel->getTime();

  // update particles from odometry
  updateParticlesFromOdometry();

  // update particles from observations
  updateParticlesFromObservations();

  // possibly resample
//  if ((current->frameNum+1) % partParams.RESAMPLE_FREQ == 0)
  if ((worldModel->getCycle()+1) % partParams.RESAMPLE_FREQ == 0) {
//    cerr << "resampling particles" << endl;
    resampleParticles();
  }
  // TODO: here we assumed we don't skip frameNums!!! Is it TRUE?

  // random walk
  randomWalkParticles();

  // update pose
  Point2D pos;
  AngRad ang;
  estimateRobotPose(pos, ang); // compute position
  worldModel->setMyPosition( pos );
  worldModel->setMyAngRad( ang );
//  updateWorldModelMat(pos, ang); // use position
 
  // update ball
  filterBall();
 
  // update relative object locations
  estimateRelativeObjectLocations(pos, ang);// use position
  // print particles (for debug)
  //if (current->frameNum  == 0)
  
  #ifdef ALLOW_LOC_DEBUG
  if (partParams.LOC_DEBUG)
    printParticles();
  #endif

  if (partParams.WRITE_RESULT_FILE) 
    writeResults();
}


// Gets the similarity measure based on seen and expected angles of
// the landmarks. smaller sigma means less discrimination
double PFLocalization::
getBearingSim( AngRad ang1, AngRad ang2, double sigma ) {

	double normalizedDeviation = ( ang1 - ang2 ) / sigma;

  double gaussExponent = -0.5 * ( normalizedDeviation * normalizedDeviation );

  return std::exp( gaussExponent );

}


// Gets the similarity measure based on seen and expected distances to
// two objects. smaller sigma means less discrimination
double PFLocalization::
getDistanceSim( double expectedDistance, double observedDistance, 
                double sigma ) {

	double normalizedDeviation = ( expectedDistance - observedDistance ) / sigma;

 	double gaussExponent = -0.5 * ( normalizedDeviation * normalizedDeviation );
	
  return std::exp( gaussExponent );

}

/** Set particle probabilities to one and place randomly */
void PFLocalization::resetParticles(){

  for (int i = 0; i < partParams.NUM_PARTICLES; i++){
    Particle *part = &particles[i];
    part->setProbability(1);
    part->placeRandomly(m_boundary);
  }

}



/** reset particles for keeper placement */
/*void PFLocalization::resetKickoffParticles(){

  for (int i = 0; i < partParams.NUM_PARTICLES; i++){
    Particle *part = &particles[i];

    
    part->setProbability(0.01);
    // put in kickoff position
    part->M_pos = Point2D(-600.0, 0.0);
    part->M_ang = DEG_T_RAD * 0.0;
    
    part->randomWalk( partParams.DELTA_DIST, partParams.DELTA_ANG, m_boundary );    
    part->setProbability(0.99);

  }
}*/


void PFLocalization::setParticleProbabilities(double newProb){
  for (int i = 0; i < partParams.NUM_PARTICLES; i++){
    Particle *part = &particles[i];
    part->setProbability(newProb);
  }
}

// TODO: implement this function
void PFLocalization::updateParticlesFromOdometry(){
/*
  // get odometry update from somewhere in snapshot
  Vector2D disp = current->motionMem->displacement * timePassed;
  AngRad angDisp = current->motionMem->angDisplacement * timePassed;

  // update each particle with it
  // and degrade probabilities on motion
  if (disp != 0 || angDisp != 0){
    
    #ifdef ALLOW_LOC_DEBUG
    if (partParams.LOC_DEBUG)
      memory->log(40, "odom != 0, (%5.0f, %5.0f), a: %5.0f", disp.x, disp.y, 
		  Rad2Deg(angDisp));
    #endif
    
    for (int i = 0; i < partParams.NUM_PARTICLES; i++){
      Particle *part = &(current->localizationMem->particles[i]);
      part->moveRelative(disp, angDisp, m_boundary);
      part->degradeProbability( partParams.DEGRADE_FACTOR );
    }
  }
  */
//  cerr << "CURRENTLY NO UPDATE FROM ODOMETRY!! THE CODE MIGHT NOT BE WORKING...\n";
}

void PFLocalization::updateParticlesFromObservations(){
  for (int i = 0; i < partParams.NUM_PARTICLES; i++){
    
    #ifdef ALLOW_LOC_DEBUG
    if (partParams.LOC_DEBUG) {
//      memory->log(85, "Update Particle %d", i);
//      memory->log(90, "((%5.0f, %5.0f), %5.0f, %5.2f)",  
//      memory->log(85, "Update Particle %d", i);
//      memory->log(90, "((%5.0f, %5.0f), %5.0f, %5.2f)",  
//		  current->localizationMem->particles[i].M_pos.x, 
//		  current->localizationMem->particles[i].M_pos.y,
//		  Rad2Deg(current->localizationMem->particles[i].M_ang),
//		  current->localizationMem->particles[i].M_prob);
    }
    #endif
    //cout << "updating particle: " << i << endl;
    updateParticleFromObservations( &particles[i]);
  }
}
  
// update particle probabilities from observations
void PFLocalization::updateParticleFromObservations(Particle *part){

  double newProb = 1.0;

  // observations of distinct landmarks
  if (partParams.USE_LANDMARKS)
    newProb *= updateParticleFromLandmarks(part);

//  // unknown posts
//  if (partParams.USE_LANDMARKS && partParams.USE_GOALPOSTS)
//    newProb *= updateParticleFromUnknownGoalposts(part);

//  // line int observations
//  if (partParams.USE_INTERSECTIONS)
//    newProb *= updateParticleFromIntersections(part);
//	
//	// lines
//	if (partParams.USE_LINES)
//		newProb *= updateParticleFromLines(part);

  // update particle probability 
  #ifdef ALLOW_LOC_DEBUG
  if (partParams.LOC_DEBUG) 
    cerr << "-I-  Updating particle with prob " << newProb;
  #endif
  part->degradeProbability( newProb );

}

/*
 * Daniel added: HELPER FUNCTIONS FOR LOCALIZATION
 */

double PFLocalization::getProbKnownObjects(	const vector<int>& objIndices,
														const Point2D& partLoc, const AngRad& partOrient, 
														double bearSigma, double distSigma ) {
  double newProb = 1.0;

  vector<int>::const_iterator begin( objIndices.begin() ), 
                              end( objIndices.end() );
  for( ; begin != end; ++begin ){
    int i = *begin;
		const WorldObject* currentObj = worldModel->getWorldObject( i );
		if ( currentObj->currentlySeen ) {
      Point2D currentObjLoc( currentObj->pos.getX(), currentObj->pos.getY() );
			double expectedDistance = partLoc.getDistanceTo(currentObjLoc);
			double expectedBearing = partLoc.getBearingTo(currentObjLoc, partOrient);
			double visionDistance = currentObj->vision.polar.getX(); // x is r, distance
			double visionBearing = Deg2Rad( currentObj->vision.polar.getY() );
      // TODO: add phi? (vertical angle)
			newProb *= getBearingSim( expectedBearing, visionBearing, bearSigma );
			newProb *= getDistanceSim( expectedDistance, visionDistance, distSigma );
		}
  } // lm loop
	return newProb;
}

//double PFLocalization::getProbUnknownObjects( int UNKNOWN_START_INDEX, 
//																							int UNKNOWN_END_INDEX, 
//																							int KNOWN_START_INDEX,
//																							int KNOWN_END_INDEX,
//																							const Point2D& partLoc,
//																							const AngRad& partOrient,
//																							double bearSigma, double distSigma ) {
//	double newProb = 1;
//  WorldObject *worldObjects = &(current->commonMem->worldObjects[0]);
//
//  for(int i = UNKNOWN_START_INDEX; i <= UNKNOWN_END_INDEX; ++i){
//		// Get vision data for unknown seen object
//		const WorldObject& unknownObj = worldObjects[i];
//		if( !unknownObj.seen )
//			continue;
//		//cout << "Updating from unknown object " << i << endl;
//		double visionDistance = unknownObj.visionDistance;
//		double visionBearing = unknownObj.visionBearing;
//		
//		// compute probability to each known object, save the max
//		double maxProb = 0;
//		int maxObj = -1;
//		for( int j = KNOWN_START_INDEX; j <= KNOWN_END_INDEX; ++j ){
//			const WorldObject& currentObj = worldObjects[j];
//			double expectedDistance = partLoc.getDistanceTo( currentObj.loc );
//			double expectedBearing = partLoc.getBearingTo( currentObj.loc, partOrient );
//			double prob = getBearingSim( expectedBearing, visionBearing, bearSigma );
//			prob *= getDistanceSim( expectedDistance, visionDistance, distSigma );
//			if ( prob > maxProb ) {
//				maxProb = prob;
//				maxObj = j;
//			}
//		}
//		// found max prob, update the total prob
//		newProb *= maxProb;
//  } // lm loop
//	
//	return newProb;
//}



/*
 * LANDMARKS
 */

double PFLocalization::updateParticleFromLandmarks(Particle *part){
 
  // update particle from vision of distinct landmarks (blue goal, blue left post, blue right post, and same for yellow)
  double newProb = 1.0;
  
  Point2D partLoc = part->M_pos;
  AngRad partOrient = part->M_ang;
	
  vector<int> objIndices;
	objIndices.push_back( FLAG_1_L );
	objIndices.push_back( FLAG_1_R );
	objIndices.push_back( FLAG_2_L );
	objIndices.push_back( FLAG_2_R );
	objIndices.push_back( GOALPOST_1_L );
	objIndices.push_back( GOALPOST_1_R );
	objIndices.push_back( GOALPOST_2_L );
	objIndices.push_back( GOALPOST_2_R );

  newProb *= getProbKnownObjects( objIndices,
																	partLoc, partOrient,
																	partParams.GOAL_BEAR_SIGMA,
																	partParams.GOAL_DIST_SIGMA );

  return newProb;
}

/*
 * UNKNOWN GOAL POSTS
 */

//double PFLocalization::updateParticleFromUnknownGoalposts(Particle *part){
//
//  // update particle from vision of unknown goal post (don't know if its left or right post)
//  double newProb = 1.0;
//
//  // TODO: update based on seen goal posts
//  Point2D partLoc = part->M_pos;
//  AngRad partOrient = part->M_ang;
//
//	newProb *= getProbUnknownObjects( WO_BLUE_GOALPOST_UNKNOWN_1, WO_BLUE_GOALPOST_UNKNOWN_2,
//																		WO_BLUE_LEFT_GOALPOST, WO_BLUE_RIGHT_GOALPOST,
//																		partLoc, partOrient,
//																		partParams.GOAL_BEAR_SIGMA, 
//																		partParams.GOAL_DIST_SIGMA );
//	newProb *= getProbUnknownObjects( WO_YELLOW_GOALPOST_UNKNOWN_1, WO_YELLOW_GOALPOST_UNKNOWN_2,
//																		WO_YELLOW_LEFT_GOALPOST, WO_YELLOW_RIGHT_GOALPOST,
//																		partLoc, partOrient,
//																		partParams.GOAL_BEAR_SIGMA, 
//																		partParams.GOAL_DIST_SIGMA );
//
//  return newProb;
//}
//


/*
 * INTERSECTIONS
 */


// deal with T and L ints in different ways
//double PFLocalization::updateParticleFromIntersections(Particle *part){
// 
//  // update particle from vision of line intersection (don't know which int it is)
//  // note that in vision we try to determine if its an L or a T, but that may not be right, so you may just want to try all of them
//
//  double newProb = 1.0;
//  
//  Point2D partLoc = part->M_pos;
//  AngRad partOrient = part->M_ang;
//
//	newProb *= getProbKnownObjects( WO_YELLOW_FIELD_LEFT_L, 
//																	WO_BLUE_PEN_LEFT_T,
//																	partLoc, partOrient,
//																	partParams.INT_BEAR_SIGMA,
//																	partParams.INT_DIST_SIGMA );
//	newProb *= getProbUnknownObjects( WO_UNKNOWN_L_1, 
//																		WO_UNKNOWN_T_2, 
//																		WO_YELLOW_FIELD_LEFT_L,
//																		WO_BLUE_PEN_LEFT_T,
//																		partLoc,
//																		partOrient,
//																		partParams.INT_BEAR_SIGMA,
//																		partParams.INT_DIST_SIGMA );
//																		
//	
//  return newProb;
//}
//
//// deal with lines
//double PFLocalization::updateParticleFromLines(Particle *part){
// 
//  // update particle from vision of line 
//
//  double newProb = 1.0;
//  
//  Point2D partLoc = part->M_pos;
//  AngRad partOrient = part->M_ang;
//
//	newProb *= getProbKnownObjects( WO_YELLOW_GOAL_LINE, 
//																	WO_BOTTOM_SIDE_LINE,
//																	partLoc, partOrient,
//																	partParams.LINE_BEAR_SIGMA,
//																	partParams.LINE_DIST_SIGMA );
//	newProb *= getProbUnknownObjects( WO_UNKNOWN_FIELD_LINE_1, 
//																		WO_UNKNOWN_FIELD_LINE_4, 
//																		WO_YELLOW_GOAL_LINE,
//																		WO_BOTTOM_SIDE_LINE,
//																		partLoc,
//																		partOrient,
//																		partParams.LINE_BEAR_SIGMA,
//																		partParams.LINE_DIST_SIGMA );
//																		
//	
//  return newProb;
//}

/** Resample particles */
void PFLocalization::resampleParticles(){
  

  
	// prepare the array of probabilities
	double particleProbabilities[partParams.NUM_PARTICLES];
	double probSum = 0;
	for (int i = 0; i < partParams.NUM_PARTICLES; i++){
    Particle *part = &particles[i];
    double p = part->getProbability();
		particleProbabilities[i] = p;
		probSum += p;
		//cerr << "prob " << p << endl;
  }
//  // TODO: remove: DEBUG printing
//  double meanX = 0;
//  double meanY = 0;
//  double meanAng = 0;
//	for (int i = 0; i < partParams.NUM_PARTICLES; i++){
//    Particle *part = &particles[i];
//    double p = part->getProbability() / probSum;
//    double x = part->getPosition().getX();
//    double y = part->getPosition().getY();
//    double ang = part->getAngle();
//    meanX += p * x;
//    meanY += p * y; 
//    meanAng += p * ang;
//  }
//
//  double sigmaX = 0;
//  double sigmaY = 0;
//  double sigmaAng = 0;
//	for (int i = 0; i < partParams.NUM_PARTICLES; i++){
//    Particle *part = &particles[i];
//    double p = part->getProbability() / probSum;
//    double x = part->getPosition().getX();
//    double y = part->getPosition().getY();
//    double ang = part->getAngle();
//    sigmaX += p * (x - meanX) * (x - meanX);
//    sigmaY += p * (y - meanY) * (y - meanY); 
//    sigmaAng += p * (ang - meanAng) * (ang - meanAng);
//  }
//  sigmaX = sqrt( sigmaX );
//  sigmaY = sqrt( sigmaY );
//  sigmaAng = sqrt( sigmaAng );
//  cerr << "sigmaX=" << sigmaX << " sigmaY=" << sigmaY << " sigmaAng=" << sigmaAng << endl;
//  // END DEBUG PRINT
	for (int i = 0; i < partParams.NUM_PARTICLES; ++i) {
		particleProbabilities[i] /= probSum;
	} 
	// build cumulative probabilities array
	double cumPartProb[partParams.NUM_PARTICLES];
	double prev = 0;
	for (int i = 0; i < partParams.NUM_PARTICLES; ++i) {
		cumPartProb[i] = prev + particleProbabilities[i];
		prev = cumPartProb[i];
	}
	// resample
	for (int i = 0; i < partParams.NUM_PARTICLES; ++i) {
		int index = sampleIndexFromProbArray( cumPartProb );
		tmpParticles[i] = particles[index];
	}
	// Random particles (like simple reseed)
	for (int i = 0; i < partParams.RANDOM_PROB * partParams.NUM_PARTICLES; ++i) {
		int index = sampleIndexFromProbArray( cumPartProb );
		tmpParticles[index].placeRandomly(m_boundary);
	}

  // copy new array to original array
  memcpy(particles, tmpParticles, partParams.NUM_PARTICLES * sizeof(Particle));
  
  setParticleProbabilities(0.99);

}

int PFLocalization::sampleIndexFromProbArray( double cumProbArray[] ) {
	double sample = range_random(0.0, 1.0);
	for( int i = 0; i < partParams.NUM_PARTICLES; ++i ) {
		if (cumProbArray[i] > sample) 
			return i;
	}
	cerr << "-W- FAILED TO SAMPLE CORRECTLY" << endl;	
	// probably the last index - numerical error?
	return partParams.NUM_PARTICLES - 1;
}





/** Take weighted average of particles (with SD)
    and set as global location to the sent params */
void PFLocalization::estimateRobotPose(Point2D& pos, AngRad& ang){

  // init vars
  Point2D w_possum;
  w_possum.setPoint(0, 0);
  Vector2D w_ang;
  Point2D sum;
  Point2D sum_sqr;
  sum.setPoint(0, 0);
  sum_sqr.setPoint(0, 0);
  
  AngRad angsum = 0.0;
  AngRad angdiff_sqr = 0.0;
  

  // calculate average and sd's
  double probval = 0.0;
  for( int i = 0; i < partParams.NUM_PARTICLES; i++ ) {
    Particle *part = &particles[i];
    w_possum += part->getPosition() * part->getProbability();
    probval += part->getProbability();
    w_ang += Vector2D( part->getProbability(), 
		       part->getAngle(), POLAR );

    sum += part->getPosition();
    sum_sqr += part->getPosition() * part->getPosition();

    angsum += part->getAngle();
    // dont this this will work right with angles wrapping around
    //angsum_sqr += square(part->getAngle());

  }

  Point2D mean = sum / partParams.NUM_PARTICLES;
  Point2D var  = (sum_sqr - sum*mean) / partParams.NUM_PARTICLES;  
  Point2D sd ( sqrt(var.x), sqrt(var.y));

  AngRad angmean = angsum / partParams.NUM_PARTICLES;

  // Second loop to get the squared differences of the angles
  for( int i = 0; i < partParams.NUM_PARTICLES; i++ ) {
    Particle *part = &particles[i];

    angdiff_sqr += square(normalizeAngle(part->getAngle() - angmean));
  }

  AngRad angvar = angdiff_sqr / partParams.NUM_PARTICLES;
//  AngRad angsd = sqrt(angvar);

  pos = w_possum / probval;
  ang = w_ang.getDirection();

//  cerr << "Robot estimated pose: \n" 
//       << "pos: " << pos 
//       << "ang: " << ang << endl;
//  current->commonMem->worldObjects[current->commonMem->WO_SELF].loc = ( w_possum / probval );
//  current->commonMem->worldObjects[current->commonMem->WO_SELF].orientation = w_ang.getDirection();

  // set WO_SELF sd
//  current->commonMem->worldObjects[current->commonMem->WO_SELF].sd = sd;
//  current->commonMem->worldObjects[current->commonMem->WO_SELF].sdOrientation = angsd;

}




/** For each world object, calculate relative location and 
    update its world object. */
void PFLocalization::estimateRelativeObjectLocations(Point2D& pos, AngRad& ang){

//  Point2D& myloc = pos;
//  AngRad& myorient = ang;
//
//  for (int i = 0; i < NUM_WORLD_OBJS; i++){
//    // calculate distance and bearing to each object
//    WorldObject* obj = worldModel->getWorldObject( i );
//    // TODO: currently this is 2D, like NAO. Change->3D?
//    Point2D objLoc ( obj->pos.getX(), obj->pos.getY() );
//    obj->distance = 
//      myloc.getDistanceTo(objLoc);
//    obj->bearing = 
//      myloc.getBearingTo(objLoc, myorient);
//
//  }

}


double PFLocalization::getProbabilitySum(Particle* part, int numPart){
  double probSum = 0.0;
  for (int i = 0; i < numPart; i++){
    probSum += part[i].getProbability();
  }
  return probSum;
}


void PFLocalization::printParticles(){
  
  for (int i = 0; i < partParams.NUM_PARTICLES; i++){
    Particle *part = &particles[i];
    printf ("-I- Particle %d: ((%5.0f, %5.0f), %5.0f, %5.2f)", i, part->M_pos.x, 
		part->M_pos.y, Rad2Deg(part->M_ang), part->M_prob);
  }
}

void PFLocalization::writeResults(){
//  fprintf(resultFile,"%i,%i\n",current->frameNum,partParams.NUM_PARTICLES);
//  WorldObject* wo = &current->commonMem->worldObjects[current->commonMem->WO_SELF];
//  fprintf(resultFile,"%lf,%lf,%lf\n",wo->loc.x,wo->loc.y,wo->orientation);
//  fprintf(resultFile,"%lf,%lf,%lf\n",wo->sd.x,wo->sd.y,wo->sdOrientation);
//  for (int i = 0; i < partParams.NUM_PARTICLES; i++){
//    Particle *part = &(current->localizationMem->particles[i]);     
//    fprintf(resultFile,"%i,%lf,%lf,%lf,%lf\n",i,part->M_pos.x,part->M_pos.y,part->M_ang,part->M_prob);
//  }
}


/** Update the kf for the ball */
void PFLocalization::filterBall(){
  
  // ball - time update 
  //cout << "Time passed: " << timePassed << endl;
//  ball.timeUpdate(0, timePassed);
//
//  // update the pose in the kf using pf estimate
//  ball.poseMeasurementUpdate(0);
//
//  // ball - measurement update 
//  ball.ballMeasurementUpdate(0);
//
//  // set ball kf estimate into ball world object
//  ball.updateBallFromKF(0);

}


/**
 * Comparison function for quick sort
 */
int compareProbabilities( const void *one, const void *two ) {
  Particle *a, *b;
  a = (Particle*) one;
  b = (Particle*) two;
  if( a->getProbability() >= b->getProbability() ) {
    return -1;
  }
  else {
    return 1;
  }
}


/** Perform a random walk on the particles */
void PFLocalization::randomWalkParticles(){
  for ( int i = 0; i < partParams.NUM_PARTICLES; i++ ) {
    Particle *part = &particles[i];
    part->randomWalk( partParams.DELTA_DIST, partParams.DELTA_ANG, m_boundary );    
  }
}
  
void PFLocalization::updateWorldModelMat(Point2D& pos, AngRad& ang) {
  // TODO: currently we assume that z is constant
  const double myX = pos.getX();
  const double myY = pos.getY();
  const double myZ = 0.5;
  const double myOrient = Rad2Deg( ang );
  
  VecPosition globalPlusPlus, globalPlusMinus, globalMinusPlus, globalMinusMinus;
  if (worldModel->getSide() == SIDE_LEFT ) {
    globalPlusPlus = worldModel->getWorldObject( FLAG_1_R )->pos;
    globalPlusMinus = worldModel->getWorldObject(FLAG_2_R)->pos;
    globalMinusPlus = worldModel->getWorldObject(FLAG_1_L)->pos;
    globalMinusMinus = worldModel->getWorldObject(FLAG_2_L)->pos;
  } else { // SIDE_RIGHT
    globalPlusPlus = worldModel->getWorldObject(FLAG_2_L)->pos;
    globalPlusMinus = worldModel->getWorldObject(FLAG_1_L)->pos;
    globalMinusPlus = worldModel->getWorldObject(FLAG_2_R)->pos;
    globalMinusMinus = worldModel->getWorldObject(FLAG_1_R)->pos;
  }


  // Compute local coordinates of corners
  VecPosition fieldXPlusYPlus( globalPlusPlus.getX() - myX,
                               globalPlusPlus.getY() - myY,
                               0 - myZ );
  fieldXPlusYPlus = fieldXPlusYPlus.rotateAboutZ( myOrient );                          

  VecPosition fieldXPlusYMinus(  globalPlusMinus.getX() - myX,
                                 globalPlusMinus.getY() - myY,
                                 0 - myZ );
  fieldXPlusYMinus = fieldXPlusYMinus.rotateAboutZ( myOrient );                               

  VecPosition fieldXMinusYPlus(  globalMinusPlus.getX() - myX,
                                 globalMinusPlus.getY() - myY,
                                 0 - myZ );
  fieldXMinusYPlus = fieldXMinusYPlus.rotateAboutZ( myOrient );

  VecPosition fieldXMinusYMinus( globalMinusMinus.getX() - myX,
                                 globalMinusMinus.getY() - myY,
                                  0 - myZ );
  fieldXMinusYMinus = fieldXMinusYMinus.rotateAboutZ( myOrient );                            

  worldModel->updateMatricesAndMovingObjs( fieldXPlusYPlus, fieldXPlusYMinus,
                                        fieldXMinusYPlus, fieldXMinusYMinus );
//  cerr << "fieldXPlusYPlus " << fieldXPlusYPlus 
//<<  " fieldXPlusYMinus " << fieldXPlusYMinus 
//<< " fieldXMinusYPlus " << fieldXMinusYPlus 
//<< " fieldXMinusYMinus " << fieldXMinusYMinus << endl;
return ;
                               
}
