#ifndef _PARTICLE_H
#define _PARTICLE_H

#include "Geometry.h"
#include "WorldObject.h"


class Particle 
{
 public : // protected my arse !
  double M_prob;
  Point2D M_pos;
  AngRad M_ang;

  // data to keep track of expected obs/ neg info
  int M_expectedObs [NUM_WORLD_OBJS];
  int M_seenObs [NUM_WORLD_OBJS];

 public:
  Particle();

  Particle( double prob,
	    Point2D pos,
	    AngRad ang,
	    Rectangle boundary );
  

  double getProbability();
  Point2D getPosition();
  AngRad getAngle();

  void setProbability( double prob );
  void degradeProbability( double factor );
  bool setPosition( Point2D pos, Rectangle boundary );
  void setAngle( AngRad ang );

  void setParticle( double prob, Point2D pos, AngRad ang,
		    Rectangle boundary );


  void moveRelative( Vector2D relativeDisplacement,
		     AngRad rotation, Rectangle boundary );

  void moveGlobal( Vector2D globalDisplacement,
		   AngRad rotation, Rectangle boundary );

  void randomWalk( double maxDistance, AngRad maxRotation,
		   Rectangle boundary );

  void placeRandomly( Rectangle boundary );

  // for lines
  Point2D getNearestPointOnLine( Line2D l );
  double getDistanceToPoint( Point2D p );
  AngRad getBearingToPoint( Point2D p );

  friend ostream& operator << ( ostream &os,
				Particle p );

  // functions for expected observations / neg info 
  void resetObservations();
  int getNumFramesExpected(int i);
  int getNumFramesNotSeen(int i);
  void incrNumFramesExpected(int i);
  void incrNumFramesNotSeen(int i);
  void incrNumFramesNotSeenForAll();
  void resetNumFramesExpected(int i);
  void resetNumFramesNotSeen(int i);
  

};


#endif
