#ifndef HEADERS_H
#define HEADERS_H

#include <boost/assign/list_of.hpp> // for 'map_list_of()'
using boost::assign::map_list_of;      // bring 'map_list_of()' into scope
#include <map>

// One of these should be defined
#define RESTRICTED_VISION
//#define PERFECT_VISION

//Hinge Joints

#define HJ_H1  0
#define HJ_H2  1

#define HJ_LA1 2
#define HJ_LA2 3
#define HJ_LA3 4
#define HJ_LA4 5

#define HJ_RA1 6
#define HJ_RA2 7
#define HJ_RA3 8
#define HJ_RA4 9

#define HJ_LL1 10
#define HJ_LL2 11
#define HJ_LL3 12
#define HJ_LL4 13
#define HJ_LL5 14
#define HJ_LL6 15

#define HJ_RL1 16
#define HJ_RL2 17
#define HJ_RL3 18
#define HJ_RL4 19
#define HJ_RL5 20
#define HJ_RL6 21

#define HJ_NUM 22


//Effectors

#define EFF_H1  0
#define EFF_H2  1

#define EFF_LA1 2
#define EFF_LA2 3
#define EFF_LA3 4
#define EFF_LA4 5

#define EFF_RA1 6
#define EFF_RA2 7
#define EFF_RA3 8
#define EFF_RA4 9

#define EFF_LL1 10
#define EFF_LL2 11
#define EFF_LL3 12
#define EFF_LL4 13
#define EFF_LL5 14
#define EFF_LL6 15

#define EFF_RL1 16
#define EFF_RL2 17
#define EFF_RL3 18
#define EFF_RL4 19
#define EFF_RL5 20
#define EFF_RL6 21

#define EFF_NUM 22


//Flags
//#define FLAG_1_L  0
//#define FLAG_1_R  1
//#define FLAG_2_L  2
//#define FLAG_2_R  3
#define FLAG_NUM  4

//Goal Posts
//#define GOALPOST_1_L  0
//#define GOALPOST_1_R  1
//#define GOALPOST_2_L  2
//#define GOALPOST_2_R  3
#define GOALPOST_NUM  4

//Team Playing Side
#define SIDE_LEFT    0
#define SIDE_RIGHT   1

#define TEAMMATE_NUM 11
#define OPPONENT_NUM 11


//Body Components
#define COMP_TORSO     0

#define COMP_NECK      1 
#define COMP_HEAD      2  

#define COMP_LSHOULDER 3
#define COMP_LUPPERARM 4
#define COMP_LELBOW    5
#define COMP_LLOWERARM 6

#define COMP_RSHOULDER 7
#define COMP_RUPPERARM 8
#define COMP_RELBOW    9 
#define COMP_RLOWERARM 10

#define COMP_LHIP1     11 
#define COMP_LHIP2     12 
#define COMP_LTHIGH    13
#define COMP_LSHANK    14 
#define COMP_LANKLE    15
#define COMP_LFOOT     16

#define COMP_RHIP1     17
#define COMP_RHIP2     18
#define COMP_RTHIGH    19
#define COMP_RSHANK    20
#define COMP_RANKLE    21
#define COMP_RFOOT     22

#define COMP_NUM       23


//INDICES for arms and legs
#define ARM_LEFT 0
#define ARM_RIGHT 1
#define ARM_NUM 2

#define LEG_LEFT 0
#define LEG_RIGHT 1
#define LEG_NUM 2

// Floating point comparison tolerance
#define EPSILON 0.0001  


//HCT Matrix Creation modes
#define HCT_IDENTITY            0
#define HCT_ROTATE_X            1
#define HCT_ROTATE_Y            2
#define HCT_ROTATE_Z            3 
#define HCT_GENERALIZED_ROTATE  4
#define HCT_TRANSLATE           5

//Play Modes
#define PM_BEFORE_KICK_OFF 0
#define PM_KICK_OFF_LEFT   1
#define PM_KICK_OFF_RIGHT  2 
#define PM_PLAY_ON         3 
#define PM_KICK_IN_LEFT    4
#define PM_KICK_IN_RIGHT   5
#define PM_GOAL_LEFT       6
#define PM_GOAL_RIGHT      7
#define PM_GAME_OVER       8

//Extra added.
#define PM_CORNER_KICK_LEFT   9
#define PM_CORNER_KICK_RIGHT  10
#define PM_GOAL_KICK_LEFT     11
#define PM_GOAL_KICK_RIGHT    12
#define PM_OFFSIDE_LEFT       13
#define PM_OFFSIDE_RIGHT      14
#define PM_FREE_KICK_LEFT     15
#define PM_FREE_KICK_RIGHT    16

//Directions
#define DIR_LEFT  0
#define DIR_RIGHT 1

//Skills
enum SkillType {
 SKILL_WALK_FRONT,
 SKILL_WALK_BACK, 
 SKILL_WALK_LEFT,
 SKILL_WALK_RIGHT,
 SKILL_TURN_LEFT,
 SKILL_TURN_RIGHT,
 SKILL_KICK_FRONT,
 SKILL_STAND,
 SKILL_FIND_BALL_LEFT,
 SKILL_FIND_BALL_RIGHT
};

static std::map<int,const char*> SkillType2Str = map_list_of
( SKILL_WALK_FRONT, "SKILL_WALK_FRONT")
(SKILL_WALK_BACK, "SKILL_WALK_BACK")
(SKILL_WALK_LEFT, "SKILL_WALK_LEFT")
(SKILL_WALK_RIGHT, "SKILL_WALK_RIGHT")
(SKILL_TURN_LEFT, "SKILL_TURN_LEFT")
(SKILL_TURN_RIGHT, "SKILL_TURN_RIGHT")
(SKILL_KICK_FRONT, "SKILL_KICK_FRONT")
(SKILL_STAND, "SKILL_STAND")
(SKILL_FIND_BALL_LEFT,"SKILL_FIND_BALL_LEFT")
(SKILL_FIND_BALL_RIGHT,"SKILL_FIND_BALL_RIGHT")
;

//Kick Parameters
#define KICK_LONG    0
#define KICK_SHORT   1
#define KICK_DRIBBLE 2



#endif // HEADERS_H

