#include "naobehavior.h"
#include <iostream>
#include <sstream>
#include <cmath>
#include <map>
extern map<string, float> namedParams;

NaoBehavior::NaoBehavior(const std::string teamName, int uNum){

  cout << "Constructing of Nao Behavior" << endl;
  mInit = false;
  initBeamed = false;

  agentTeamName = teamName;
  agentUNum = uNum;
  
  worldModel = new WorldModel();
  bodyModel = new BodyModel(worldModel);
  particleFilter = new PFLocalization( worldModel );
  parser = new Parser(worldModel, bodyModel, teamName, particleFilter);

  initBeamed = false;
  initialized = false;

  fallState = 0;
  fallenLeft = false;
  fallenRight = false;
  fallenUpRollLeft = false;
  fallenUpRollRight = false;
  fallenDown = false;

  resetSkills();
}

NaoBehavior::~NaoBehavior(){

  delete parser;
  delete worldModel;
  delete bodyModel;
}

string NaoBehavior::Init(){

  return "(scene rsg/agent/nao/nao.rsg)";
}

string NaoBehavior::Think(const std::string& message){

//  cout << "(NaoBehavior) received message " << message << endl;
    

    bool parseSuccess = parser->parse(message);
    if(!parseSuccess && (worldModel->getPlayMode() != PM_BEFORE_KICK_OFF)){
      cout << "****************************************\n";
      cout << "Could not parse message: " << message << "\n";
      cout << "****************************************\n";
    }
    
    //  cout << "\nparseSuccess: " << parseSuccess << "\n";
  // worldModel->display();
  worldModel->quickDisplay();
    bodyModel->refresh();
    //  bodyModel->display();
    //  bodyModel->displayDerived();
    


  string action;

  if (!mInit){
    
    mInit = true;
    stringstream ss;
    ss << "(init (unum " << agentUNum << ")(teamname " << agentTeamName << "))";
    action = ss.str();
    return action;
  }

  if(!initialized){
    
    if(!worldModel->getUNumSet() || !worldModel->getSideSet()){
      //      cout << "UNum and side not received yet.\n";
      action = "";
      return action;
    }

    if(!initBeamed){

      initBeamed = true;

      double beamX, beamY, beamAngle;

      //1: Goalie.
      if(worldModel->getUNum() == 1){
	beamX = -HALF_FIELD_X + .5;
	beamY = 0;
	beamAngle = 0;
      }
      else if(worldModel->getUNum() == 2){
        // This is the agent we're optimizing, start it at a known position:
        beamX = -8.5;
        beamY = -3.00;
        beamAngle = 0;
      }
      else if(worldModel->getUNum() == 3){
	//	beamX = -0.5;
	//	beamY = 1.5;
	//	beamAngle = 0;

	beamX = -2.2;
	beamY = 0.05;
	beamAngle = 0;
      }
      else if(worldModel->getUNum() == 4){
	beamX = -4;
	beamY = 1;
	beamAngle = 0;
      }
      else if(worldModel->getUNum() == 5){
	beamX = -4;
	beamY = -1;
	beamAngle = 0;
      }
      else if(worldModel->getUNum() == 6){
	beamX = -6;
	beamY = 0;
	beamAngle = 0;
      }
      else{
	beamX = -2.0 - ((double)(worldModel->getUNum()) / 100.0);
	beamY = (drand48() - 0.5) * ((double)(worldModel->getUNum()) / 100.0);
	beamAngle = 0;
      }

      stringstream ss;
      ss << "(beam " << beamX << " " << beamY << " " << beamAngle << ")";
      action = ss.str();
      return action;
    }
    else{

      cout << "Not Initialized.\n";
      initialized = setInitialHead();
      initialized = setInitialArm(ARM_LEFT) && initialized;
      initialized = setInitialArm(ARM_RIGHT) && initialized;
      initialized = setInitialLeg(LEG_LEFT) && initialized;
      initialized = setInitialLeg(LEG_RIGHT) && initialized;
    }
  }
  else{
    
    act();
  }

  //action = "(say Hello)(he2 1)(he1 1)";
  //checkAngles();

  action = composeAction();
  
  //std::cout << "Sending action: " << action << "\n";
  return action;
}

void NaoBehavior::act(){

  refresh();

  if(false && checkingFall()){
    resetSkills();
    return;
  }

  if(improperPlayMode()){
    resetSkills();
    skill = SKILL_STAND;
  }
  else{
    
    
    //  if(worldModel->getCycle() < 500){
    //    return;
    //  }
    
    //  skill = SKILL_KICK_FRONT;
    //  skill = SKILL_TURN_RIGHT;
    //    skill = SKILL_STAND;
    //  skill = SKILL_WALK_BACK;
    //  skill = SKILL_WALK_FAST;
    
    //  cout << "LF: " << bodyModel->getFootCG(LEG_LEFT) << "\n";
    //  cout << "RF: " << bodyModel->getFootCG(LEG_RIGHT) << "\n";
    //cout << "Ball: " << ball << "\n";
    
    if(skillState == 0){
      skill = selectSkill();
    }
    
  }

  //cerr << "Selected skill: " << SkillType2Str[skill] << endl;  


  if(skill == SKILL_WALK_FRONT){
    walk_front();
  }
  else if(skill == SKILL_WALK_BACK){
    walk_back();
  }
  else if(skill == SKILL_WALK_LEFT){
    walk_left();
  }
  else if(skill == SKILL_WALK_RIGHT){
    walk_right();
  }
  else if(skill == SKILL_TURN_LEFT){
    turn_left();
  }
  else if(skill == SKILL_TURN_RIGHT){
    turn_right();
  }
  else if(skill == SKILL_KICK_FRONT){
    kick_front();
  }
  else if(skill == SKILL_STAND){
    stand();
  }
  else if (skill == SKILL_FIND_BALL_LEFT){
    find_ball_left();
  }
  else if (skill == SKILL_FIND_BALL_RIGHT){
    find_ball_right();
  }
  worldModel->setLastSkill(skill);
}

