#ifndef NAOBEHAVIOR_H
#define NAOBEHAVIOR_H

#include "behavior.h"
#include "headers.h"
#include "parser.h"
#include "worldmodel.h"
#include "bodymodel.h"
#include "PFLocalization.h"

using namespace std;

enum TurnState {NONE, TURNING_LEFT, TURNING_RIGHT, TURNING_LEFT_FOR_BALL, TURNING_RIGHT_FOR_BALL};

class NaoBehavior : public Behavior{

 protected:
  
  std::string agentTeamName;
  int agentUNum;
  
  Parser *parser;
  WorldModel *worldModel;
  BodyModel *bodyModel;
  PFLocalization* particleFilter;
  
  bool mInit;
  bool initBeamed;
  
  bool initialized;

  TurnState currentTurnState;

  int skill;
  int skillState;

  int fallState;
  bool fallenLeft, fallenRight, fallenUpRollLeft, fallenUpRollRight, fallenDown;

  VecPosition turnTarget;
  double turnTargetAngleThreshold;
  
  int kickLeg;
  VecPosition kickDirection;
  int kickType;

  bool positioning;


  VecPosition me;
  VecPosition myXDirection, myYDirection, myZDirection;

  VecPosition ball;

  bool closestTeammateToBall;
  bool closestTeammateToGoal;

  string composeAction();
  void checkAngles();
  
  bool setInitialHead();
  bool setInitialArm(const int &arm);
  bool setInitialLeg(const int &leg);

  void resetSkills();
  void refresh();
  void act();
  int selectSkill();

  void walk_front();
  void walk_back();
  void walk_left();
  void walk_right();
  void turn_left();
  void turn_right();
  void kick_front();
  void find_ball_left();
  void find_ball_right();

  bool checkingFall();
  void riseLeft();
  void riseRight();
  void riseUpLeft();
  void riseUpRight();
  void riseDown();
  void stand();
  void walk_fast();

  int goToTarget(const VecPosition &target);
  //  int goToBall();
  int driveBallToGoal();
  int findBall();

  void getTargetDistanceAndAngle(const VecPosition &target, double &distance, double &angle);
  void setTurn(const int &direction, const VecPosition &target, const double &angleThreshold);

  bool positioningToKick(const VecPosition &direction, const double &angleThreshold, const double &lateralThreshold, const VecPosition &targetPos, int &skillToTake);

  bool improperPlayMode();
  int goalie();

 public:

  NaoBehavior(const std::string teamName, int uNum);
  virtual ~NaoBehavior();
  
  virtual std::string Init();
  virtual std::string Think(const std::string& message);

};

#endif // NAOBEHAVIOR_H

