#include "parser.h"
#include "naobehavior.h"
#include "simplelocalization.h"
#include "WorldObject.h"

using boost::shared_ptr;

///////////////////////////
//Tak care of side both halves - constructor wrong currently.
///////////////////////////

Parser::Parser(WorldModel *worldModel, BodyModel *bodyModel, const string &teamName,
               PFLocalization* particleFilter ){

  this->worldModel = worldModel;
  this->bodyModel = bodyModel;
  this->particleFilter = particleFilter;

  //Both side and UNum not necessarily correct, but stop gap initially.
  this->side = SIDE_LEFT;
  this->uNum = 1;

  this->teamName = teamName;

}

Parser::~Parser(){

}

vector<string> Parser::tokenise(const string &s){
  
  int length = s.length();
  string currentString = "";
  bool currentValid = false;

  vector<string> v;

  for(int i = 0; i < length; ++i){

    char c = s.at(i);
    if(c != '(' && c != ')' && c != ' '){
      currentString.append(1, c);
    }
    else{

      if(currentString.length() > 0){
	v.push_back(currentString);
	currentString = "";
      }
    }
  }

  /*
    cout << "Tokens: ";
    for(int i = 0; i < v.size(); ++i){
    cout << v[i] << " ";
    }
    cout << "\n";
  */

  return v;
}

bool Parser::parseTime(const string &str){

  bool valid = false;
  double time = 0;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size() - 1; ++i){
    if(!(tokens[i].compare("now"))){
      time = atof(tokens[i + 1].c_str());
      valid = true;
    }
  }

  if(valid){
    //    cout << "Time: " << time << "\n";
    worldModel->setTime(time);

    worldModel->incrementCycle();

  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }

  return valid;
}

bool Parser::parseGameState(const string &str){

  bool gameTimeValid = false;
  bool playModeValid = false;
  bool uNumValid = false;
  bool sideValid = false;

  double gameTime = 0;
  string playModeStr = "";

  int playMode = -1;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size() - 1; ++i){

    if(!(tokens[i].compare("t"))){
      gameTime = atof(tokens[i + 1].c_str());
      gameTimeValid = true;
      ++i;
    }
    else if(!(tokens[i].compare("pm"))){
      
      playModeStr = tokens[i + 1];
      
      
      if(!(playModeStr.compare("BeforeKickOff"))){
	playMode = PM_BEFORE_KICK_OFF;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickOff_Left"))){
	playMode = PM_KICK_OFF_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickOff_Right"))){
	playMode = PM_KICK_OFF_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PlayOn"))){
	playMode = PM_PLAY_ON;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickIn_Left"))){
	playMode = PM_KICK_IN_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("KickIn_Right"))){
	playMode = PM_KICK_IN_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("Goal_Left"))){
	playMode = PM_GOAL_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("Goal_Right"))){
	playMode = PM_GOAL_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("GameOver"))){
	playMode = PM_GAME_OVER;
	playModeValid = true;
      }
      //Extra
      else if(!(playModeStr.compare("PM_CORNER_KICK_LEFT"))){
	playMode = PM_CORNER_KICK_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PM_CORNER_KICK_RIGHT"))){
	playMode = PM_CORNER_KICK_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("goal_kick_left"))){
	playMode = PM_GOAL_KICK_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("goal_kick_right"))){
	playMode = PM_GOAL_KICK_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PM_OFFSIDE_LEFT"))){
	playMode = PM_OFFSIDE_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PM_OFFSIDE_RIGHT"))){
	playMode = PM_OFFSIDE_RIGHT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PM_FREE_KICK_LEFT"))){
	playMode = PM_FREE_KICK_LEFT;
	playModeValid = true;
      }
      else if(!(playModeStr.compare("PM_FREE_KICK_RIGHT"))){
	playMode = PM_FREE_KICK_RIGHT;
	playModeValid = true;
      }
      else{
	playModeValid = false;
	cout << "Unknown play mode: " << playModeStr << "\n";
      }
      ++i;
    }
    else if(!(tokens[i].compare("unum"))){

      uNum = atoi(tokens[i + 1].c_str());
      if(1 <= uNum && uNum <= 11){
	uNumValid = true;
      }
      ++i;
    }
    else if(!(tokens[i].compare("team"))){

      if(!(tokens[i + 1].compare("left"))){
	side = SIDE_LEFT;
	sideValid = true;
      }
      else if(!(tokens[i + 1].compare("right"))){
	side = SIDE_RIGHT;
	sideValid = true;
      }
      ++i;
    }

  }
  
  if(gameTimeValid){
    //    cout << "Game Time: " << gameTime << "\n";
    worldModel->setGameTime(gameTime);
  }

  if(playModeValid){
    //    cout << "Play Mode: " << playMode << "\n";
    worldModel->setPlayMode(playMode);
  }

  if(uNumValid){
    //    cout << "UNum: " << uNum << "\n";
    worldModel->setUNum(uNum);
    worldModel->setUNumSet(true);
  }

  if(sideValid){
    //    cout << "Side: " << side << "\n";
    worldModel->setSide(side);
    worldModel->setSideSet(true);
  }

  bool valid = gameTimeValid || playModeValid;

  //  if(!valid){
  //    cout << str << " not valid.\n";
  //  }

  return valid;
}

bool Parser::parseGyro(const string &str){
  bool valid = false;

  double rateX, rateY, rateZ;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){
    if(!tokens[i].compare("rt")){
      if(i + 3 < tokens.size()){
	
	rateX = atof(tokens[i + 1].c_str());
	rateY = atof(tokens[i + 2].c_str());
	rateZ = atof(tokens[i + 3].c_str());
	valid = true;
      }
    }

  }

  if(valid){
    //cout << "Gyro: " << rateX << " " << rateY << " " << rateZ << "\n";
    bodyModel->setGyroRates(rateX, rateY, rateZ);
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }

  return valid;
}

bool Parser::parseAccelerometer(const string &str){
  bool valid = false;

  double rateX, rateY, rateZ;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){
    if(!tokens[i].compare("a")){
      if(i + 3 < tokens.size()){
	
	rateX = atof(tokens[i + 1].c_str());
	rateY = atof(tokens[i + 2].c_str());
	rateZ = atof(tokens[i + 3].c_str());
	valid = true;
      }
    }

  }

  if(valid){
    //cout << "Accel: " << rateX << " " << rateY << " " << rateZ << "\n";
    bodyModel->setAccelRates(rateX, rateY, rateZ);
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }

  return valid;
}

//to handle -- do when needed.
bool Parser::parseHear(const string &str){

  bool valid = false;

  double hearTime;
  bool self;
  double angle;
  string message;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 4);
  
  hearTime = atof(tokens[1].c_str());
  if(!(tokens[2].compare("self"))){
    self = true;
    angle = 0;
  }
  else{
    self = false;
    angle = atof(tokens[2].c_str());
  }
  
  message = tokens[3];

  if(valid){
    //    cout << "Hear Time: " << hearTime << "\n";
    //    cout << "Self: " << self << "\n";
    //    cout << "Angle: " << angle << "\n";
    //    cout << "Message: " << message << "\n";
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }

  return valid;
}

bool Parser::parseHingeJoint(const string &str){

  bool valid;

  string name;
  double angle;

  bool validName = false;
  bool validAngle = false;

  int hingeJointIndex = -1;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){

    if(!tokens[i].compare("n")){
      if(i + 1 < tokens.size()){
	
	name = tokens[i + 1];
	
	if(!(name.compare("hj1"))){
	  hingeJointIndex = HJ_H1;
	  validName = true;
	}
	else if(!(name.compare("hj2"))){
	  hingeJointIndex = HJ_H2;
	  validName = true;
	}
	else if(!(name.compare("laj1"))){
	  hingeJointIndex = HJ_LA1;
	  validName = true;
	}
	else if(!(name.compare("laj2"))){
	  hingeJointIndex = HJ_LA2;
	  validName = true;
	}
	else if(!(name.compare("laj3"))){
	  hingeJointIndex = HJ_LA3;
	  validName = true;
	}
	else if(!(name.compare("laj4"))){
	  hingeJointIndex = HJ_LA4;
	  validName = true;
	}
	else if(!(name.compare("raj1"))){
	  hingeJointIndex = HJ_RA1;
	  validName = true;
	}
	else if(!(name.compare("raj2"))){
	  hingeJointIndex = HJ_RA2;
	  validName = true;
	}
	else if(!(name.compare("raj3"))){
	  hingeJointIndex = HJ_RA3;
	  validName = true;
	}
	else if(!(name.compare("raj4"))){
	  hingeJointIndex = HJ_RA4;
	  validName = true;
	}
	else if(!(name.compare("llj1"))){
	  hingeJointIndex = HJ_LL1;
	  validName = true;
	}
	else if(!(name.compare("llj2"))){
	  hingeJointIndex = HJ_LL2;
	  validName = true;
	}
	else if(!(name.compare("llj3"))){
	  hingeJointIndex = HJ_LL3;
	  validName = true;
	}
	else if(!(name.compare("llj4"))){
	  hingeJointIndex = HJ_LL4;
	  validName = true;
	}
	else if(!(name.compare("llj5"))){
	  hingeJointIndex = HJ_LL5;
	  validName = true;
	}
	else if(!(name.compare("llj6"))){
	  hingeJointIndex = HJ_LL6;
	  validName = true;
	}
	else if(!(name.compare("rlj1"))){
	  hingeJointIndex = HJ_RL1;
	  validName = true;
	}
	else if(!(name.compare("rlj2"))){
	  hingeJointIndex = HJ_RL2;
	  validName = true;
	}
	else if(!(name.compare("rlj3"))){
	  hingeJointIndex = HJ_RL3;
	  validName = true;
	}
	else if(!(name.compare("rlj4"))){
	  hingeJointIndex = HJ_RL4;
	  validName = true;
	}
	else if(!(name.compare("rlj5"))){
	  hingeJointIndex = HJ_RL5;
	  validName = true;
	}
	else if(!(name.compare("rlj6"))){
	  hingeJointIndex = HJ_RL6;
	  validName = true;
	}
      }
    }

    if(!tokens[i].compare("ax")){
      if(i + 1 < tokens.size()){
	
	angle = atof(tokens[i + 1].c_str());
	validAngle = true;
      }
    }

  }

  valid = validName && validAngle;

  if(valid){
    //    cout << "HJ: " << name << " " << angle << "\n";
    bodyModel->setJointAngle(hingeJointIndex, angle);
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }

  return valid;
}

bool Parser::parseSee(const string &str, vector< shared_ptr<VisionObject> >& visionObjs){

//  visionSense->teammatePolar.clear();
//  visionSense->opponentPolar.clear();
//  visionSense->ballPolar = NULL;
//  for (int i = 0; i < FLAG_NUM; i++) {
//    visionSense->flagPolar[i] = NULL;
//  }
//  for (int i = 0; i < GOALPOST_NUM; i++) {
//    visionSense->goalPostPolar[i] = NULL;
//  }

  bool valid = true;
  visionObjs.clear();

  vector<string> strSegments = segment(str, true);
  
  for(int i = 0; i < strSegments.size(); ++i){
    
    //Goalpost
    if(strSegments[i].at(1) == 'G'){
      valid = parseGoalPost(strSegments[i], visionObjs) && valid;
    }
    //Flags
    else if(strSegments[i].at(1) == 'F'){
      valid = parseFlag(strSegments[i], visionObjs) && valid;
    }
    //Ball
    else if(strSegments[i].at(1) == 'B'){
      valid = parseBall(strSegments[i], visionObjs) && valid;
    }
    //Player
    else if(strSegments[i].at(1)== 'P'){
      valid = parsePlayer(strSegments[i], visionObjs) && valid;
    }
    // MyPos
    else if(strSegments[i].at(1)== 'm'){
      valid = parseMyPos(strSegments[i]) && valid;
    }
    //See token (ignore)
    else if (strSegments[i].at(1)== 'S') {
    }
    else{
      valid = false;
    }
    
  }
  
    if(!valid){
      cout << str << " not valid.\n";
    }
  return valid;
}

bool Parser::parseGoalPost(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs){

  bool valid = false;

  string name;
  double r, theta, phi;
  
  int goalPostIndex = -1;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 5);
  
  if(valid){
    name = tokens[0];
    valid = !(tokens[1].compare("pol")) && valid;

    r = atof(tokens[2].c_str());
    theta = atof(tokens[3].c_str());
    phi = atof(tokens[4].c_str());

    if(!(name.compare("G1L"))){
      goalPostIndex = GOALPOST_1_L;
    }
    else if(!(name.compare("G1R"))){
      goalPostIndex = GOALPOST_1_R;
    }
    else if(!(name.compare("G2L"))){
      goalPostIndex = GOALPOST_2_L;
    }
    else if(!(name.compare("G2R"))){
      goalPostIndex = GOALPOST_2_R;
    }
    else{
      valid = false;
    }
  }

  if(valid){
    //    cout << name << ": " << r << ", " << theta << ", " << phi << "\n";
    shared_ptr<VisionObject> obj( new VisionObject(r, theta, phi, goalPostIndex ) ); 
    visionObjs.push_back( obj ); 
//    visionSense->goalPostPolar[goalPostIndex] = VecPosition(r, theta, phi);
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }
  return valid;
}

bool Parser::parseFlag(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs){
  bool valid = false;

  string name;
  double r, theta, phi;

  int flagIndex = -1;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 5);
  
  if(valid){
    name = tokens[0];
    valid = !(tokens[1].compare("pol")) && valid;

    r = atof(tokens[2].c_str());
    theta = atof(tokens[3].c_str());
    phi = atof(tokens[4].c_str());

    if(!(name.compare("F1L"))){
      flagIndex = FLAG_1_L;
    }
    else if(!(name.compare("F1R"))){
      flagIndex = FLAG_1_R;
    }
    else if(!(name.compare("F2L"))){
      flagIndex = FLAG_2_L;
    }
    else if(!(name.compare("F2R"))){
      flagIndex = FLAG_2_R;
    }
    else{
      valid = false;
    }
  }

  if(valid){
    //    cout << name << ": " << r << ", " << theta << ", " << phi << "\n";
    shared_ptr<VisionObject> obj( new VisionObject(r, theta, phi, flagIndex) ); 
    visionObjs.push_back( obj ); 
//    visionSense->flagPolar[flagIndex] = VecPosition(r, theta, phi);
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }
  return valid;
}

bool Parser::parseBall(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs){
  bool valid = false;
  string name;
  double r, theta, phi;

  vector<string> tokens = tokenise(str);
  valid = (tokens.size() == 5);
  
  if(valid){
    name = tokens[0];
    valid = !(tokens[1].compare("pol")) && valid;

    r = atof(tokens[2].c_str());
    theta = atof(tokens[3].c_str());
    phi = atof(tokens[4].c_str());
  }

  if(valid){
    //    cout << name << ": " << r << ", " << theta << ", " << phi << "\n";
    shared_ptr<VisionObject> obj( new VisionObject(r, theta, phi, WO_BALL) ); 
    visionObjs.push_back( obj ); 
//    visionSense->ballPolar = VecPosition(r, theta, phi);
  }
  //  else{
  //    cout << str << " not valid.\n";
  //  }
  return valid;
}

bool Parser::parsePlayer(const string &str, vector< boost::shared_ptr<VisionObject> >& visionObjs){
  double headAnglePan = bodyModel->getJointAngle(EFF_H1);
  double headAngleTilt = bodyModel->getJointAngle(EFF_H2);
  VecPosition headAnglePolarCorrection = VecPosition(0, headAnglePan, headAngleTilt);
  
  bool valid = false;

  string playerTeamName;
  bool playerTeamNameValid = false;

  int uNum;
  bool uNumValid = false;

  double x = 0, y = 0, z = 0;
  bool coordinatesValid = false;

  vector<string> tokens = tokenise(str);
  int numBodyParts = 0;
  bool notMe = false;
  for(int i = 0; i < tokens.size(); ++i){

    if(!(tokens[i].compare("P"))){
      //Nothing
    }
    else if(!(tokens[i].compare("rlowerarm") && tokens[i].compare("llowerarm") && 
	      tokens[i].compare("rfoot") && tokens[i].compare("lfoot") && 
	      tokens[i].compare("head"))) {
      numBodyParts++;
    }
    else if(!(tokens[i].compare("team")) && i < tokens.size() - 1){

      playerTeamName = tokens[i + 1];
      ++i;
      playerTeamNameValid = true;
    }
    else if(!(tokens[i].compare("id")) && i < tokens.size() - 1){
      uNum = atoi(tokens[i + 1].c_str());
      ++i;
      uNumValid = true;
    }
    else if(!(tokens[i].compare("pol")) &&  i < tokens.size() - 3){
      double r = atof(tokens[i + 1].c_str());
      double theta = atof(tokens[i + 2].c_str());
      double phi = atof(tokens[i + 3].c_str());
      i += 3;
      if (playerTeamName.compare(teamName) || worldModel->getUNum() != uNum) {
	VecPosition posCartesian = (VecPosition(r, theta, z)+headAnglePolarCorrection).getCartesianFromPolar();
	x += posCartesian.getX();
	y += posCartesian.getY();
	z += posCartesian.getZ();
	notMe = true;
      }
      coordinatesValid = true;
    }
    else{
      cout << "Unknown token " << tokens[i] << "\n";
    }
    
  }

  valid = playerTeamNameValid && uNumValid && coordinatesValid;
  // TODO: we assume that players unums will always be between 1 and 11.
  if( uNum < 1 || uNum > 11 ) {
    cout << "We only support players numbers between 1 and 11\n";
    valid = false;
  }
  
  if(valid && notMe){
    if (numBodyParts == 0) {
      // Preserving backwards compatibility with 2008 sim
      // Can be removed when backwards compatibility no longer an issue
      numBodyParts = 1;
    }
    x /= numBodyParts;
    y /= numBodyParts;
    z /= numBodyParts;

	VecPosition avgPos = VecPosition(x,y,z);
	double r = avgPos.getMagnitude();
	double theta = avgPos.getTheta();
	double phi = avgPos.getPhi();

    //cout << "P, " << playerTeamName << ", " << uNum << ": " << r << ", " << theta << ", " << phi << "\n";
    if(!playerTeamName.compare(teamName)){
      int playerIndex = ( WO_TEAMMATE1 - 1 ) + uNum; 
      shared_ptr<VisionObject> obj( new VisionObject(r, theta, phi, playerIndex ) ); 
      visionObjs.push_back( obj ); 
//      visionSense->teammatePolar.push_back(VecPosition(r, theta, phi));
    }
    else{
      int playerIndex = ( WO_OPPONENT1 - 1 ) + uNum; 
      shared_ptr<VisionObject> obj( new VisionObject(r, theta, phi, playerIndex ) ); 
      visionObjs.push_back( obj ); 
//      visionSense->opponentPolar.push_back(VecPosition(r, theta, phi));
    }
  }


  //  else{
  //    cout << "Player String" << str << "not valid.\n";
  //  }


  return valid;
}

bool Parser::parseFRP(const string &str){

  bool valid;

  string name;
  double centreX, centreY, centreZ;
  double forceX, forceY, forceZ;

  bool validName = false;
  bool validCentre = false;
  bool validForce = false;

  int FRPIndex = -1;

  vector<string> tokens = tokenise(str);
  for(int i = 0; i < tokens.size(); ++i){

    if(!tokens[i].compare("n")){
      if(i + 1 < tokens.size()){
	
	name = tokens[i + 1];

	if(!(name.compare("lf"))){
	  FRPIndex = LEG_LEFT;
	  validName = true;
	}
	if(!(name.compare("rf"))){
	  FRPIndex = LEG_RIGHT;
	  validName = true;
	}
      }
    }

    if(!tokens[i].compare("c")){
      if(i + 3 < tokens.size()){
	
	centreX = atof(tokens[i + 1].c_str());
	centreY = atof(tokens[i + 2].c_str());
	centreZ = atof(tokens[i + 3].c_str());
	validCentre = true;
      }
    }

    if(!tokens[i].compare("f")){
      if(i + 3 < tokens.size()){
	
	forceX = atof(tokens[i + 1].c_str());
	forceY = atof(tokens[i + 2].c_str());
	forceZ = atof(tokens[i + 3].c_str());
	validForce = true;
      }
    }

  }

  valid = validName && validCentre && validForce;

  if(valid){

//        cout << "FRP: " << name << " (c: " << centreX << " " << centreY << " " << centreZ << ") (f: " << forceX << " " << forceY << " " << forceZ << ")\n";

    bodyModel->setFRP(FRPIndex, VecPosition(centreX, centreY, centreZ), VecPosition(forceX, forceY, forceZ));
  }
    else{
      cout << str << " not valid.\n";
    }

  return valid;
}

bool Parser::parseMyPos(const string &str){
  bool valid = false;
  vector<string> tokens = tokenise(str);
  
  if(!tokens[0].compare("mypos") && tokens.size() == 4){
    double x = atof(tokens[1].c_str());
    double y = atof(tokens[2].c_str());
    double z = atof(tokens[3].c_str());
    worldModel->setMyPositionGroundTruth(VecPosition(x, y, z));
    valid = true;
  }
  
  return valid;
}

vector<string> Parser::segment(const string &str, const bool &omitEnds){

  //cout << "Segmenting " << str << "\n";

  vector<string> v;

  int ptr = 0;
  int length = str.length();

  if(omitEnds){
    ptr = 1;
    length = str.length() - 1;
  }

  int bracCount = 0;

  string currentString = "";

  do{
    
    while(ptr < length && str.at(ptr) != '('){
      ptr++;
    }

    if(ptr < length){

      currentString = "";

      do{
        char c = str.at(ptr);

        if(c == '('){
          bracCount++;
        }
        else if(c == ')'){
          bracCount--;
        }

        currentString.append(1, c);
        ptr++;

      }while(bracCount != 0 && ptr < length);

      if(bracCount == 0){
        v.push_back(currentString);
        currentString = "";
      }
    }
  }while(ptr < length);

  return v;
}

bool Parser::parse(const string &input){
  bool valid = true;
  bodyModel->setFRP(LEG_LEFT, VecPosition(0, 0, 0), VecPosition(0, 0, 0));
  bodyModel->setFRP(LEG_RIGHT, VecPosition(0, 0, 0), VecPosition(0, 0, 0));
  vector<string> inputSegments = segment(input, false);

  //cout << input << endl;

  for(int i = 0; i < inputSegments.size(); ++i){
    //Time
    if(inputSegments[i].at(1)== 't'){
      valid = parseTime(inputSegments[i]) && valid;
    }
    else if(inputSegments[i].at(1) == 'G'){
      //GameState
      if(inputSegments[i].at(2) == 'S'){
        valid = parseGameState(inputSegments[i]) && valid;
      }
      //Gyro
      else{
	valid = parseGyro(inputSegments[i]) && valid;
      }
    }
    //Hear
    else if(inputSegments[i].at(1) == 'h'){
      //shivaram valid = parseHear(inputSegments[i]) && valid;
    }
    //Hinge Joint
    else if(inputSegments[i].at(1) == 'H'){
      ///shivaram
      //      valid = parseHingeJoint(inputSegments[i]) && valid;

      //    if(worldModel->getCycle() % 2 == 0){
      valid = parseHingeJoint(inputSegments[i]) && valid;
      //      }


    }
    //See
    else if(inputSegments[i].at(1) == 'S'){

      //shivaram

      /*
         valid = parseSee(inputSegments[i]) && valid;

         if(valid){
         processVision();
         }
         */

      //  if(worldModel->getCycle() % 2 == 0){
      vector< shared_ptr<VisionObject> > visionObjs;
      valid = parseSee(inputSegments[i], visionObjs) && valid;

      if(valid){
        processVision(visionObjs);
      }
      //      }


    }
    //FRP
    else if(inputSegments[i].at(1) == 'F'){
      valid = parseFRP(inputSegments[i]) && valid;
    }
    //Accelerometer
    else if(inputSegments[i].at(1) == 'A') {
      valid = parseAccelerometer(inputSegments[i]) && valid;
    }
    else{
      valid = false;
    }

  }
  return valid;
}

void Parser::processVision(vector< shared_ptr<VisionObject> >& visionObjs){
  
  // Print vision info
  vector< shared_ptr<VisionObject> >::iterator begin( visionObjs.begin() );
  vector< shared_ptr<VisionObject> >::iterator end( visionObjs.end() );
  for (; begin != end; ++begin ) {
    const VisionObject& obj = **begin;
  //  cout << "Printing vision object:\n";
  //  cout << "index=" << WorldObjType2Str[obj.id] << " r=" << obj.polar.getX() << " theta=" 
  //       << obj.polar.getY() 
  //       << " phi=" << obj.polar.getZ() << endl;
  }

  // Fix vision objects w.r.t. head pan and tilt
  double headAnglePan = bodyModel->getJointAngle(EFF_H1);
  double headAngleTilt = bodyModel->getJointAngle(EFF_H2);
  VecPosition headAnglePolarCorrection = VecPosition(0, headAnglePan, headAngleTilt);
  begin = visionObjs.begin();
  end = visionObjs.end();
  for( ; begin != end; ++begin ) {
    VisionObject& obj = **begin;
    obj.polar += headAnglePolarCorrection;
//    cout << "Printing vision object AFTER HEAD CORRECTION:\n";
//    cout << "index=" << WorldObjType2Str[obj.id] << " r=" << obj.polar.getX() << " theta=" 
//         << obj.polar.getY() 
//         << " phi=" << obj.polar.getZ() << endl;
  }

  // update the "seen" information to all world objects
  for( int i = WO_BALL; i < NUM_WORLD_OBJS; ++i ) {
    worldModel->getWorldObject( i )->currentlySeen = false;
  }
  begin = visionObjs.begin();
  end = visionObjs.end();
  for( ; begin != end; ++begin ) {
    VisionObject& visObj = **begin;
    WorldObject* pObj = worldModel->getWorldObject( visObj.id );
    pObj->currentlySeen = true;
    pObj->cycleLastSeen = worldModel->getTime();
    pObj->timeLastSeen = worldModel->getCycle();
    pObj->vision = visObj;
  } 

#ifdef PERFECT_VISION
  map< int, VecPosition> flagPos;
  vector<VecPosition> flagLocal;
  vector< shared_ptr<VisionObject> >::iterator b( visionObjs.begin() );
  vector< shared_ptr<VisionObject> >::iterator e( visionObjs.end() );
  for (; b != e; ++b ) {
    const VisionObject& obj = **b;
    bool isFlag = ( FLAG_1_L  <=  obj.id   && 
                    obj.id    <=  FLAG_2_R );
    if( isFlag ) {
      int flagnum = obj.id - FLAG_1_L;      
      flagPos[flagnum] = obj.polar;
    }
  }

  for(int i = 0; i <= FLAG_2_R - FLAG_1_L; ++i){
    flagLocal.push_back(flagPos[i].getCartesianFromPolar());
  }

  //double fieldWidth = flagLocal[FLAG_1_L].getDistanceTo(flagLocal[FLAG_2_L]);
  //double fieldLength = flagLocal[FLAG_1_L].getDistanceTo(flagLocal[FLAG_1_R]);
  
  //worldModel->setFieldLength(FIELD_LENGTH);
  //worldModel->setFieldWidth(FIELD_WIDTH);

  // Currently we're not setting/using this value
  //worldModel->setLocalized(false);

  VecPosition fieldXPlusYPlus, fieldXPlusYMinus, fieldXMinusYPlus, fieldXMinusYMinus;
  if(side == SIDE_LEFT){
    fieldXPlusYPlus = flagLocal[FLAG_1_R - FLAG_1_L];
    fieldXPlusYMinus = flagLocal[FLAG_2_R - FLAG_1_L];
    fieldXMinusYPlus = flagLocal[FLAG_1_L - FLAG_1_L];
    fieldXMinusYMinus = flagLocal[FLAG_2_L - FLAG_1_L];
    /*
    cout << "FLAG_1_R = " << worldModel->l2g(fieldXPlusYPlus) << endl;
    cout << "FLAG_2_R = " << worldModel->l2g(fieldXPlusYMinus) << endl;
    cout << "FLAG_1_L " << worldModel->l2g(fieldXMinusYPlus) << endl;
    cout << "FLAG_2_L " << worldModel->l2g(fieldXMinusYMinus) << endl;
    */
  }
  else{//side == SIDE_RIGHT
    fieldXPlusYPlus = flagLocal[FLAG_2_L - FLAG_1_L];
    fieldXPlusYMinus = flagLocal[FLAG_1_L - FLAG_1_L];
    fieldXMinusYPlus = flagLocal[FLAG_2_R - FLAG_1_L];
    fieldXMinusYMinus = flagLocal[FLAG_1_R - FLAG_1_L];
    /*
    cout << "FLAG_2_L = " << worldModel->l2g(fieldXPlusYPlus) << endl;
    cout << "FLAG_1_L = " << worldModel->l2g(fieldXPlusYMinus) << endl;
    cout << "FLAG_2_R " << worldModel->l2g(fieldXMinusYPlus) << endl;
    cout << "FLAG_1_R " << worldModel->l2g(fieldXMinusYMinus) << endl;
    */
  }

  worldModel->updateMatricesAndMovingObjs( fieldXPlusYPlus,
                                           fieldXPlusYMinus,
                                           fieldXMinusYPlus, 
                                           fieldXMinusYMinus );
 
#endif

#ifdef RESTRICTED_VISION
// NOTE: choose between one of the following lines
  SimpleLocalization::getLocalization(worldModel, bodyModel);
  //particleFilter->processFrame(); 
  //cout << "Robot estimated pose: \n" 
  //     << "pos: " << worldModel->getMyPosition() 
  //     << "ang: " << worldModel->getMyAngDeg() << endl;
  //cout << "Ground truth pos: " << worldModel->getMyPositionGroundTruth() << endl;
// update world matrices and moving objects
VecPosition fieldXPlusYPlus;
VecPosition fieldXPlusYMinus;
VecPosition fieldXMinusYPlus;
VecPosition fieldXMinusYMinus;
computeLocalCornerPositions( fieldXPlusYPlus, fieldXPlusYMinus, 
                             fieldXMinusYPlus, fieldXMinusYMinus );

worldModel->updateMatricesAndMovingObjs( fieldXPlusYPlus, fieldXPlusYMinus,
                                        fieldXMinusYPlus, fieldXMinusYMinus );
#endif

}


void Parser::
computeLocalCornerPositions( VecPosition& fieldXPlusYPlus,
                             VecPosition& fieldXPlusYMinus,
                             VecPosition& fieldXMinusYPlus, 
                             VecPosition& fieldXMinusYMinus ) {
  
  // TODO: currently we assume that z is constant
  VecPosition myPos = worldModel->getMyPosition();
  const double myX = myPos.getX();
  const double myY = myPos.getY();
  const double myZ = 0.5;
  const double myOrient = worldModel->getMyAngDeg();
  
  VecPosition globalPlusPlus, globalPlusMinus, globalMinusPlus, globalMinusMinus;
  if (worldModel->getSide() == SIDE_LEFT ) {
    globalPlusPlus = worldModel->getWorldObject( FLAG_1_R )->pos;
    globalPlusMinus = worldModel->getWorldObject(FLAG_2_R)->pos;
    globalMinusPlus = worldModel->getWorldObject(FLAG_1_L)->pos;
    globalMinusMinus = worldModel->getWorldObject(FLAG_2_L)->pos;
  } else { // SIDE_RIGHT
    globalPlusPlus = worldModel->getWorldObject(FLAG_2_L)->pos;
    globalPlusMinus = worldModel->getWorldObject(FLAG_1_L)->pos;
    globalMinusPlus = worldModel->getWorldObject(FLAG_2_R)->pos;
    globalMinusMinus = worldModel->getWorldObject(FLAG_1_R)->pos;
  }


  // Compute local coordinates of corners
  fieldXPlusYPlus.setVecPosition( globalPlusPlus.getX() - myX,
                               globalPlusPlus.getY() - myY,
                               0 - myZ );
  fieldXPlusYPlus = fieldXPlusYPlus.rotateAboutZ( myOrient );                          

  fieldXPlusYMinus.setVecPosition(  globalPlusMinus.getX() - myX,
                                 globalPlusMinus.getY() - myY,
                                 0 - myZ );
  fieldXPlusYMinus = fieldXPlusYMinus.rotateAboutZ( myOrient );                               

  fieldXMinusYPlus.setVecPosition(  globalMinusPlus.getX() - myX,
                                 globalMinusPlus.getY() - myY,
                                 0 - myZ );
  fieldXMinusYPlus = fieldXMinusYPlus.rotateAboutZ( myOrient );


  fieldXMinusYMinus.setVecPosition( globalMinusMinus.getX() - myX,
                                 globalMinusMinus.getY() - myY,
                                  0 - myZ );
  fieldXMinusYMinus = fieldXMinusYMinus.rotateAboutZ( myOrient );
}
