#include "worldmodel.h"
#include "fstream.h"

extern map<string,float> namedParams;
extern string outputFile;

WorldModel::WorldModel(){
    written = 0;
        
    cycle = 0;
    time = -1.0;
    gameTime = -1.0;
    playMode = PM_BEFORE_KICK_OFF;

    uNum = 1;//dummy
    uNumSet = false;

    side = SIDE_LEFT;//dummy
    sideSet = false;

    lastSkill = SKILL_STAND;//dummy

	for (int i = 0; i < NUM_WORLD_OBJS; ++i) {
      WorldObject& obj = worldObjects[i];
      obj.id = i;
    }
	
//    for(int i = 0; i < GOALPOST_NUM; ++i){
//      goalPost.push_back(VecPosition(0, 0, 0));
//    }
//
//    ball = NULL;
//    
//    for(int i = 0; i < TEAMMATE_NUM; ++i){
//      teammate.push_back(VecPosition(0, 0, 0));
//    }
//
//    for(int i = 0; i < OPPONENT_NUM; ++i){
//      opponent.push_back(VecPosition(0, 0, 0));
//    }
//

    numTeammates = 0;
    numOpponents = 0;

    //In general, will be inferred from flags.
    //fieldLength = 12.0;
    //fieldWidth = 8.0;

    fLocalized = false;
    // TODO: this is not correct, but anyway localization should update it
    // Is there a better solution for init them?
    myPosition.setVecPosition( 0, 0, 0 );
    myAngDegrees = 0;

    myPositionGroundTruth.setVecPosition(0, 0, 0);
    
    localToGlobal = HCTMatrix();
    globalToLocal = HCTMatrix();
}

WorldModel::~WorldModel(){
}

void WorldModel::display(){

    cout << "****************World Model******************************\n";
    std::cout << "Cycle: " << cycle << "\n";
    std::cout << "Time: " << time << "\n";
    std::cout << "Game Time: " << gameTime << "\n";
    std::cout << "Play Mode: " << playMode << "\n";

    std::cout << "uNumSet: " << uNumSet << ", uNum: " << uNum << "\n";
    std::cout << "sideSet: " << sideSet << ", side: " << side << "\n";

    for (int i = WO_BALL; i < NUM_WORLD_OBJS; ++i) {
      WorldObject& obj = worldObjects[i];
      cout << "World Object: " << WorldObjType2Str[i] << ":\n" 
           << obj.pos << '\n'
           << " currently seen: " << ( obj.currentlySeen ? "YES" : "NO" ) << '\n' 
           << " cycle last seen: " << obj.cycleLastSeen 
           << " time last seen: " << obj.timeLastSeen << endl;
    }

//    cout << "GoalPost_1_l: " << goalPost[GOALPOST_1_L] << "\n";
//    cout << "GoalPost_1_r: " << goalPost[GOALPOST_1_R] << "\n";
//    cout << "GoalPost_2_l: " << goalPost[GOALPOST_2_L] << "\n";
//    cout << "GoalPost_2_r: " << goalPost[GOALPOST_2_R] << "\n";
//
//    cout << "Ball: " << ball << "\n";
//
//    cout << "Teammates:\n";
//    for(int i = 0; i < numTeammates; ++i){
//        cout << i << ": " << teammate[i] << "\n";
//    }
//
//    cout << "Opponents:\n";
//    for(int i = 0; i < numOpponents; ++i){
//        cout << i << ": " << opponent[i] << "\n";
//    }

//    cout << "Field Length: " << fieldLength << "\n";
//    cout << "Field Width: " << fieldWidth << "\n";

    cout << "*********************************************************\n";
}

void WorldModel::updateGoalPostsAndFlags() {

  VecPosition fieldXPlusYPlus, fieldXPlusYMinus, fieldXMinusYPlus, fieldXMinusYMinus;
  if(side == SIDE_LEFT){
    // flags
    worldObjects[FLAG_1_R].pos.setVecPosition( HALF_FIELD_X,
                                               HALF_FIELD_Y,
                                               0 );
    worldObjects[FLAG_2_R].pos.setVecPosition(  HALF_FIELD_X,
                                               -HALF_FIELD_Y,
                                                0 );
    worldObjects[FLAG_1_L].pos.setVecPosition( -HALF_FIELD_X,
                                                HALF_FIELD_Y,
                                                0 );
    worldObjects[FLAG_2_L].pos.setVecPosition( -HALF_FIELD_X,
                                               -HALF_FIELD_Y,
                                                0 );
    // goalposts                                                
    worldObjects[GOALPOST_1_R].pos.setVecPosition( HALF_FIELD_X,
                                                   HALF_GOAL_Y,
                                                   GOAL_Z);
    worldObjects[GOALPOST_2_R].pos.setVecPosition(  HALF_FIELD_X,
                                                   -HALF_GOAL_Y,
                                                    GOAL_Z);
    worldObjects[GOALPOST_1_L].pos.setVecPosition( -HALF_FIELD_X,
                                                    HALF_GOAL_Y,
                                                    GOAL_Z);
    worldObjects[GOALPOST_2_L].pos.setVecPosition( -HALF_FIELD_X,
                                                   -HALF_GOAL_Y,
                                                    GOAL_Z);

  }
  else{//side == SIDE_RIGHT
    // flags
    worldObjects[FLAG_1_R].pos.setVecPosition( -HALF_FIELD_X,
                                               -HALF_FIELD_Y,
                                                0 );
    worldObjects[FLAG_2_R].pos.setVecPosition( -HALF_FIELD_X,
                                                HALF_FIELD_Y,
                                                0 );
    worldObjects[FLAG_1_L].pos.setVecPosition(  HALF_FIELD_X,
                                               -HALF_FIELD_Y,
                                                0 );
    worldObjects[FLAG_2_L].pos.setVecPosition( HALF_FIELD_X,
                                               HALF_FIELD_Y,
                                                0 );
    // goal posts
    worldObjects[GOALPOST_1_R].pos.setVecPosition( -HALF_FIELD_X,
                                                   -HALF_GOAL_Y,
                                                    GOAL_Z );
    worldObjects[GOALPOST_2_R].pos.setVecPosition( -HALF_FIELD_X,
                                                    HALF_GOAL_Y,
                                                    GOAL_Z);
    worldObjects[GOALPOST_1_L].pos.setVecPosition(  HALF_FIELD_X,
                                                   -HALF_GOAL_Y,
                                                    GOAL_Z );
    worldObjects[GOALPOST_2_L].pos.setVecPosition( HALF_FIELD_X,
                                                   HALF_GOAL_Y,
                                                   GOAL_Z );
  }

}

// A functionality that was previously in parser.cc (in perfect vision mode)
// Updates the l2g and g2l matrices based on 4 points of the field
void WorldModel::
updateMatricesAndMovingObjs( VecPosition& fieldXPlusYPlus,
                          VecPosition& fieldXPlusYMinus,
                          VecPosition& fieldXMinusYPlus, 
                          VecPosition& fieldXMinusYMinus ) {
  
  VecPosition localFieldCentre = (fieldXPlusYPlus + fieldXMinusYMinus) * 0.5;
  VecPosition localFieldXDirection = (fieldXPlusYPlus - fieldXMinusYPlus).normalize();
  VecPosition localFieldYDirection = (fieldXPlusYPlus - fieldXPlusYMinus).normalize();
  VecPosition localFieldZDirection = (localFieldXDirection.crossProduct(localFieldYDirection)).normalize();
  
  setLocalToGlobal(0, 0, localFieldXDirection.getX());
  setLocalToGlobal(0, 1, localFieldXDirection.getY());
  setLocalToGlobal(0, 2, localFieldXDirection.getZ());
  setLocalToGlobal(0, 3, -(localFieldCentre.dotProduct(localFieldXDirection)));
  
  setLocalToGlobal(1, 0, localFieldYDirection.getX());
  setLocalToGlobal(1, 1, localFieldYDirection.getY());
  setLocalToGlobal(1, 2, localFieldYDirection.getZ());
  setLocalToGlobal(1, 3, -(localFieldCentre.dotProduct(localFieldYDirection)));
  
  setLocalToGlobal(2, 0, localFieldZDirection.getX());
  setLocalToGlobal(2, 1, localFieldZDirection.getY());
  setLocalToGlobal(2, 2, localFieldZDirection.getZ());
  setLocalToGlobal(2, 3, -(localFieldCentre.dotProduct(localFieldZDirection)));
  
  setLocalToGlobal(3, 0, 0);
  setLocalToGlobal(3, 1, 0);
  setLocalToGlobal(3, 2, 0);
  setLocalToGlobal(3, 3, 1.0);
       

  VecPosition localBodyCentre = VecPosition(0, 0, 0);
  VecPosition localBodyXDirection = VecPosition(1.0, 0, 0);
  VecPosition localBodyYDirection = VecPosition(0, 1.0, 0);
  VecPosition localBodyZDirection = VecPosition(0, 0, 1.0);
  
  VecPosition bodyCentre = l2g(localBodyCentre);
  VecPosition bodyXDirection = (l2g(localBodyXDirection) - bodyCentre).normalize();
  VecPosition bodyYDirection = (l2g(localBodyYDirection) - bodyCentre).normalize();
  VecPosition bodyZDirection = (l2g(localBodyZDirection) - bodyCentre).normalize();
  
  setGlobalToLocal(0, 0, bodyXDirection.getX());
  setGlobalToLocal(0, 1, bodyXDirection.getY());
  setGlobalToLocal(0, 2, bodyXDirection.getZ());
  setGlobalToLocal(0, 3, -(bodyCentre.dotProduct(bodyXDirection)));
  
  setGlobalToLocal(1, 0, bodyYDirection.getX());
  setGlobalToLocal(1, 1, bodyYDirection.getY());
  setGlobalToLocal(1, 2, bodyYDirection.getZ());
  setGlobalToLocal(1, 3, -(bodyCentre.dotProduct(bodyYDirection)));
  
  setGlobalToLocal(2, 0, bodyZDirection.getX());
  setGlobalToLocal(2, 1, bodyZDirection.getY());
  setGlobalToLocal(2, 2, bodyZDirection.getZ());
  setGlobalToLocal(2, 3, -(bodyCentre.dotProduct(bodyZDirection)));
  
  setGlobalToLocal(3, 0, 0);
  setGlobalToLocal(3, 1, 0);
  setGlobalToLocal(3, 2, 0);
  setGlobalToLocal(3, 3, 1.0);

//  cerr << "Update coordinates for moving objects " << endl;
  // Set coordinates for moving objects
  for( int i = WO_BALL; i <= WO_OPPONENT11; ++i ) {

    WorldObject* pObj = getWorldObject(i);

    if( pObj->currentlySeen ) {

      VecPosition objLocal = pObj->vision.polar.getCartesianFromPolar();
      VecPosition objGlobal = l2g( objLocal );
 //     cerr << "Object " << WorldObjType2Str[pObj->id] 
 //          << " is seen in pos:" << objGlobal << endl;

      if( pObj->id == WO_BALL ) {

        setBall( objGlobal );
	//cerr << "Object " << WorldObjType2Str[pObj->id] 
	//<< " is seen in pos:" << objGlobal << endl;

      } else if( WO_OPPONENT1 <= pObj->id && pObj->id <= WO_OPPONENT11 ) {

        setOpponent( pObj->id, objGlobal ); 

      } else if( WO_TEAMMATE1 <= pObj->id && pObj->id <= WO_TEAMMATE11 ) {

        setTeammate( pObj->id, objGlobal ); 

      }

    } else {

      if ( pObj->id == WO_BALL ) {
//        cerr << "Object " << WorldObjType2Str[pObj->id] << " is not seen" << endl;
      }
    }
  }

}


/* 
 * Writes the value of the objective function to the file specified on
 * the command line with the --experimentout option.
 */
void WorldModel::quickDisplay() {
  if (gameTime > 0.8 && gameTime < 1.0) {
    at1s = getMyPositionGroundTruth();
  }
  
  if (gameTime > 0.5 && gameTime < 31.0) {
    cerr << "I am at: " << getMyPositionGroundTruth() << endl;
  }

  if (gameTime > 31.0 && !written) { 
    VecPosition me = getMyPositionGroundTruth(); 
    VecPosition start = at1s; 
    double walkdist = start.getDistanceTo(me);
    fstream file;
    file.open(outputFile.c_str(), ios::out );
    file <<  walkdist << endl;
    file.close();
    cerr << "At 1s, I was at: " << at1s << endl;
    cerr << "I am at: " << me << endl; 
    cerr << "Distance walked: " << walkdist << endl;
    written = 1;
  }
}
