#ifndef WORLDMODEL_H
#define WORLDMODEL_H

#include "headers.h"
#include "Geometry.h"
#include "hctmatrix.h"
#include "WorldObject.h"
#include "Field.h"

#include <vector>

using namespace std;

//Field Constants
//#define FIELD_WIDTH  8
//#define FIELD_LENGTH 12

class WorldModel{

 private:
  
  unsigned long cycle;
  double time;
  double gameTime;
  int playMode;
  
  int uNum;
  int side;
  
  bool uNumSet;
  bool sideSet;
  
  int written;
  VecPosition at1s;

  WorldObject worldObjects[NUM_WORLD_OBJS];
  VecPosition myPosition;
  AngDeg myAngDegrees;
  VecPosition myPositionGroundTruth;

  int lastSkill;
//  std::vector<VecPosition> goalPost;
//  VecPosition ball;
//  std::vector<VecPosition> teammate;
//  std::vector<VecPosition> opponent;

  int numTeammates, numOpponents;
  
  //double fieldLength, fieldWidth;
  
  bool fLocalized;

  HCTMatrix localToGlobal, globalToLocal;

 public:
  
  WorldModel();
  ~WorldModel();
  
  inline WorldObject* getWorldObject( int index ) { return &worldObjects[index]; }
  void updateGoalPostsAndFlags();
  void updateMatricesAndMovingObjs( VecPosition& fieldXPlusYPlus,
                                 VecPosition& fieldXPlusYMinus,
                                 VecPosition& fieldXMinusYPlus, 
                                 VecPosition& fieldXMinusYMinus );

  inline void setMyPosition( const VecPosition& newPos ) { myPosition = newPos; }
  inline void setMyPosition( const Point2D& newPos ) { 
    myPosition.setX( newPos.getX() ); 
    myPosition.setY( newPos.getY() );
    // Z is not changed, stays at the default
  }
  inline VecPosition getMyPosition() { return myPosition; }

  inline void setMyAngleDeg( AngDeg newAng ) { myAngDegrees = newAng; }
  inline void setMyAngRad( AngRad newAng ) { myAngDegrees = Rad2Deg( newAng ); }
  inline AngDeg getMyAngDeg() { return myAngDegrees; }
  inline AngRad getMyAngRad() { return Deg2Rad( myAngDegrees ); }
  
  inline void setMyPositionGroundTruth( const VecPosition& newPos ) { myPositionGroundTruth = newPos; }
  inline VecPosition getMyPositionGroundTruth() { return myPositionGroundTruth; }

  inline unsigned long getCycle(){ return cycle; }
  inline void incrementCycle(){ cycle++;}
  inline void setCycle(const unsigned long &cycle){ this->cycle = cycle; }
  
  inline double getTime(){ return time; }
  inline void setTime(const double &time){ this->time = time; }

  inline double getGameTime(){ return gameTime; }
  inline void setGameTime(const double &gameTime){ this->gameTime = gameTime; }
  
  inline int getPlayMode(){ return playMode; }
  inline void setPlayMode(const int &playMode){ this->playMode = playMode; }
  
  inline int getUNum(){ return uNum; }
  inline void setUNum(const int &uNum){ this->uNum = uNum; }
  
  inline bool getUNumSet(){ return uNumSet; }
  inline void setUNumSet(const bool &uNumSet){ this->uNumSet = uNumSet; }

  inline int getLastSkill(){ return lastSkill; }
  inline void setLastSkill(const int &lastSkill){ this->lastSkill = lastSkill; }

  inline int getSide(){ return side; }
  inline void setSide(const int &side){ this->side = side; updateGoalPostsAndFlags(); }
  
  inline bool getSideSet(){ return sideSet; }
  inline void setSideSet(const bool &sideSet){ this->sideSet = sideSet; }
  
  inline VecPosition getGoalPost(const int &i){ return worldObjects[GOALPOST_1_L  + i].pos; };
//  inline void setGoalPost(const int &i, const VecPosition &goalPost){ worldObjects[GOALPOST_1_L + i].pos = goalPost; }
  
  inline VecPosition getMyLeftGoalPost(){ return ((side == SIDE_LEFT)? worldObjects[GOALPOST_1_L].pos : worldObjects[GOALPOST_2_R].pos ); }
  inline VecPosition getMyRightGoalPost(){ return ((side == SIDE_LEFT)? worldObjects[GOALPOST_2_L].pos : worldObjects[GOALPOST_1_R].pos); }
  inline VecPosition getOppLeftGoalPost(){ return ((side == SIDE_LEFT)? worldObjects[GOALPOST_1_R].pos : worldObjects[GOALPOST_2_L].pos); }
  inline VecPosition getOppRightGoalPost(){ return ((side == SIDE_LEFT)? worldObjects[GOALPOST_2_R].pos : worldObjects[GOALPOST_1_L].pos ); }

  inline double distanceToOppGoal(VecPosition &p){
    
    VecPosition oppLeftGoalPost = getOppLeftGoalPost();
    VecPosition oppRightGoalPost = getOppRightGoalPost();

    if(p.getY() > oppLeftGoalPost.getY()){
      return p.getDistanceTo(oppLeftGoalPost);
    }
    else if(p.getY() < oppRightGoalPost.getY()){
      return p.getDistanceTo(oppRightGoalPost);
    }
    else{
      return fabs(oppLeftGoalPost.getX() - p.getX());
    }
  }

  inline double distanceToMyGoal(VecPosition &p){
    
    VecPosition myLeftGoalPost = getMyLeftGoalPost();
    VecPosition myRightGoalPost = getMyRightGoalPost();

    if(p.getY() > myLeftGoalPost.getY()){
      return p.getDistanceTo(myLeftGoalPost);
    }
    else if(p.getY() < myRightGoalPost.getY()){
      return p.getDistanceTo(myRightGoalPost);
    }
    else{
      return fabs(myLeftGoalPost.getX() - p.getX());
    }
  }


  inline VecPosition getBall(){ return worldObjects[WO_BALL].pos; };
  inline void setBall(const VecPosition &ball){ worldObjects[WO_BALL].pos = ball; }
 
  inline VecPosition getTeammate(const int &i){ return worldObjects[i].pos; };
  inline void setTeammate(const int &i, const VecPosition &teammate){ worldObjects[i].pos = teammate; }
  
//  inline int getNumTeammates(){ return this->numTeammates; }
//  inline void setNumTeammates(const int &num){ this->numTeammates = num; }

  inline VecPosition getOpponent(const int &i){ return worldObjects[i].pos; };
  inline void setOpponent(const int &i, const VecPosition &opponent){ worldObjects[i].pos = opponent; }
  
//  inline int getNumOpponents(){ return this->numOpponents; }
//  inline void setNumOpponents(const int &num){ this->numOpponents = num; }

  //inline double getFieldLength(){ return fieldLength; }
  //inline void setFieldLength(const double &fieldLength){ this->fieldLength = fieldLength; }
  
  //inline double getFieldWidth(){ return fieldWidth; }
  //inline void setFieldWidth(const double &fieldWidth){ this->fieldWidth = fieldWidth; }
  
  inline void setGlobalToLocal(const int &i, const int &j, const double &v){ this->globalToLocal.setCell(i, j, v); }

  inline void setLocalToGlobal(const int &i, const int &j, const double &v){ this->localToGlobal.setCell(i, j, v); }


  inline bool isLocalized() {return fLocalized; }
  inline void setLocalized(bool fLocalized) {this->fLocalized = fLocalized; }

  inline VecPosition g2l(const VecPosition &global){ return globalToLocal.transform(global); }
  inline VecPosition l2g(const VecPosition &local){ return localToGlobal.transform(local); }

  void display();
  void quickDisplay();
  void writeValue();
  double value();
};

#endif // WORLDMODEL_H

