package cma.examples;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StreamTokenizer;

import cma.*;

//Command Line Iterations : IterNo
public class CMARoboWalk {
	static PrintWriter pri;
	static String exppath = "";
	static Reader r;
	
	static String paramNames[] = {
    "wf_LL3target1",
    "wf_LL4target1",
    "wf_LL5target1",
    "wf_RL3target1",
    "wf_RL4target1",
    "wf_RL5target1",
    "wf_LL3target2",
    "wf_LL4target2",
    "wf_LL5target2",
    "wf_RL3target2",
    "wf_RL4target2",
    "wf_RL5target2",
    "wf_LA1target1"
  };     
	static double initParams[] = {
    -20.296, 
    -42.831,
     57.819,
     0,
     0,
     0,
     53.51878,
     0,
    -29.927797,
     0,
     0,
     0,
     -100
  };
	//static double initSigma[]={0.01,0.01,1,1,1,1,1,1,1,1,0.005,5.0};
	static double initSigma[]={10,10,10,10,10,10,10,10,10,10,10,10,10};

	static int noParams = paramNames.length;
	static int popSize = 0;
	static int totIter = 0;
	public static void writeParams(double pop[][],int curIter) {
		try {
			for(int i =0;i<pop.length;i++) {
				pri = new PrintWriter(new FileWriter(exppath+"params_"+curIter+"_i_"+i+".txt", true));				
				for (int j = 0; j < noParams; j++) {					
					pri.println(paramNames[j]+"\t"+pop[i][j]);
			
				}
				pri.close();
			}				
			pri = new PrintWriter(new FileWriter(exppath+"paramswritten_"+curIter+".txt", true));				
			//pri.println("\n");
			pri.close();
		}
		catch (Exception e) {
			e.printStackTrace();
			System.out.println("No such file exists.");
		}							

	
	}

	public static double[] readFitness(int curIter) {
		double fitness[] = new double[popSize];
		try {			
			for(int i =0;i<popSize;i++) {
        String expectedFile = exppath+"value_"+curIter+"_i_"+i+".txt";
        if (! new File(expectedFile).exists()) {
				  fitness[i] = 100000; // Absurdly high value, taken as +INFINITY
          System.out.println("Could not find file " + expectedFile + " so we'll assume bad fitness");
				} else { 
					r = new BufferedReader(new FileReader(expectedFile));
					StreamTokenizer stok = new StreamTokenizer(r);
					stok.parseNumbers();
					stok.nextToken();				    
					while (stok.ttype != StreamTokenizer.TT_EOF) {
						if (stok.ttype == StreamTokenizer.TT_NUMBER){
							fitness[i]=-1*stok.nval;
						}			          
						stok.nextToken();
					}
					r.close();
				}
			}
		}

		catch (Exception e) {
			e.printStackTrace();
			System.out.println("No such file exists.");
		}
		return fitness;
	}

	public static void main(String[] args) throws InterruptedException {
		exppath = args[0] + "/results/";
		totIter = Integer.parseInt(args[1]);
		popSize = Integer.parseInt(args[2]);
		System.out.println(exppath);
		System.out.println(""+totIter);
		CMAEvolutionStrategy cma = new CMAEvolutionStrategy();
		cma.readProperties(); // read options`, see file CMAEvolutionStrategy.properties
		cma.setDimension(noParams); // overwrite some loaded properties
		cma.parameters.setPopulationSize(popSize);
		cma.setInitialX(initParams);
		cma.setInitialStandardDeviations(initSigma); // also a mandatory setting
		double[] fitness = cma.init();
		for (int curIter = 1; curIter <= totIter ; curIter++) {
			double[][] pop = cma.samplePopulation(); // get a new population of solutions	    	
			writeParams(pop,curIter);
			if (curIter == totIter) { 
			  System.out.println("CMA-ES done.  The last iteration should be evaluating now.");
			  break; 
			}
			String expectedFile = exppath + "valuationdone_" + curIter + ".txt";

			File f = new File(expectedFile);
			while(true) {
				//wait
				if(f.exists()) {
					System.out.println("Found " + expectedFile + "; Generating next parameter set\n");
					break;
				} else {
					Thread.sleep(5000);
					System.out.println("Waiting on " + expectedFile);
				}
			}
			//Read fitness function
			fitness = readFitness(curIter);
			cma.updateDistribution(fitness);        // pass fitness array to update search distribution
		}
	} // main  
} // class
