# Policy gradient search parameter generator for Robocup 3D-Sim
# kick optimization.
#
# Suyog Jain
# 2009-12-01

import sys
import random
import cPickle

#basedir --> basedir of our results
#curiter --> the current iteration
#numpolicies --> No of policies we want to generate 
basedir = sys.argv[1] + '/'
curiter=int(sys.argv[2])
numpolicies= int(sys.argv[3])

#ej --> Small perturbations to be done on every parameter

ej = {
  'agent2beamX':0.001, # keep almost constant for now
  'agent2beamY':0.001, 
  'agent2beamAngle':1,     
  'returnKickScale':0.05,     
  'forwardKickScaleLL1':0.05, 
  'forwardKickScaleLL2':0.05,  
  'forwardKickScaleLL3':0.05,  
  'forwardKickScaleLL4':0.05,  
  'forwardKickScaleLL5':0.05,  
  'forwardKickScaleLL6':0.05,  
  'targetHeight':0.005,         
  'angleAtKick':2.5    
}      

#map--> is used to ensure that parameters are stored in the correct order
map = {
  'agent2beamX':0,
  'agent2beamY':1, 
  'agent2beamAngle':2,     
  'returnKickScale':3,     
  'forwardKickScaleLL1':4, 
  'forwardKickScaleLL2':5,  
  'forwardKickScaleLL3':6,  
  'forwardKickScaleLL4':7,  
  'forwardKickScaleLL5':8,  
  'forwardKickScaleLL6':9,  
  'targetHeight':10,         
  'angleAtKick':11    
}      


#Read the final policy file of previous iteration and conver it to a dictionary 'basepolicy'
basepolicyfile = open(basedir+'finalpolicy_'+str(curiter-1)+'.txt',"r")
basepolicyf = basepolicyfile.read()
temp=basepolicyf.split("\n")
basepolicyfile.close()
basepolicy={}
for i in range(len(temp)):
	basepolicy[temp[i].split('\t')[0]]=temp[i].split('\t')[1]

#define an empty list 3*noparams where each params can have any no of value depending on how many values
#changed as 0 ( row1) +ve (row 2) -ve (row 3)
clusterlist = [[[] for i in range(len(ej))] for j in range(3)]

#Does a random perturbation on the policy files.
#Start with a copy of policy, then changes each parameter randomly
#Stores the changes in cluster list
#Writes the randompolicy file
for i in range(numpolicies):
	params = basepolicy.copy()
	par=""
	for k,v in params.items():
		rd = (random.sample([0,1,2],1))[0]
		if rd == 0:
			clusterlist[0][map[k]].append(i)
			params[k]=float(v)
		elif rd == 1:
			clusterlist[1][map[k]].append(i)
			params[k]=float(v)+ej[k]
		else:
			clusterlist[2][map[k]].append(i)
			params[k]=float(v)-ej[k]
		par = (par + "".join(["%s\t%s\n" % (k,params[k])]))
	
	file=open(basedir+'params_'+str(curiter)+'_i_'+str(i)+'.txt',"w")
	file.write(par)
	file.close()
		
#Writes the changes list
file = open(basedir+'randompolicy_cluster_'+str(curiter)+'.txt',"w")
cPickle.dump(clusterlist,file)
file.close()
